#
# Bank accounting.
#
init python:

    #
    # Basic bank account.
    #
    class Account:
        balanceM = 0                        # Current balance
        balanceOrigM = 0                    # Original balance
        overdraftLimM = 0                   # Overdraft limit
        daysOverM = 0                       # Days overdrawn

        foodDebtM = 0                       # Food expenditure since last shop
        foodDebtAllM = 0                    # All spending on food
        foodLimM = False                    # Is food limited

        # Add a food debt to be replenished with food shopping.
        #
        def addFoodCost(self, money):
            self.foodDebtM += money
            self.foodDebtAllM += money
            if debug:
                dbg("addFoodCost({:0.3f})={:0.3f}".format(money, self.foodDebtM))

        # Pay food shopping bill.
        # Offsets food debt from balance.
        # If the debt is not paid off foodLim is set.
        #
        def payFood(self, money):
            self.foodDebtM -= money
            self.foodLimM = self.foodDebtM > 0
            self.balanceM -= money
            return self.foodLimM

        # Can a cost be afforded?
        #
        def canAfford(self, money):
            return self.balanceM + self.overdraftLimM > money

        # Earn some money.
        #
        def earn(self, money):
            self.balanceM += money

        # Get the current balance.
        #
        def getBalance(self):
            bal = math.floor(self.balanceM * 100.0) / 100.0
            return bal

        # Get the balance as a proprotion of the target.
        #
        def getBalancePer(self, tgt=None):
            if tgt == None:
                tgt = self.balanceOrigM
            return int(round(100 * self.balanceM / tgt))

        # Get the current food bill.
        # Round up to penny.
        #
        def getFoodBill(self):
            return math.ceil(self.foodDebtM * 100.0) / 100.0

        # Get the sum of all food bills.
        #
        def getFoodBillAllTime(self):
            return self.foodDebtAllM

        # Get the overdraft limit.
        #
        def getOverdraftLim(self):
            return self.overdraftLimM

        # Get the number of days the account has been beyond limits.
        #
        def getOverDays(self):
            return self.daysOverM

        # Is the supply of food limited?
        #
        def isFoodLim(self):
            return self.foodLimM or self.daysOverM > 0

        # Is the account overdrawn?
        #
        def isOverdrawn(self):
            return self.balanceM < 0

        # Is the account overdrawn past the agreed limit?
        #
        def isOverLim(self):
            return self.balanceM + self.overdraftLimM < 0

        # Process end of day limits.
        #
        def newDay(self):
            if self.balanceM < 0:
                self.balanceM *= 1.0003 # About 12% AER
            if self.isOverLim():
                self.daysOverM += 1
            else:
                self.daysOverM = 0
            return self.daysOverM > 0

        # Round up an amount of money to a nice number.
        #
        def roundUp(self, money):
            result = math.ceil(money)
            if result > 1000:
                result = 100 * math.ceil(result / 100.0)
            elif result > 200:
                result = 50 * math.ceil(result / 50.0)
            elif result > 50:
                result = 25 * math.ceil(result / 25.0)
            elif result > 10:
                result = 10 * math.ceil(result / 10.0)
            return int(result)

        # Set of clear food limit.
        #
        def setFoodLim(self, value):
            self.foodLimM = value

        # Set the opening balance.
        #
        def setOpeningBalance(self, money):
            self.balanceM = money
            self.balanceOrigM = money

        # Set the overdraft limit.
        #
        def setOverdraftLim(self, money):
            self.overdraftLimM = money
            if self.isOverLim():
                # If still over limit, set/keep food limit.
                self.foodLimM = True
            else:
                # If overdraft means no longer over limit, reset over days
                # and clear food limit.
                self.daysOverM = 0
                self.foodLimM = False
