#
#  Class for clothes availability and fit.
#

# Key constants for clothing items.
#
define  ckBra = 'bra'
define  ckJeans = 'jeans'
define  ckLeggings = 'leggings'
define  ckMuumuu = 'muumuu'
define  ckPanties = 'panties'
define  ckSlacks = 'slacks'
define  ckSwimsuit = 'swimsuit'
define  ckTshirt = 'tshirt'
define  ckVachWare = 'vware'
define  ckWork = 'work'

init python:

    # Very simple clothing system.
    #
    # Records the largest size owned, assuming all smaller sizes are
    # already owned.
    #
    class Clothing:
        sizeMap = dict()            # Map of clothing part to size
        dbgStr = "Empty"            # Text representation

        # Constructor.
        #
        # sizeMap   - clothing size map
        #
        def __init__(self, sizeMap):
            self.sizeMap = sizeMap
            self.dbgStr = "Empty"
            self.init()

        # Assign the map to use.
        #
        def setMap(self, map):
            self.sizeMap = map
            self.init()
            if debug:
                dbg("Clothing.setMap({}) {}".format(type(map), self.dbgStr))

        # Set up initial state.
        # Ensure minimum sises exist.
        #
        def init(self):
            if not ckBra in self.sizeMap:
                self.sizeMap[ckBra] = 2;
            if not ckJeans in self.sizeMap:
                self.sizeMap[ckJeans] = 2;
            if not ckTshirt in self.sizeMap:
                self.sizeMap[ckTshirt] = 2;
            if not ckPanties in self.sizeMap:
                self.sizeMap[ckPanties] = 2;
            if not ckWork in self.sizeMap:
                self.sizeMap[ckWork] = 4;
            self.dbgStr = self.debug()

        # Is an article of the right size owned?
        #
        def has(self, gsize, cKey):
            result = False
            if cKey in self.sizeMap:
                got = self.sizeMap[cKey]
                # if debug:
                    # dbg("Clothing.has: found {} size {}".format(cKey, got))
                result = gsize <= got
            if debugM:
                dbg("Clothing.has({},{})={}".format(gsize, cKey, result))
            return result

        # Are all listed articles of the right size?
        #
        def hasAll(self, gsize, *cKeys):
            for cKey in cKeys:
                if not self.has(gsize, cKey):
                    return False
            return True

        # Are all listed articles of the right size?
        #
        def hasAllList(self, gsize, cKeyList):
            for cKey in cKeyList:
                if not self.has(gsize, cKey):
                    return False
            return True

        # List all articles which are avaliable in a size.
        #
        def listAll(self, gsize, *cKeys):
            result = []
            for cKey in cKeys:
                if self.has(gsize, cKey):
                    result.append(cKey)
            return result

        # List all articles which are avaliable in a size.
        #
        def listAllList(self, gsize, cKeyList):
            result = []
            for cKey in cKeyList:
                if self.has(gsize, cKey):
                    result.append(cKey)
            return result

        # List all articles which are not avaliable in a size.
        #
        def listAllNot(self, gsize, *cKeys):
            result = []
            for cKey in cKeys:
                if not self.has(gsize, cKey):
                    result.append(cKey)
            return result

        # List all articles which are not avaliable in a size.
        #
        def listAllNotList(self, gsize, cKeyList):
            result = []
            for cKey in cKeyList:
                if not self.has(gsize, cKey):
                    result.append(cKey)
            return result

        # Is an article of any size owned?
        #
        def any(self, cKey):
            return cKey in self.sizeMap

        # Buy a new size.
        #
        def buy(self, cKey, gsize):
            if cKey in self.sizeMap:
                self.sizeMap[cKey] = max(gsize, self.sizeMap[cKey])
            else:
                self.sizeMap[cKey] = gsize
            self.dbgStr = self.debug()
            if debug:
                dbg("Clothing.buy({},{})".format(cKey, gsize))

        # Get the current max size of an item.
        #
        def get(self, cKey):
            if cKey in self.sizeMap:
                return self.sizeMap[cKey]
            return None

        # Lose an article of a given size.
        # Sets the available sizes to gsize -1.
        # If gsize is zero removes the item completely.
        #
        def lose(self, cKey, gsize):
            if cKey in self.sizeMap:
                if gsize == 0:
                    self.sizeMap.pop(cKey)
                else:
                    self.sizeMap[cKey] = gsize - 1
            self.dbgStr = self.debug()
            if debug:
                dbg("Clothing.lose({},{})".format(cKey, gsize))

        # Lose all articles.
        #
        def loseAll(self, cKey):
            self.lose(cKey, 0)

        # List the available sizes.
        #
        def debug(self, group=4):
            total = len(self.sizeMap)
            if total == 0:
                return "Empty"

            count = 1
            result = ""
            for name in sorted(self.sizeMap.keys()):
                result += "{}={}".format(name, self.sizeMap[name])
                total -= 1
                if count < group:
                    result += " "
                    count += 1
                elif total > 0:
                    result += "\n"
                    count = 0
            return result
