# Generic set of counters.
#
init python:

    class Counters:
        # dbgStr = "Unused"
        # map = dict()

        # Constructor.
        #
        # map   - storage map
        # cbMap - optional callback map
        #
        def __init__(self, map, cbMap=None):
            self.map = map
            self.cbMap = cbMap
            self.dbgStr = "Unused"

        # Assign the map to use.
        #
        def setMap(self, map, cbMap=None):
            self.map = map
            self.cbMap = cbMap
            self.dbgStr = self.debug()
            if debug:
                dbg("Counters.setMap({}, {}) size={}".format(type(map), type(cbMap), len(map)))

        # Get the current value of the named counter.
        # If the counter is not defined return 0
        #
        def get(self, name):
            if name in self.map:
                return self.map[name]
            return 0

        # Add a value to a counter.
        # If the counter is not defined set it to the value.
        # Returns the new value
        #
        def add(self, name, value):
            if name in self.map:
                self.map[name] += value
            else:
                self.map[name] = value
            self.dbgStr = self.debug()
            return self.map[name]

        # Increment the named counter.
        # If the counter is not defined it is set to 1
        # Returns the new value.
        #
        def inc(self, name):
            if name in self.map:
                self.map[name] += 1
            else:
                self.map[name] = 1
            self.dbgStr = self.debug()
            return self.map[name]

        # Decrement the named counter.
        # If the counter is not defined it is set to -1
        #
        def dec(self, name):
            if name in self.map:
                self.map[name] -= 1
            else:
                self.map[name] = -1
            self.dbgStr = self.debug()
            return self.map[name]

        # Decrement the named counter.
        # If the counter is not defined it is set to -1
        # If the new vale is zero or below return True
        # Otherwise return False.
        #
        def decIsZero(self, name):
            if name in self.map:
                self.map[name] -= 1
            else:
                self.map[name] = -1
            self.dbgStr = self.debug()
            return self.map[name] <= 0

        # Decrement all entries.
        # Remove ones that are now zero or below.
        # If a counter reaches zero and there is a callback defined, call it.
        #
        def decAllRm(self):
            discard = []
            for name in self.map.keys():
                value = self.map[name] -1
                if value > 0:
                    self.map[name] = value
                else:
                    self.map[name] = 0
                    discard.append(name)
            for name in discard:
                del self.map[name]
                if self.cbMap is not None and name in self.cbMap:
                    pyExpr = self.cbMap[name]
                    if pyExpr is not None:
                        eval(pyExpr)
                    del self.cbMap[name]
            self.dbgStr = self.debug()

        # Set a named counter to a value (default 1).
        # Note: When using callbacks the expression may not jump/call RenPy labels.
        #
        # name  - counter name
        # value - initial value (default 1)
        # zcb   - dec to zero callback python expression (default None)
        #
        def set(self, name, value=1, zcb=None):
            self.map[name] = value
            if zcb is not None:
                self.cbMap[name] = zcb
            self.dbgStr = self.debug()

        # Set a named counter to zero if it exists.
        #
        def reset(self, name):
            if name in self.map:
                self.map[name] = 0
            self.dbgStr = self.debug()

        # Delete a named counter.
        #
        def delete(self, name):
            if name in self.map:
                del self.map[name]
                if hasattr(self, 'cbMap') and self.cbMap is not None and name in self.cbMap:
                    del self.cbMap[name]
                self.dbgStr = self.debug()

        # Returns true only if the named counter is set to 2.
        #
        def isDone(self, name):
            result = False
            if name in self.map:
                result = self.map[name] == 2
            return result

        # Returns true if the named count is less than lim or undefined.
        # Increments the named counter.
        #
        def isFirst(self, name, lim=1):
            result = True
            if name in self.map:
                result = self.map[name] < lim
                self.map[name] += 1
            else:
                self.map[name] = 1
            self.dbgStr = self.debug()
            return result

        # Returns true only if the named counter is set to 1.
        #
        def isOne(self, name):
            result = False
            if name in self.map:
                result = self.map[name] == 1
            return result

        # Returns true only if the named counter is set to 1, and changes it to 2.
        #
        def isOnce(self, name):
            result = False
            if name in self.map:
                value = self.map[name]
                if value == 1:
                    self.map[name] = 2
                    result = True
                    self.dbgStr = self.debug()
            return result

        # Returns true only if the named counter is set to a value other than zero.
        #
        def isNonZero(self, name):
            result = False
            if name in self.map:
                result = self.map[name] != 0
            return result

        # Returns true only if the named counter is zero or not set.
        #
        def isZero(self, name):
            result = True
            if name in self.map:
                result = self.map[name] == 0
            return result

        # Returns true only if the named counter is less than
        # the given value.
        #
        def isLt(self, name, value):
            tmp = 0
            if name in self.map:
                tmp = self.map[name]
            return tmp < value

        # Returns true only if the named counter is less than or equal to
        # the given value.
        #
        def isLte(self, name, value):
            tmp = 0
            if name in self.map:
                tmp = self.map[name]
            return tmp <= value

        # Returns true only if the named counter is equal to
        # the given value.
        #
        def isEq(self, name, value):
            tmp = 0
            if name in self.map:
                tmp = self.map[name]
            return tmp == value

        # Returns true only if the named counter is greater than
        # the given value.
        #
        def isGt(self, name, value):
            tmp = 0
            if name in self.map:
                tmp = self.map[name]
            return tmp > value

        # Returns true only if the named counter is greater than or equal to
        # the given value.
        #
        def isGte(self, name, value):
            tmp = 0
            if name in self.map:
                tmp = self.map[name]
            return tmp >= value

        # Clear all entries.
        #
        def clearAll(self):
            self.map.clear()
            self.dbgStr = self.debug()

        # List the set flags.
        #
        def debug(self, group=4):
            total = len(self.map)
            if total == 0:
                return "Empty"

            count = 1
            result = ""
            for name in sorted(self.map.keys()):
                result += "{}={}".format(name, self.map[name])
                total -= 1
                if count < group:
                    result += " "
                    count += 1
                elif total > 0:
                    result += "\n"
                    count = 0
            return result

# Dictionary to work with.
# Should be a member, but then renpy won't persist it properly.
#default self.map = dict()     # general purpose counters
