# Foods dictionary.
#
init python:

    class Food:
        fat = 0             # Fat gain in kg
        full = 0            # Fullness gained in litres
        drink = 0           # Alcohol units
        cost = 0            # Cost
        meal = False        # True for a meal, false for snack/drink

        # Add a food from the food dictionary by name.
        #
        def add(self, name):
            if name in foods:
                tuple = foods[name]
                self.fat += tuple[0]
                self.full += tuple[1]
                self.drink += tuple[2]
                self.cost += tuple[3]

                if debugM:
                    dbg("food.add({})={}".format(name, self.debug()))
            else:
                dbg("food.add({}) failed".format(name))

        # Add a list of foods from the food dictionary by name.
        #
        def addList(self, *names):
            for name in names:
                self.add(name)

        # Add a food from the food dictionary by name, but scaled.
        #
        # name      - string name of this food in the dictionary
        # scale     - int/float how much bigger and more expensive to make this food
        # costFac   - int/float additional scaling of expense, default 1
        #
        def addScale(self, name, scale, costFac=1):
            if name in foods:
                tuple = foods[name]
                self.fat += tuple[0] * scale
                self.full += tuple[1] * scale
                self.drink += tuple[2] * scale
                self.cost += tuple[3] * scale * costFac

                if debugM:
                    dbg("food.addScale({}, {:0.2f})={}".format(name, scale, self.debug()))
            else:
                dbg("food.addScale({}, {:0.2f}) failed".format(name, scale))

        # Scale cost, fat, and fullness.
        # Cost is rounded to the nearest 5c.
        #
        # scale     - int/float how much bigger and more expensive to make this food
        #
        def scaleAll(self, scale):
            self.cost = int(round(self.cost * 20 * scale, 5)) / 20.0    # Nearest 5c
            self.fat *= scale
            self.full *= scale
            self.drink *= scale

        # Scale cost only.
        # Cost is rounded to the nearest 5c.
        #
        # scale     - int/float how much more expensive to make this food
        #
        def scaleCost(self, scale):
            self.cost = int(round(self.cost * 20 * scale, 5)) / 20.0    # Nearest 5c

        # Scale up the existing fat and fullness.
        #
        # scale     - int/float how much bigger to make this food
        #
        def scaleFatSize(self, scale):
            self.fat *= scale
            self.full *= scale

        # Clear to zero value snack.
        #
        def clear(self):
            self.fat = 0
            self.full = 0
            self.drink = 0
            self.cost = 0
            self.meal = False

        # Get a food tuple from the library by name.
        # Returns null if not in library.
        #
        def getFood(self, name):
            if name in foods:
                return foods[name]
            else:
                dbg("food.getFood({}) failed unknown food name".format(name))
                return None

        # Get the cost of a named food item.
        #
        def getCost(self, name, scale=1.0):
            if name in foods:
                tuple = foods[name]
                cost = pc.roundUp(tuple[3] * scale)
                if debugM:
                    dbg("food.getCost({}, {})={}".format(name, scale, cost))
                return cost
            else:
                dbg("food.getCost({}) failed unknown food name".format(name))
                return 0

        # Get the cost string of a named food item.
        #
        def getCostStr(self, name, scale=1.0):
            if name == None:
                return "$0.00"
            if name in foods:
                tuple = foods[name]
                cost = pc.roundUp(tuple[3] * scale)
                return "${:0.2f}".format(cost)
            else:
                dbg("food.getCostStr({}) failed unknown food name".format(name))

        # Get the cost string of a named food items.
        #
        def getCostStrList(self, *names):
            acc = 0
            for name in names:
                if name in foods:
                    tuple = foods[name]
                    acc += tuple[3]
                else:
                    dbg("food.getCostStrList({}) failed unknown food name".format(name))

            return "${:0.2f}".format(acc)

        # Get how many portions would be needed to fill a given volume.
        #
        # volume    - the required volume
        #
        # Returns the nearest whole number of portions.
        #
        def portionsForVol(self, volume):
            result = 1
            if self.full > 0:
                result = int(round(float(volume) / self.full))
            dbg("food.portionsForVol({})={} full={:0.2f}".format(volume, result, self.full))
            return result

        # Get the scale that would be needed to fill a given volume.
        #
        # volume    - the required volume
        #
        # Returns the scaling factor required.
        #
        def scaleForVol(self, volume):
            result = 1
            if self.full > 0:
                result = float(volume) / self.full
            dbg("food.scaleForVol({})={} full={:0.2f}".format(volume, result, self.full))
            return result

        # Get the scaling needed for a named food to occupy the given volume.
        #
        def getScale(self, name, full):
            vol = 1
            if name in foods:
                tuple = foods[name]
                vol = tuple[1]
            else:
                dbg("food.getScale({}, {}) failed unknown food name".format(name, full))
            result = full / vol
            if debug:
                dbg("food.getScale({}, {})={}".format(name, full, result))
            return result

        # Is the food empty?
        #
        def isEmpty(self):
            return self.full < 0.001

        # Set the meal flag.
        #
        def setMeal(self, value):
            self.meal = value

        # Generate a debug string.
        #
        def debug(self):
            return "Food: fat={} full={} drink={} cost={}".format(self.fat, self.full, self.drink, self.cost)

        # List the foods keys.
        #
        def listFoods(self):
            result = "Foods: ({}) ".format(len(foods))

            for name in sorted(foods.keys()):
                result += name + " "

            return result

# Food Library.
#
# name -> (fat, full, drink, cost)
#
# Use define to reload when game starts.
# These are not persisted in a save game.
#
define foods = {
    "arancini"  : (0.05, 0.40, 0, 2.5),     # Arancini ragù
    "bananaspl" : (0.12, 0.50, 0, 3.5),     # banana split
    "beer"      : (0.03, 0.55, 1, 2.5),     # pint beer
    "beerbtl"   : (0.02, 0.33, 1, 2.0),     # beer bottle
    "blackberry": (0.01, 0.20, 0, 0.0),     # blackberries
    "browniesun": (0.10, 0.30, 0, 3.0),     # brownie sundae
    "bruschetta": (0.02, 0.25, 0, 2.5),     # bruschetta slices
    "burger"    : (0.05, 0.30, 0, 2.0),     # simple burger
    "burgc"     : (0.07, 0.30, 0, 2.5),     # cheese burger
    "burgmf"    : (0.13, 0.60, 0, 5.0),     # burger & fries
    "burgmcf"   : (0.14, 0.65, 0, 5.5),     # cheeseburger & fries
    "burgq"     : (0.12, 0.60, 0, 4.0),     # quartpounder
    "burgqc"    : (0.14, 0.60, 0, 4.5),     # double cheeseburger
    "burgqf"    : (0.20, 0.90, 0, 6.5),     # quartpounder & fries
    "burgqcf"   : (0.23, 1.00, 0, 7.0),     # double cheeseburger & fries
    "burgk"     : (0.20, 0.90, 0, 5.5),     # burger kingsize
    "burgkf"    : (0.26, 1.20, 0, 8.0),     # burger kingsize & fries
    "burrito"   : (0.32, 0.80, 0, 4.5),     # burrito
    "cafelait"  : (0.02, 0.10, 0, 1.5),     # cafe au lait
    "cafewien"  : (0.03, 0.10, 0, 2.0),     # vienesse coffee
    "cake"      : (0.12, 0.20, 0, 2.0),     # large cake slice
    "cakew"     : (0.96, 2.80, 0, 16.0),    # whole cake
    "cappucino" : (0.02, 0.10, 0, 1.5),     # cappucino
    "carbonara" : (0.08, 0.40, 0, 5.5),     # pasta carbonara
    "cheesecake": (0.08, 0.30, 0, 3.0),     # cheesecake
    "chicktend" : (0.13, 0.35, 0, 3.0),     # chicken tenders starter
    "chillirice": (0.07, 0.45, 0, 4.5),     # bowl of chilli and rice
    "chimichan" : (0.42, 0.90, 0, 5.5),     # chimichanga
    "chipsfmly" : (0.13, 0.20, 0, 3.0),     # family bag of crisps
    "chmeal"    : (0.26, 1.50, 0, 9.0),     # chinese meal whole inc starters
    "chocbox"   : (0.20, 0.20, 0, 10.0),    # luxury chocolate box (24 chocs)
    "chocs"     : (0.03, 0.05, 0, 0.5),     # 50g bar
    "chocm"     : (0.07, 0.10, 0, 1.0),     # 100g bar
    "chocl"     : (0.14, 0.20, 0, 2.0),     # 200g bar
    "chocpud"   : (0.04, 0.15, 0, 1.5),     # Chocolate pudding (moose) 300ml
    "cocktail"  : (0.03, 0.30, 3, 3.5),     # random cocktail
    "coffee"    : (0.00, 0.30, 0, 1.0),     # regular coffee
    "coke"      : (0.02, 0.30, 0, 1.0),     # coke can
    "cokez"     : (0.00, 0.30, 0, 1.0),     # coke can diet
    "cokept"    : (0.03, 0.50, 0, 1.5),     # coke pint
    "cokeptz"   : (0.03, 0.50, 0, 1.5),     # coke pint diet
    "creampt"   : (0.25, 0.55, 0, 1.0),     # whipping cream pt
    "croissant" : (0.04, 0.20, 0, 1.5),     # croissant and butter
    "cupcake"   : (0.03, 0.05, 0, 1.5),     # frosted cupcake
    "dbchoc"    : (0.11, 0.45, 0, 4.0),     # death by chocolate
    "dbchocw"   : (0.88, 3.60, 0, 32.0),    # death by chocolate (whole cake)
    "dimsum"    : (0.02, 0.01, 0, 0.5),     # dim sum one piece
    "donut"     : (0.04, 0.20, 0, 1.5),     # donut
    "espresso"  : (0.01, 0.05, 0, 1.0),     # espresso
    "friess"    : (0.03, 0.20, 0, 0.8),     # fries small
    "friesm"    : (0.04, 0.30, 0, 1.0),     # fries medium
    "friesl"    : (0.06, 0.40, 0, 1.5),     # fries large
    "friorim"   : (0.09, 0.60, 0, 2.0),     # fries and onion rings medium
    "frioril"   : (0.13, 0.80, 0, 3.0),     # fries and onion rings large
    "frybfast"  : (0.05, 0.30, 0, 1.0),     # fried breakfast
    "footlong"  : (0.10, 0.50, 0, 5.0),     # big sub sandwich
    "garlicb"   : (0.04, 0.25, 0, 1.0),     # garlic bread
    "hotchoc"   : (0.04, 0.25, 0, 2.0),     # hot chocolate w. cream
    "hotdog"    : (0.07, 0.25, 0, 2.5),     # hotdog
    "hotdogml"  : (0.13, 0.85, 0, 4.5),     # hotdog meal w. fries and coke
    "icecream"  : (0.03, 0.15, 0, 1.0),     # ice cream
    "icecreampt": (0.13, 0.50, 0, 3.5),     # ice cream pint
    "icedtea"   : (0.03, 0.45, 0, 1.5),     # iced tea
    "lasagna"   : (0.10, 0.40, 0, 6.0),     # lasagna meal
    "loadskins" : (0.12, 0.35, 0, 2.5),     # loaded potato skins
    "limoncello": (0.01, 0.03, 1, 2.0),     # limoncello shot
    "mozzsticks": (0.11, 0.30, 0, 2.5),     # mozzeralla sticks
    "orings"    : (0.04, 0.20, 0, 0.8),     # onion rings small
    "oringm"    : (0.05, 0.30, 0, 1.0),     # onion rings medium
    "oringl"    : (0.07, 0.40, 0, 1.5),     # onion rings large
    "pannacotta": (0.08, 0.15, 1, 3.0),     # panna cotta
    "panini"    : (0.10, 0.04, 0, 3.5),     # panini
    "peanuts"   : (0.04, 0.05, 0, 1.0),     # peanuts individual bag
    "peanutm"   : (0.16, 0.20, 0, 1.5),     # peanuts 200g bag
    "piea"      : (0.04, 0.35, 0, 2.0),     # apple pie
    "pieai"     : (0.06, 0.40, 0, 2.5),     # apple pie and ice cream
    "pizzas"    : (0.12, 0.30, 0, 5.5),     # pizza small 8in
    "pizzam"    : (0.16, 0.40, 0, 6.0),     # pizza medium 10in
    "pizzal"    : (0.23, 0.60, 0, 6.5),     # pizza large 12in
    "pizzaxl"   : (0.40, 1.20, 0, 10.0),    # pizza large 16in
    "popcorn"   : (0.06, 0.06, 0, 1.0),     # buttered popcorn serving
    "prawncrack": (0.08, 0.01, 0, 2,0),     # prawn crackers, large bag
    "regbreak"  : (0.18, 0.50, 0, 2.5),     # Reg breakfast
    "reglunch"  : (0.36, 0.80, 0, 4.5),     # Reg lunch
    "regmain"   : (0.72, 1.30, 0, 7.5),     # Reg generic evening meal
    "regsnack"  : (0.13, 0.10, 0, 1.0),     # Reg snack
    "regchoc"   : (0.16, 0.12, 0, 1.5),     # Reg snack
    "ribshalf"  : (0.12, 0.35, 0, 4.0),     # ribs half rack
    "ribsfull"  : (0.18, 0.55, 0, 6.0),     # ribs full rack
    "rumbaba"   : (0.08, 0.20, 1, 3.5),     # rum baba
    "salad"     : (0.01, 0.02, 0, 2.0),     # salad
    "sandwich"  : (0.05, 0.30, 0, 2.0),     # normal sandwich
    "shot"      : (0.01, 0.04, 1, 2.5),     # random shot
    "sliders"   : (0.06, 0.35, 0, 2.5),     # sliders x2
    "sofigliat" : (0.06, 0.20, 0, 2.5),     # sfogliatella
    "steak16"   : (0.24, 0.46, 0, 8.0),     # steak 16oz
    "steak24"   : (0.34, 0.71, 0, 10.0),    # steak 24oz
    "thetray"   : (1.10, 2.10, 0, 16.0),    # The meat tray
    "tiramisu"  : (0.08, 0.50, 0, 3.0),     # tiramisu bowl
    "trailmix"  : (0.03, 0.50, 0, 0.5),     # trail mix, handful
    "water"     : (0.00, 0.30, 0, 0.0),     # glass of water
    "wine"      : (0.03, 0.25, 2, 2.0),     # glass of wine
    "wrap"      : (0.07, 0.30, 0, 3.5),     # a tortilla wrap
    "zabaione"  : (0.02, 0.15, 1, 3.0)      # zabaione aka sabayon
}

default food = Food()

# Ref
# 7770kcal = 1kg fat
#
# kcal  fat
# 2000  0.260
# 1000  0.130
# 500   0.065
# 200   0.026
# 100   0.013
