#
# Customisation of food menu.
#
init python:

    # Additional information passed to food menus.
    #
    # If an entry matches the menu item prompt then the entries here replace it
    # in screen foodMenu.
    # When an entry is chosen in a food menu it's prompt is recorded in
    # foodMenuSeln. This can be passed to addMealAcc to keep track of the
    # total meal size.
    #
    class FoodMenu:
        mapM = dict()               # Map of name to FoodItem
        stylePrefixM = "food"       # Food menu style prefix
        titleM = None               # Menu title eg. "Bill's Fine Dining"
        subtitleM = None            # Menu subtitle eg. "Dessert Menu"
        minSizeM = 0                # Minimum meal size to consider
        maxSizeM = None             # Maximum meal size to allow
        maxSizeMsgM = None          # Maximum size message
        mealAcc = 0                 # Meal size accumulator

        # Constructor.
        #
        # foodMap   - dictionary to map menu prompt to FoodItem
        #
        def __init__(self, foodMap):
            self.mapM = foodMap             # Map of name to FoodItem
            self.stylePrefixM = "food"      # Food menu style prefix
            self.titleM = None              # Menu title eg. "Bill's Fine Dining"
            self.subtitleM = None           # Menu subtitle eg. "Dessert Menu"
            self.minSizeM = 0               # Minimum meal size to consider
            self.maxSizeM = None            # Maximum meal size to allow
            self.maxSizeMsgM = None         # Maximum size message
            self.mealAcc = 0                # Meal size accumulator

        # Start a new menu.
        # Optionally assign title and subtitle.
        # Does not clear meal size accumulator.
        #
        def start(self, title=None, subtitle=None):
            self.mapM.clear()
            self.titleM = title
            self.subtitleM = subtitle
            self.minSizeM = 0
            self.maxSizeM = None
            self.maxSizeMsgM = None

        # Start a new menu.
        # Optionally assign title and subtitle.
        # Clears the meal size accumulator.
        #
        def startClear(self, title=None, subtitle=None):
            self.mapM.clear()
            self.titleM = title
            self.subtitleM = subtitle
            self.minSizeM = 0
            self.mealAcc = 0
            self.maxSizeM = None
            self.maxSizeMsgM = None

        # Reset the meal size accumulator.
        #
        def resetMealAcc(self):
            self.mealAcc = 0

        # Add to the meal size accumulator.
        # name: the food item name used in the menu.
        #
        def addMealAcc(self, name):
            if name in self.mapM:
                self.mealAcc += self.mapM[name].mealSize
            if debug:
                dbg("FoodMenu.addMealAcc({})={}".format(name, self.mealAcc))

        # Does the PC have space for a dish of a given size?
        #
        # The global variable mealsize must be defined before use.
        #
        def isSpaceFor(self, dsize):
            result = mealsize >= dsize + self.mealAcc
            if debug:
                dbg("FoodMenu.isSpaceFor({})={} mealsize={}".format(dsize, result, mealsize))
            return result

        # Add a food item.
        #
        # name:     the food name from FoodDict
        # prompt:   the human food name
        # cost:     what the pc has spent so far
        # msize:    the meal size for this item
        # treat:    true if this meal is free
        # food:     optional food dict key to replace name
        # price:    optional price to override the one from food dict
        # enough:   override food minSize test
        #
        # The global variables pc, mealsize, and foods must be defined before use.
        #
        def addFood(self, name, prompt, cost=0, msize=0, treat=False, food=None, price=None, enough=False):
            item = FoodItem(prompt, msize, treat)
            if food == None:
                food = name
            if food in foods:
                if price != None:
                    item.cost = price
                else:
                    tuple = foods[food]
                    item.cost = tuple[3]
                if msize + self.mealAcc > mealsize:
                    # Item is too large to eat.
                    item.tooBig = True
                if self.maxSizeM != None and msize >= self.maxSizeM:
                    # Item would fit, but is bigger than set max size.
                    item.tooGreedy = True
                if not enough and msize < self.minSizeM and msize < mealsize - self.mealAcc:
                    # Item is to small.
                    item.tooSmall = True
                if not treat and not pc.canAfford(item.cost, cost):
                    # Item is too expensive.
                    item.tooPricey = True
                if self.maxSizeMsgM:
                    item.setPost("({})".format(self.maxSizeMsgM))
                else:
                    item.setPost()
                if debugM:
                    dbg("FoodMenu.addFood({},{},{},{},{},{})\n={}".format(name, prompt, cost, msize, treat, food, item.debugStr()))
            else:
                item.post = " (unknown food ''" + name + "'')"
                if price != None:
                    item.cost = price
                dbg("FoodMenu.addFood({},{},{},{},{},{})\n={}".format(name, prompt, cost, msize, treat, food, item.debugStr()))
            self.mapM[name] = item

        # Add a scaled food item.
        # The menu key is the namePref and the scale.
        # Only scales the cost.
        #
        # namePref: the food name from FoodDict or a unique name
        # prompt:   the human food name
        # cost:     what the pc has spent so far
        # msize:    the meal size for this item
        # treat:    true if this meal is free
        # food:     optional food dict key to replace name
        # price:    optional price to override the one from food dict
        # enough:   override food minSize test
        #
        # The global variables pc, mealsize, and foods must be defined before use.
        #
        def addFoodScale(self, scale, namePref, prompt, cost=0, msize=0, treat=False, food=None, price=None, enough=False):
            name = namePref + str(scale)
            item = FoodItem(prompt, msize, treat)
            if food == None:
                food = namePref
            if food in foods:
                if price != None:
                    item.cost = price
                else:
                    tuple = foods[food]
                    item.cost = floorPrec(tuple[3] * scale, 0.05)
                if msize + self.mealAcc > mealsize:
                    # Item is too large to eat.
                    item.tooBig = True
                if self.maxSizeM != None and msize >= self.maxSizeM:
                    # Item would fit, but is bigger than set max size.
                    item.tooGreedy = True
                if not enough and msize < self.minSizeM and msize < mealsize - self.mealAcc:
                    # Item is to small.
                    item.tooSmall = True
                if not treat and not pc.canAfford(item.cost, cost):
                    # Item is too expensive.
                    item.tooPricey = True
                if self.maxSizeMsgM:
                    item.setPost("({})".format(self.maxSizeMsgM))
                else:
                    item.setPost()
                if debugM:
                    dbg("FoodMenu.addFoodScale({},{},{},{},{},{})\n={}".format(name, prompt, cost, msize, treat, food, item.debugStr()))
            else:
                item.post = " (unknown food ''" + name + "'')"
                if price != None:
                    item.cost = price
                dbg("FoodMenu.addFoodScale({},{},{},{},{},{})\n={}".format(name, prompt, cost, msize, treat, food, item.debugStr()))
            self.mapM[name] = item

        # Add a menu section header.
        # name: the header text
        #
        def addHeader(self, name):
            item = FoodItem(name)
            item.header = True
            self.mapM[name] = item
            if debugM:
                dbg("FoodMenu.addHeader({})={}".format(name, item.text))

        # Add a "nothing" item.
        # Normally all menus should include 'nothing' even if it is not visible
        # by default in case the PC is completely full or has no money.
        #
        # visible   - make entry visible, default True
        # prompt    - prompt to offer, default "Nothing"
        # name      - caption key, default 'nothing'
        #
        def addNothing(self, visible=True, prompt="Nothing", name='nothing'):
            item = FoodItem(prompt)
            item.hidden = not visible
            self.mapM[name] = item
            if debugM:
                dbg("FoodMenu.addNothing({}, '{}', '{}')={}".format(visible, prompt, name))


        def check(self):
            """
            Make sure at least one entry is selectable.

            Entries with too small a meal size are added first.
            Hidden nothing entries are revealled if needed.
            """
            while not self.isAny() and self.minSizeM > 0:
                # Try reducing minSize.
                self.minSizeM -= 1
                for entry in self.mapM.values():
                    if entry.mealSize == self.minSizeM and entry.tooSmall:
                        entry.tooSmall = False
                        entry.setPost(self.maxSizeMsgM)
                        if debug:
                            dbg("FoodMenu.check() allowing {}".format(entry))
            # Still nothing? Reveal hidden entries.
            if not self.isAny():
                for entry in self.mapM.values():
                    if entry.mealSize == 0 and entry.hidden:
                        entry.hidden = False
                        if debug:
                            dbg("FoodMenu.check() revealing {}".format(entry))





        # ---------------------------------------------------------------------
        # Accessors
        # - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -

        # Get the food item corresponding to a name.
        #
        def get(self, name):
            return self.mapM[name]

        # Get the current tally of plates.
        #
        def getAccPlates(self):
            return self.mealAcc

        # Get the cost of the food item corresponding to a name.
        # If the item was a treat it's cost is zero.
        #
        def getCost(self, name):
            result = 0
            if name in self.mapM:
                item = self.mapM[name]
                result = item.cost
                if item.treat:
                    result = 0
            if debug:
                dbg("FoodMenu.getCost({})={}".format(name, result))
            return result

        # Get the cost of the food item corresponding to a name.
        # Ignores whether the item is a treat.
        #
        def getCostRaw(self, name):
            result = 0
            if name in self.mapM:
                item = self.mapM[name]
                result = item.cost
            if debug:
                dbg("FoodMenu.getCostRaw({})={}".format(name, result))
            return result

        # Get the number of entries in the map.
        #
        def getLen(self):
            return len(self.mapM)

        # Get the subtitle.
        #
        def getSubtitle(self):
            return self.subtitleM

        # Get the title.
        #
        def getTitle(self):
            return self.titleM

        # Determine if a name is in the food menu map.
        #
        def has(self, name):
            if name in self.mapM:
                return True
            return False

        def isAny(self):
            """
            Are there any selectable entries?
            """
            for entry in self.mapM.values():
                if entry.available:
                    return True
            return False

        # Determine if an item is available to select.
        #
        def isAvailable(self, name):
            return self.mapM[name].available

        # Assign the map to use.
        #
        def setMap(self, map):
            self.mapM = map
            if debug:
                dbg("FoodMenu.setMap({}) size={}".format(type(map), len(map)))

        # Minimum size to consider.
        # Anything below this could be marked as too small unless it "just fits".
        # Ensure a range of bigger options are available.
        # Do not use when dieting!
        # Always call check() after all items are added.
        #
        def setMinSize(self, minSize):
            self.minSizeM = minSize
            if debug:
                dbg("FoodMenu.setMinSize({})".format(minSize))

        # Maximum size to consider.
        # Anything this big or above is not selectable, and marked with the message.
        # Ensure smaller options are available.
        #
        def setMaxSize(self, maxSize, maxSizeMsg="too much"):
            self.maxSizeM = maxSize
            self.maxSizeMsgM = maxSizeMsg
            if debug:
                dbg("FoodMenu.setMaxSize({}, {})".format(maxSize, maxSizeMsg))

        # Get the text associated with an item.
        #
        def text(self, name):
            return self.mapM[name].text

# EoF
