# NPC Character stats
#
define  npcRomFac = 800.0       # Romance impact on fat fact

init python:

    # Basic character stats.
    #
    class Npc:
        varname = None          # Global variable name string
        metCount = 0            # Number of times met
        fancy = 0               # How much the NPC fancies the PC 0..100
        dateCount = 0           # Number of dates
        dateLastDay = 0         # Last date day number
        canCall = False         # True if pc has their number
        romance = 0             # Romance level 0..100
        feeder = 0              # Feeder level 0..100
        feedee = 0              # Feedee level 0..100
        breakup = False         # True if broken up
        breakCount = 0          # Number of breakups
        breakUntil = 0          # On a break until this day
        pcMinBfp = 10           # Minimum player bfp
        pcMaxBfp = 35           # Maximum player bfp
        pcLastMass = 0          # Mass of the PC last time they met
        weight = 50             # My weight in kg
        tipYes = 0              # Tips given
        tipNo = 0               # Tips not given
        talkNum = 0             # Talk topic sequence
        genFemale = False       # Has female parts
        genMale = False         # Has male parts

        # ---------------------------------------------------------------------
        # Methods.
        # ---------------------------------------------------------------------

        # Add to the fancy stat.
        # Negative values reduce, but not below zero.
        # Positive values increase, but not above 100.
        #
        def addFancy(self, value):
            self.fancy += value
            self.fancy = max(0.0, min(100.0, self.fancy))
            if debug:
                dbg("npc.addFancy({})={}".format(value, self.fancy))

        # Add to the feedee stat.
        # Negative values reduce, but not below zero.
        # Positive values increase, but not above 100.
        #
        def addFeedee(self, value):
            self.feedee += value

            if self.feedee < 0:
                self.feedee = 0
            if self.feedee > 100:
                self.feedee = 100
            if debug:
                dbg("npc.addFeedee({})={}".format(value, self.feedee))

        # Add to the feeder stat.
        # Negative values reduce, but not below zero.
        # Positive values increase, but not above 100.
        #
        def addFeeder(self, value):
            self.feeder += value

            if self.feeder < 0:
                self.feeder = 0
            if self.feeder > 100:
                self.feeder = 100
            if debug:
                dbg("npc.addFeeder({})={}".format(value, self.feeder))

        # Add to the romance stat.
        # Negative values reduce, but not below zero.
        # Positive values increase, but not beyond 100.
        #
        def addRomance(self, value):
            self.romance += value
            if self.romance < 0:
                self.romance = 0
            if self.romance > 100:
                self.romance = 100
            if debug:
                dbg("npc.addRomance({})={}".format(value, self.romance))

        # Increase dateCount by one and set dateLastDay.
        #
        def date(self):
            self.dateCount += 1
            self.dateLastDay = gt.dayNum

        # Build a debug string for current values.
        #
        def debug(self):
            return "metCount={} canCall={} feeder={} romance={}".format(self.metCount, self.canCall, self.feeder, self.romance)

        # Get back together.
        #
        def makeup(self):
            self.breakup = False;
            self.breakUntil = 0;

        # Increase metCount by one.
        # Record PC's last mass.
        # For this to be useful call at the end of each meeting.
        #
        def meet(self):
            self.metCount += 1
            self.pcLastMass = pc.getMass()

        # Determine NPC opinion of pc's weight.
        #
        # 'tooThin' : Much too thin
        # 'thin'    : On the thin side
        # 'good'    : A good weight
        # 'fat'     : Too fat
        # 'tooFat'  : Much too fat
        #
        def opinionPc(self):
            bfp = pc.getBodyFatPer()
            result = "good"

            if bfp < self.pcMinBfp:
                if self.isPcTooThin():
                    result = "tooThin"
                else:
                    result = "thin"

            if bfp > self.pcMaxBfp:
                if self.isPcTooFat():
                    result = "tooFat"
                else:
                    result = "fat"

            if debug:
                dbg("opinionPc() {}".format(result))

            return result

        # Get next topic number 0..(max-1).
        # Results are an ascending sequence 0, 1, 2, 3, ... 15
        # Each topic has it's own sequence.
        #
        # max: sequence length 1..16
        # topic: topic number, default 0
        #
        def talkTopic(self, max, topic = 0):
            shift = topic * 4
            bitmask =  15 << (topic * 4)
            result = (self.talkNum >> shift) & 15
            if result >= max:
                result = 0
            self.talkNum = (self.talkNum & ~bitmask) + ((result + 1 & 15) << shift)
            if debugM:
                dbg("talkTopic({}, {})={} shift={} bitmask={:08x} talkNum={:08x}".format(max, topic, result, shift, bitmask, self.talkNum))
            return result

        # Record tipping.
        #
        # tipped    - True for tipped, False for not tipped
        #
        def tip(self, tipped):
            if tipped:
                if self.tipYes:
                    self.tipYes += 1
                else:
                    self.tipYes = 1
            else:
                if self.tipNo:
                    self.tipNo += 1
                else:
                    self.tipNo = 1



        # ---------------------------------------------------------------------
        # Random chances.
        # ---------------------------------------------------------------------

        # Random chance of at least min, but rising to max with max fancy.
        # If the fancy member is below thresh (default 0.001) or during break period this test always fails.
        #
        # min       - minimum percentage 0..100
        # max       - maximum percentage 0..100
        # thresh    - minimum value of fancy to allow test
        #
        def randomFancy(self, min, max, thresh=None):
            if thresh == None:
                thresh = 0.001
            if self.fancy < thresh or self.isOnBreak():
                chance = 0
                result = False
            else:
                chance = min + (max - min) * self.fancy / 100.0
                result = pc.randomBool(chance)

            if debug:
                dbg("randomFancy({},{})={} chance={}".format(min, max, result, chance))
            return result

        # Random chance of NPC being flirty.
        #
        # lowLim    - lower fancy limit below which chance is zero
        # highLim   - upper fancy limit where chance becomes prob
        # prob      - max probability of success 0..100 (default 100)
        #
        # Always returns False if fancy below lowLim.
        # Returns True prob percent of the time if fancy above highLim.
        # Probability proportional to where fancy lies between limits otherwise.
        #
        def randomFancyStat(self, lowLim, highLim, prob=100):
            result = pc.randomBoolStat(self.fancy, lowLim, highLim, prob)
            if debug:
                dbg("randomFancyStat({}, {}, {})={}".format(lowLim, highLim, prob, result))
            return result

        # Random chance of at least min, but rising to max with max feedee.
        # If the feedee member is zero this test always fails.
        #
        # min   - minimum percentage 0..100
        # max   - maximum percentage 0..100
        #
        def randomFeedee(self, min, max):
            if self.feedee == 0:
                chance = 0
                result = False
            else:
                chance = min + (max - min) * self.feedee / 100.0
                result = pc.randomBool(chance)

            if debugM:
                dbg("randomFeedee({},{})={} chance={}".format(min, max, result, chance))
            return result

        # Random chance of NPC wanting to be fed.
        #
        # lowLim    - lower feedee limit below which chance is zero
        # highLim   - upper feedee limit where chance becomes prob
        # prob      - max probability of success 0..100 (default 100)
        #
        # Always returns False if feedee below lowLim.
        # Returns True prob percent of the time if feedee above highLim.
        # Probability proportional to where feedee lies between limits otherwise.
        #
        def randomFeedeeStat(self, lowLim, highLim, prob=100):
            result = pc.randomBoolStat(self.feedee, lowLim, highLim, prob)
            if debug:
                dbg("randomFeedeeStat({}, {}, {})={}".format(lowLim, highLim, prob, result))
            return result

        # Random chance of at least min, but rising to max with max feeder.
        # If the feeder member is zero this test always fails.
        #
        # min   - minimum percentage 0..100
        # max   - maximum percentage 0..100
        #
        def randomFeeder(self, min, max):
            if self.feeder == 0:
                chance = 0
                result = False
            else:
                chance = min + (max - min) * self.feeder / 100.0
                result = pc.randomBool(chance)

            if debugM:
                dbg("randomFeeder({},{})={} chance={}".format(min, max, result, chance))
            return result

        # Random chance of NPC feeding.
        #
        # lowLim    - lower feeder limit below which chance is zero
        # highLim   - upper feeder limit where chance becomes prob
        # prob      - max probability of success 0..100 (default 100)
        #
        # Always returns False if feeder below lowLim.
        # Returns True prob percent of the time if feeder above highLim.
        # Probability proportional to where feeder lies between limits otherwise.
        #
        def randomFeederStat(self, lowLim, highLim, prob=100):
            result = pc.randomBoolStat(self.feeder, lowLim, highLim, prob)
            if debug:
                dbg("randomFeederStat({}, {}, {})={}".format(lowLim, highLim, prob, result))
            return result

        # Random chance of at least min, but rising to max with max romance.
        # Ignores breakup or romance is zero.
        #
        # min   - minimum percentage 0..100
        # max   - maximum percentage 0..100
        #
        def randomPlatRomance(self, min, max):
            chance = min + (max - min) * self.romance / 100.0
            result = pc.randomBool(chance)

            if debug:
                dbg("randomPlatRomance({},{})={} chance={}".format(min, max, result, chance))
            return result

        # Random chance of at least min, but rising to max with max romance.
        # Always False if on a breakup or romance is zero.
        #
        # min   - minimum percentage 0..100
        # max   - maximum percentage 0..100
        #
        def randomRomance(self, min, max):
            if self.breakup or self.romance == 0:
                chance = 0
                result = False
            else:
                chance = min + (max - min) * self.romance / 100.0
                result = pc.randomBool(chance)

            if debug:
                dbg("randomRomance({},{})={} chance={}".format(min, max, result, chance))
            return result

        # Random chance of couple being romantic.
        #
        # lowLim    - lower romance limit below which chance is zero
        # highLim   - upper romance limit where chance becomes prob
        # prob      - max probability of success 0..100 (default 100)
        #
        # Always returns False if romance below lowLim or on a breakup
        # Returns True prob percent of the time if romance above highLim.
        # Probability proportional to where romance lies between limits otherwise.
        #
        def randomRomanceStat(self, lowLim, highLim, prob=100):
            if self.breakup or self.romance == 0:
                result = False
            else:
                result = pc.randomBoolStat(self.romance, lowLim, highLim, prob)
            if debug:
                dbg("randomRomanceStat({}, {}, {})={}".format(lowLim, highLim, prob, result))
            return result

        # Random chance of at least min, but rising to max with more tips.
        # Always False if fewer than ten tipping oppertunities are recorded.
        #
        # min       - minimum percentage 0..100
        # max       - maximum percentage 0..100
        # unknown   - return value if not enough tips yet, default False
        #
        def randomTip(self, min, max, unknown=False):
            tot = self.tipYes + self.tipNo
            # dbg("yes={} no={} tot={}".format(self.tipYes, self.tipNo, tot))
            result = unknown
            chance = 0
            if tot >= 10:
                chance = min + (max - min) * (self.tipYes * 1.0 / tot)
                result = pc.randomBool(chance)

            if debug:
                dbg("randomTip({},{},{})={} chance={} tot={}".format(min, max, unknown, result, chance, tot))
            return result



        # ---------------------------------------------------------------------
        # Accessors
        # - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -

        # Get number of breakups.
        #
        def getBreakCount(self):
            return self.breakCount

        # Get the difference in weight of the pc between now and when they last met.
        #
        def getDeltaKg(self):
            return pc.getMass() - self.pcLastMass

        # Get the effective romance value.
        # Always returns zero when on a break.
        #
        def getEffRomance(self):
            result = 0
            if not self.breakup:
                result = self.romance
            return result

        # Get the effective fancy value.
        # Always returns zero when on a break.
        #
        def getEffFancy(self):
            result = 0
            if not self.breakup:
                result = self.fancy
            return result

        # Get the fancy value.
        #
        def getFancy(self):
            return self.fancy

        # Get NPCs mass in kg.
        #
        def getMass(self):
            return self.weight

        # Get the difference in mass between mass in kg which is regarded as
        # being too fat and the PCs current weight.
        #
        # romance - romance level to use, or None for current
        #
        def getTooFatDeltaKg(self, romance=None):
            return pc.getMass() - self.getTooFatMass(romance)

        # Calculate the threshold mass in kg which is regarded as being too fat.
        #
        # romance - romance level to use, or None for current
        #
        def getTooFatMass(self, romance=None):
            if romance == None:
                romance = self.romance
            fact = 0.9 - romance/npcRomFac
            bfpd = self.pcMaxBfp / 100.0
            adjFat = bfpd * pc.base / (1 - bfpd)
            result = pc.base + adjFat / fact
            if debug:
                dbg("getTooFatMass()={:0.1f}/{:0.1f} bfpd={:0.4f} fact={:0.3f} adjFat={:0.2f}".format(result, result*2.2, bfpd, fact, adjFat))
            return result

        # Is this NPC broken up with?
        #
        def isBreak(self):
            return self.breakup

        # Is the number of break counts greater or equal to the count supplied?
        #
        def isBreakCountGte(self, count):
            return self.breakCount >= count

        # Is date count greater or equal to value?
        # If the value is not given, true if dateCount > 0.
        #
        def isDate(self, count=None):
            if count == None:
                return self.dateCount > 0
            return self.dateCount >= count

        # Is fancy greater or equal to value?
        # If the value is not given, true if fancy > 0.
        #
        def isFancy(self, value=None):
            if value == None:
                return self.fancy > 0
            return self.fancy >= value

        # Is feedee greater or equal to value?
        # If the value is zero, true if feeder > 0.
        #
        def isFeedee(self, value=0):
            if value == 0:
                return self.feedee > 0

            return self.feedee >= value

        # Is feeder greater or equal to value?
        # If the value is zero, true if feeder > 0.
        #
        def isFeeder(self, value=0):
            if value == 0:
                return self.feeder > 0

            return self.feeder >= value

        # Do they have female parts?
        #
        def isFemale(self):
            return self.genFemale

        # Do they have male parts?
        #
        def isMale(self):
            return self.genMale

        # Has the pc met them before?
        #
        def isMet(self):
            return self.metCount >= 1

        # Has the pc met them count times?
        #
        def isMetCount(self, count):
            return self.metCount == count

        # Has the pc met them count times or more?
        #
        def isMetOrMore(self, count):
            return self.metCount >= count

        # Had the pc not met them before?
        #
        def isNotMet(self):
            return self.metCount == 0

        # Is the PC still taking a timed break from the NPC?
        # Tests the breakUntil against the day number.
        # Does not change the break state, that needs a call to makeup.
        #
        def isOnBreak(self, days=0):
            if self.breakUntil == 0:
                return False
            result = self.breakUntil + days > gt.dayNum
            # if debugM:
                # dbg("isOnBreak()={}".format(result))
            return result

        # Is the PC fat in the npc's opinion?
        #
        def isPcFat(self):
            bfp = pc.getBodyFatPer()
            result = bfp > self.pcMaxBfp
            if debugM:
                dbg("isPcFat()={}".format(result))
            return result

        # Is pc fatter now then when they last met by at least deltaKg?
        #
        def isPcFatter(self, deltaKg=None):
            if deltaKg == None:
                deltaKg = 0.46    # One pound+
            return pc.getMass() >= self.pcLastMass + deltaKg

        # Is the PC thin in the npc's opinion?
        #
        def isPcThin(self):
            bfp = pc.getBodyFatPer()
            result = bfp < self.pcMinBfp
            if debugM:
                dbg("isPcThin()={}".format(result))
            return result

        # Is pc thinner now then when they last met by at least deltaKg?
        #
        def isPcThinner(self, deltaKg=None):
            if deltaKg == None:
                deltaKg = 0.46    # One pound+
            return pc.getMass() <= self.pcLastMass - deltaKg

        # Is the PC too fat, taking romance into account, in the NPC's opinion?
        #
        # deltaKg - adjustment to PCs mass.
        #
        # Factor can vary from 0.9 to 0.875 or 11% to 14.3%.
        #
        def isPcTooFat(self, deltaKg=0):
            fact = 0.9 - self.romance/npcRomFac
            adjFat = (pc.fat + deltaKg) * fact
            bfp = 100.0 * adjFat / (adjFat + pc.base)
            result = bfp > self.pcMaxBfp
            # dbg("isPcTooFat()={} fact={} fat={} bfp={} max={}".format(result, fact, fat, bfp, self.pcMaxBfp))
            if debugM:
                dbg("isPcTooFat({:0.2f})={}".format(deltaKg, result))
            return result

        # Is the pc too thin, taking in to account romance, for this NPC?
        #
        # Factor can vary from 1.1 to 1.6.
        #
        def isPcTooThin(self):
            fact = 1.1 + self.romance/200.0
            fat = pc.fat * fact
            bfp = 100.0 * fat / (fat + pc.base)
            result = bfp < self.pcMinBfp
            # dbg("isPcTooThin()={} fact={} fat={} bfp={} min={}".format(result, fact, fat, bfp, self.pcMinBfp))
            if debugM:
                dbg("isPcTooThin()={}".format(result))
            return result

        # Is the romance greater or equal to value?
        # If the value is zero, true if romance > 0.
        # During breakup this is always false even if breakUntil has elapsed.
        #
        def isRomance(self, value=0):
            if self.breakup:
                return False
            if value == 0:
                return self.romance > 0
            return self.romance >= value

        # Is the raw romance greater or equal to value?
        # If the value is zero, true if romance > 0.
        #
        def isRomanceRaw(self, value=0):
            if value == 0:
                return self.romance > 0
            return self.romance >= value

        # Is tipping percentage greater than a value (0..100)?
        #
        def isTip(self, pvalue=1):
            tot = self.tipYes + self.tipNo
            result = False
            perc = 0
            # dbg("yes={} no={} tot={}".format(self.tipYes, self.tipNo, tot))
            if tot >= 10:
                perc = self.tipYes * 100 / tot
                result = perc >= pvalue
            if debug:
                dbg("isTip({})={} perc={}".format(pvalue, result, perc))
            return result

        # Set a number of days to break up with this NPC.
        #
        def setBreakDays(self, days):
            if not self.breakup:
                self.breakup = True
                self.breakCount += 1
            self.breakUntil = gt.dayNum + days

        # Set the fancy value.
        #
        def setFancy(self, value):
            self.fancy = value

        # Raise min bfp to at least the given value.
        #
        def setMinBfpAtLeast(self, bfp):
            if self.pcMinBfp < bfp:
                self.pcMinBfp = bfp
            if debug:
                dbg("setMinBfpAtLeast({:0.1f}) min={:0.1f}".format(bfp, self.pcMinBfp))

        # Raise max bfp to at least the given value.
        #
        def setMaxBfpAtLeast(self, bfp):
            if self.pcMaxBfp < bfp:
                self.pcMaxBfp = bfp
            if debug:
                dbg("setMaxBfpAtLeast({:0.1f}) max={:0.1f}".format(bfp, self.pcMaxBfp))

        # Set max bfp to at least a value where the PC's weight will trigger tooFat.
        #
        # kg        - the tooFat limit in kilos
        # atRomance - this NPCs target romance, default 100
        # override  - True to allow bfp to be reduced, otherwise it is only ever increased
        #
        def setMaxBfpByWeightKg(self, kg, atRomance=100, override=False):
            atFat = kg - pc.base
            fact = 0.9 - atRomance / npcRomFac
            kgFat = atFat * fact
            bfp = 100.0 * kgFat / (kgFat + pc.base)
            if override:
                self.pcMaxBfp = bfp
            else:
                self.pcMaxBfp = max(self.pcMaxBfp, bfp)
            if debug:
                dbg("setMaxBfpByWeightKg({}, {}, {})={:0.4f} fact={} atFat={:0.2f} kgFat={:0.2f} bfp={:0.4f}".format(kg, atRomance, override, self.pcMaxBfp, fact, atFat, kgFat, bfp))

        # Raise max bfp to at lease a value where the PC's weight will trigger tooFat.
        #
        # lb        - the tooFat limit in lbs
        # atRomance - this NPCs target romance, default 100
        # override  - True to allow bfp to be reduced, otherwise it is only ever increased
        #
        def setMaxBfpByWeightLb(self, lb, atRomance=100, override=False):
            kg = lb / 2.2
            atFat = kg - pc.base
            fact = 0.9 - atRomance / npcRomFac
            kgFat = atFat * fact
            bfp = 100.0 * kgFat / (kgFat + pc.base)
            if override:
                self.pcMaxBfp = bfp
            else:
                self.pcMaxBfp = max(self.pcMaxBfp, bfp)
            if debug:
                dbg("setMaxBfpByWeightLb({}, {})={:0.4f} kg={:0.2f} fact={} atFat={:0.2f} kgFat={:0.2f} bfp={:0.4f}".format(lb, atRomance, self.pcMaxBfp, kg, fact, atFat, kgFat, bfp))

        # Set romance value.
        #
        # value     - new value
        #
        def setRomance(self, value):
            self.romance = value

        # Set this PCs weight in pounds.
        #
        def setWeightLb(self, lb):
            self.weight = lb / 2.2

# EoF
