# Player character class
#
init python:
    import math

    class Player:
        name = "You"            # The pc's name
        drunk = 0               # Alcohol units drunk today
        ageBase = 22            # Age in years at start
        heightCm = 158          # Height in cm
        bmrMassFac = None       # BMR mass factor for plateau
        base = 40               # Base weight in kg
        fat = 0                 # Fat gained in kg
        fatLim = 12             # Limit to have fat in kg the player is willing to get
        massSoftLim = 382       # Soft limit on pc mass
        massHardLim = 400       # Hard limit on pc mass
        full = 0                # Fullness in litres
        greed = 0               # Greediness 0..100
        diet = False            # Dieting
        dietDays = 0            # Max days to diet
        dietGoal = 0            # Goal weight to stop
        dietImmob = False       # Immobile diet sequence in progress
        dietStartKg = 0         # Initial weight at diest start
        dietWill = 0            # Dieting willpower
        dietHard = False        # True to enforce dieting
        rawMealsize = 0         # Mealsize before diet control
        cheatDay = False        # It's a cheat day now
        dietCount = 0           # Number of diets started
        meals = 0               # Meals eaten today
        glutton = False         # PC has become a glutton
        gluttonRdy = False      # PC ready to become a glutton
        gsize = 0               # Graphical size of pc
        gsizeMax = 11           # Upper gsize limit
        money = 5               # Money owned
        energy = 10             # Energy to do things
        kink = 0                # Kinkyness trait 0..100
        stuffing = None         # Stuffing trait None, 0..100
        mood = 50               # Mood 0..100
        moodStr = "okay"        # Mood name
        weightKg = 40           # Weight in kg
        weightLb = 88           # Weight in lbs
        weightMaxKg = 50        # Maximum weight in kg
        weightPeak = False      # Current weight was new maximum
        lastScaleLb = 88        # Last measured weight in lbs
        lastWeightLb = "88?lb"  # Last weight
        lastWeightKg = "40?kg"  # Last weight in kg



        # ---------------------------------------------------------------------
        # Food and eating
        # - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -

        # How much to eat in one sitting in plates?
        # This is an approximation of 500cc plates of food.
        #
        # Result is determined by time of day, and what has already been consumed.
        # Greed, drunkeness can increase the result.
        # Mood and confidence can increase or decrease the result.
        #
        # For general prompt generation use changeMood False.
        # For a meal use changeMood True so dieting affects mood.
        # Dieting forces the result to 1 or 0.
        #
        # Min = 0
        # Max = 15(7.5*2) + 3(gf) + 2(df) + 2(cf) + 2(mf) = 24 (12l/26.4lbs)
        #
        def getMealSize(self, changeMood=False):
            # Start with full limit (litres) which depends on fatness
            #
            fullLim = self.getFullLimit()
            fl = fullLim
            reason = "fulLim={:0.1f}".format(fullLim)

            # Use full factor to adjust full limit based on time of day.
            #
            ff = self.getFullFactor()
            fullLim *= ff
            reason += " ff={:0.2f} tgtVol={:0.1f}".format(ff, fullLim)

            # Subtract volume eaten already.
            # This produces a negativve result if overfull.
            #
            result = fullLim - self.full
            reason += " vol={:0.1f}".format(result)

            # Adjust negative volumes towards zero.
            #
            if result < 0:
                result *= 0.25
                reason += ",adjNeg={:0.1f}".format(result)

            # Convert to plates.
            #
            result *= 2
            reason += "/plates={:0.1f}".format(result)

            # Greed can add up to 3 plates, 4 for a glutton.
            #
            if self.glutton:
                gf = 4.0 * self.greed / 100.0
            else:
                gf = 3.0 * self.greed / 100.0
            reason += " greed+{:0.1f}".format(gf)
            result += gf

            # Drunkeness can add up to 2 plates.
            #
            df = 2.0 * min(1.0, self.drunk/12.0)
            reason += " drunk+{:0.1f}".format(df)
            result += df

            # Confidence can add or subtract 2 plates
            #
            cf = 2.0 * max(-1,min(1,((self.fatLim - self.fat) / 50.0)))
            reason += " conf{:+0.1f}".format(cf)
            result += cf

            # Mood can add 2 or subtract 2 plates.
            #
            if self.mood >= 60:
                mf = (self.mood - 60) / 20.0
                reason += " happy{:+0.1f}".format(mf)
            elif self.mood >= 40:
                mf = 0
            elif self.mood >= 20:
                mf = (10 - abs(self.mood - 30)) / 10.0
                reason += " unhappy{:+0.1f}".format(mf)
            else:
                mf = 2 * (20 - self.mood) / 20.0
                reason += " misery{:+0.1f}".format(mf)
            result += mf

            # Stuffing can add up to 4 plates.
            #
            if self.stuffing != None:
                sf = 4.0 * self.stuffing / 100.0
                reason += " stuff{:+0.1f}".format(sf)
                result += sf

            # Capacity can reduce by one fifth.
            #
            cf = (self.full - fl)/fl
            # reason += " fullness={:0.2f}".format(cf)
            if cf > 0.0:
                cf = 1.0 - 0.2 * min(1, cf)
                result *= cf
                reason += " cap*{:0.2f}".format(cf)

            # Checks and quantisation.
            #
            if result < 0:
                result = 0
                reason += " neg becomes 0"
            elif result < 0.4:
                result = 0
                reason += " small becomes 0"
            elif result < 0.6:
                reason += " frac becomes 1"
                result = 1
            else:
                result = int(round(result))

            self.rawMealsize = result
            reason += " raw={:0.2f}".format(result)

            # Diet can limit result to one.
            # If changeMood is true, subtract 1 from mood.
            #
            if self.diet and not self.cheatDay and result > 1:
                result = 1          # eat minimum amount
                reason += " but dieting so 1"
                if changeMood:
                    self.addMood(-1)    # having to diet leads to more unhappiness

            if debug:
                dbg("getMealSize({})={} {}".format(changeMood, result, reason))

            return result

        # Eat a meal.
        # fat: fat gain in kg
        # full: full gain in litres
        # Increments meal count.
        #
        def eatMeal(self, fat, full):
            self.meals += 1
            self.fat = max(0, self.fat + fat)
            self.full = max(0, self.full + full)
            self.updateWeight()
            if debug:
                dbg("eatMeal({}, {}) Meals: {}".format(fat, full, self.meals))

        # Eat a snack.
        # fat: fat gain in kg
        # full: full gain in litres
        # Does not change meal count.
        #
        def eatSnack(self, fat, full):
            self.fat = max(0, self.fat + fat)
            self.full = max(0, self.full + full)
            self.updateWeight()
            if debug:
                dbg("eatSnack({}, {})".format(fat, full))

        # Drink alcohol
        # fat: fat gain in kg
        # full: full gain in litres
        # drunk: units drunk
        # Does not change meal count.
        #
        def drink(self, fat, full, drunk):
            self.fat = max(0, self.fat + fat)
            self.full = max(0, self.full + full)
            self.drunk += drunk
            self.updateWeight()
            if debug:
                dbg("drink({}, {}, {})".format(fat, full, drunk))

        # Eat food.
        # Clears food variable.
        # (see class Food and food variable)
        #
        # food  - the food to eat and clear
        # scale - scale to apply (default 1)
        #
        def eatFood(self, food, scale=1.0):
            self.fat = max(0, self.fat + food.fat * scale)
            self.full = max(0, self.full + food.full * scale)
            self.drunk += food.drink * scale
            if food.meal:
                self.meals += 1
            if debug:
                dbg("eatFood({}, {}) meals={}".format(food.debug(), scale, self.meals))
            food.clear()
            self.updateWeight()

        # Eat food.
        # Does not clears food variable.
        # (see class Food and food variable)
        #
        # food  - the food to eat
        # scale - scale to apply (default 1)
        #
        def eatFoodNc(self, food, scale=1.0):
            self.fat = max(0, self.fat + food.fat * scale)
            self.full = max(0, self.full + food.full * scale)
            self.drunk += food.drink * scale
            if food.meal:
                self.meals += 1
            if debug:
                dbg("eatFoodNc({}, {}) meals={}".format(food.debug(), scale, self.meals))
            self.updateWeight()



        # ---------------------------------------------------------------------
        # Diet and weight
        # - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -

        # Clear the diet cheat day flag.
        # Restore delta will, but not above maxWill.
        #
        def dietClearCheat(self, delta=1, maxWill=80):
            if self.cheatDay:
                self.cheatDay = False
                self.dietWill = min(maxWill, self.dietWill + delta)

        # Start a diet.
        # days - maximum duration before abandon
        # goal - target weight to succeed
        #
        def dietStart(self, days, goalKg):
            self.diet = True
            self.dietWill = 90
            self.dietDays = days
            self.dietStartKg = self.weightKg
            self.dietGoal = goalKg
            self.cheatDay = False
            self.addMood(-20)
            self.dietCount += 1
            if debug:
                dbg("dietStart({}, {:0.1f}kg)".format(days, goalKg))

        # Stop a diet.
        #
        def dietStop(self):
            self.diet = False
            self.dietImmob = False
            self.dietDays = 0
            self.dietGoal = 0
            self.dietStartKg = 0
            self.dietWill = 0
            self.cheatDay = False
            self.addMood(20)

        # Test diet willpower
        # Returns true if still dieting.
        # Returns false and sets cheatDay if check failed.
        # Sets flag dietStopWill if diet ends
        #
        def testDietWill(self):
            if self.cheatDay:
                if debug:
                    dbg("testDietWill()=False cheatDay already")
                return False

            result = self.dietHard or self.randomBool(self.dietWill)

            if result:
                if self.dietWill > 80 and not self.dietHard:
                    self.dietWill -= 1
            else:
                self.decWill()

            if debug:
                dbg("testDietWill()={} dietHard={} dietWill={} cheatDay={}".format(result, self.dietHard, self.dietWill, self.cheatDay))

            return result

        # Reduce diet willpower and set cheat day.
        # If the pc is hard dieting, does nothing.
        # Sets flag dietStopWill and stops dieting if willpower reaches zero.
        #
        def decWill(self):
            if self.dietHard:
                return
            self.dietWill -= 10
            self.cheatDay = True
            if self.dietWill <= 0:
                self.dietStop()
                flags.set("dietStopWill")

        # Update player weight strings after stepping on scales.
        # Set weight peak if this is a new maximum weight
        #
        def updateLastWeight(self):
            self.lastWeightLb = "{:0,.0f}lb".format(self.weightLb)
            self.lastWeightKg = "{:0,.0f}kg".format(self.weightKg)

        # Update total weight
        #
        def updateWeight(self):
            self.weightKg = self.base + self.fat
            self.weightLb = 2.2 * self.weightKg



        # ---------------------------------------------------------------------
        # Exercise
        # - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -

        # Exercise.
        #
        # Type      Rate
        # light     300
        # moderate  500
        # hard      700
        # strenous  900
        #
        def exercise(self, rate, hours):
            cal = rate * hours
            fat = cal / kCalKg
            self.fat -= fat
            if self.fat < 0:
                self.fat = 0
            self.updateWeight()
            if debug:
                dbg("exercise({}, {}) used={:0.2f} fat={:0.2f}".format(rate, hours, fat, self.fat))

        # Walk or run a distance.
        #
        def walkRun(self, distKm):
            cal = 110 * distKm
            fat = cal / kCalKg
            self.fat -= fat
            if self.fat < 0:
                self.fat = 0
            self.updateWeight()
            if debug:
                dbg("walkRun({}) use={:0.4f} fat={:0.2f}".format(distKm, fat, self.fat))

        # Use energy.
        # Or restore it.
        #
        def useEnergy(self, delta):
            oldEnergy = self.energy
            self.energy = max(0, min(10, self.energy - delta))
            # Announce changes.
            #
            if isSelfVoicing() and oldEnergy != self.energy:
                if delta < 0:
                    alt("You feel invigorated and have {} energy.".format(self.energy))
                elif self.energy >= 5:
                    alt("You feel a little tired and have {} energy left.".format(self.energy))
                elif self.energy >= 2:
                    alt("You feel more tired and have {} energy left.".format(self.energy))
                elif self.energy == 1:
                    alt("You feel really tired and have almost no energy left.")
                else:
                    alt("You feel really tired and have no energy left.")


        # ---------------------------------------------------------------------
        # Money
        # - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -

        # Can a cost be afforded?
        #
        def canAfford(self, money, acc=0):
            return money <= self.money + acc

        # Can a scaled cost be afforded?
        #
        def canAffordScale(self, money, scale, acc=0):
            return money <= self.money * scale + acc

        # Can a food be afforded?
        #
        def canAffordFood(self, name, acc=0):
            tuple = food.getFood(name)
            if tuple is None:
                return false
            return tuple[3] + acc <= self.money

        # Can a food be afforded?
        #
        def canAffordFoodScale(self, name, scale, acc=0):
            tuple = food.getFood(name)
            if tuple is None:
                return false
            return tuple[3] * scale + acc <= self.money

        # Can a list of foods be afforded?
        #
        def canAffordFoodList(self, *names):
            acc = 0
            for name in names:
                tuple = food.getFood(name)
                if tuple is not None:
                    acc += tuple[3]
            return acc <= self.money

        # Earn money.
        #
        def earn(self, money):
            oldMoney = self.money
            self.money += money
            # Announce new total in self voicing mode.
            #
            if isSelfVoicing() and oldMoney != self.money:
                cents = int(round(self.money * 100))
                dollars = cents // 100
                cents = cents % 100
                if self.money == 0:
                    alt("You have no money left. You are broke.")
                elif dollars > 0:
                    if cents > 0:
                        alt("You now have {} dollars and {} cents.".format(dollars, cents))
                    else:
                        alt("You now have {} dollars.".format(dollars))
                else:
                    alt("You now have just {} cents to your name.".format(cents))

        def roundDown(self, value):
            return math.floor(value)

        # Add a tip to a cost and round up to whole amount.
        # If tip is zero (default) round up to nearest quarter.
        #
        def roundUp(self, cost, tipPer=0):
            if tipPer == 0:
                result = math.ceil(cost * 4) / 4.0
            else:
                result = cost + (cost * tipPer) / 100.0
                result = math.ceil(result)
            # if debug:
            #    dbg("roundUp({}, {})={}".format(cost, tipPer, result))
            return result

        # Spend money.
        #
        def spend(self, delta):
            oldMoney = self.money
            self.money -= delta
            if self.money < 0:
                self.money = 0
            # Announce new total in self voicing mode.
            #
            if isSelfVoicing() and oldMoney != self.money:
                cents = int(round(self.money * 100))
                dollars = cents // 100
                cents = cents % 100
                if self.money == 0:
                    alt("You have no money left. You are broke.")
                elif dollars > 0:
                    if cents > 0:
                        alt("You have {} dollars and {} cents left.".format(dollars, cents))
                    else:
                        alt("You have {} dollars left.".format(dollars))
                else:
                    alt("You now have just {} cents left to your name.".format(cents))



        # ---------------------------------------------------------------------
        # New day
        # - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -

        # Start a new day.
        #
        # May set flags:
        # dietStopGoal if target weight reached
        # dietStopDays if diet had elapsed with time
        #
        def newDay(self):
            self.energy = 10
            self.full = 0
            self.drunk = 0
            self.meals = 0
            self.cheatDay = False
            # self.fat -= 0.32
            fatBurn = self.getFatBmr()
            self.fat -= fatBurn

            if debug:
                dbg("pc.newDay(): fat burnt {:0.3f}kg".format(fatBurn))

            # Update weight at start of day.
            # Set the weightPeak flag.
            #
            if self.fat < 0:
                self.fat = 0
            self.updateWeight()

            if self.diet:
                if self.fat <= self.dietGoal - self.base:
                    self.dietStop()
                    flags.set("dietStopGoal")
                    self.addMood(10)

                    if debug:
                        dbg("pc.newDay() diet goal reached")
                elif self.dietHard:
                    if debug:
                        dbg("pc.newDay() still hard dieting")
                else:
                    self.dietDays -= 1

                    if self.dietDays <= 0:
                        self.dietStop()
                        flags.set("dietStopDays")

                        if debug:
                            dbg("pc.newDay() diet days reached")
                    else:
                        if debug:
                            dbg("pc.newDay() diet ongoing fat={:0.2f} weight={:0.2f} goal={:0.2f}".format(self.fat, self.weightKg, self.dietGoal))

        # Update peak weight flag.
        #
        def checkPeak(self):
            self.weightPeak = self.weightKg > self.weightMaxKg
            if self.weightPeak:
                self.weightMaxKg = self.weightKg


        # ---------------------------------------------------------------------
        # Randomness
        # - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -

        # Pick a random number between low and high, inclusive.
        # Shortcut for renpy.random.randint with debug
        #
        def random(self, low, high):
            result = renpy.random.randint(low, high)

            if debugM:
                dbg("random({},{})={}".format(low, high, result))

            return result

        # Random chance, higher percentages (0.0 .. 100.0) result in more True results.
        # A percent of zero or less is always false.
        # A percent of 100 or more is always true.
        #
        def randomBool(self, percent):
            rvalue = 80085
            if percent <= 0.0:
                result = False
            elif percent >= 100.0:
                result = True
            else:
                rvalue = renpy.random.random()
                result = rvalue <= (percent / 100.0)

            if debugM:
                dbg("randomBool({:0.1f})={} {:0.3f}".format(percent, result, rvalue))

            return result

        # Random chance based off a stat.
        #
        # stat      - the stat to test
        # lowLim    - lower limit below which chance is zero
        # highLim   - upper limit where chance becomes prob
        # prob      - max probability of success 0..100 (default 100)
        #
        # Always returns False if stat below lowLim.
        # Returns True prob percent of the time if stat above highLim.
        # Probability proportional to where the stat lies between limits otherwise.
        #
        def randomBoolStat(self, stat, lowLim, highLim, prob=100):
            chance = (stat - lowLim) * float(prob) / (highLim - lowLim)
            result = pc.randomBool(chance)
            if debug:
                dbg("randomBoolStat({}, {}, {}, {})={} chance={:0.1f}".format(stat, lowLim, highLim, prob, result, chance))
            return result

        # Random chance of at least min, but rising to max with max greed.
        #
        # min   - minimum percentage 0..100
        # max   - maximum percentage 0..100
        #
        def randomGreed(self, min, max):
            chance = min + (max - min) * self.greed / 100.0
            result = pc.randomBool(chance)

            if debug:
                dbg("randomGreed({},{})={} chance={:0.1f}".format(min, max, result, chance))
            return result

        # Random chance of player being greedy.
        #
        # lowLim    - lower greed limit below which chance is zero
        # highLim   - upper greed limit where chance becomes prob
        # prob      - max probability of success 0..100 (default 100)
        #
        # Returns True prob percent of the time if romance above highLim.
        # Probability proportional to where greed lies between limits otherwise.
        #
        def randomGreedStat(self, lowLim, highLim, prob=100):
            result = pc.randomBoolStat(self.greed, lowLim, highLim, prob)
            if debug:
                dbg("randomGreedStat({}, {}, {})={}".format(lowLim, highLim, prob, result))
            return result

        # Random chance of at least min, but rising to max with max kink.
        #
        # min   - minimum percentage 0..100
        # max   - maximum percentage 0..100
        #
        def randomKink(self, min, max):
            chance = min + (max - min) * self.kink / 100.0
            result = pc.randomBool(chance)
            if debug:
                dbg("randomKink({},{})={} chance={:0.1f}".format(min, max, result, chance))
            return result

        # Random chance of player being kinky.
        #
        # lowLim    - lower kink limit below which chance is zero
        # highLim   - upper kink limit where chance becomes prob
        # prob      - max probability of success 0..100 (default 100)
        #
        # Returns True prob percent of the time if romance above highLim.
        # Probability proportional to where kink lies between limits otherwise.
        #
        def randomKinkStat(self, lowLim, highLim, prob=100):
            result = pc.randomBoolStat(self.kink, lowLim, highLim, prob)
            if debug:
                dbg("randomKinkStat({}, {}, {})={}".format(lowLim, highLim, prob, result))
            return result

        # Random chance of at least min, but rising to max with increasing
        # happiness with weight (fatLim-fat).
        #
        # minp      - minimum percentage 0..100
        # maxp      - maximum percentage 0..100
        # spread    - spread in kg to achive max value
        #
        def randomLim(self, minp, maxp, spread=10.0):
            prop = max(-1, min(1, (self.fatLim - self.fat)/spread))
            delta = maxp - minp
            limAdj = delta * prop
            chance = minp + limAdj
            result = pc.randomBool(chance)
            if debug:
                dbg("randomLim({},{},{})={} prop={:0.1f}% delta={} limAdj={:0.2f}% chance={:0.1f}%".format(minp, maxp, spread, result, prop*100.0, delta, limAdj, chance))
            return result

        # Random chance of at least min, but rising to max with max stuffing.
        # When stuffing is off, always returns False.
        #
        # min   - minimum percentage 0..100
        # max   - maximum percentage 0..100
        #
        def randomStuffing(self, min, max):
            if self.stuffing == None:
                result = False
                chance = 0
            else:
                chance = min + (max - min) * self.kink / 100.0
                result = pc.randomBool(chance)
            if debug:
                dbg("randomStuffing({},{})={} chance={:0.1f}".format(min, max, result, chance))
            return result

        # Random chance of player wanting to be stuff herself.
        #
        # lowLim    - lower stuffing limit below which chance is zero
        # highLim   - upper stuffing limit where chance becomes prob
        # prob      - max probability of success 0..100 (default 100)
        #
        # Returns True prob percent of the time if romance above highLim.
        # Probability proportional to where stuffing lies between limits otherwise.
        #
        def randomStuffingStat(self, lowLim, highLim, prob=100):
            result = pc.randomBoolStat(self.stuffing, lowLim, highLim, prob)
            if debug:
                dbg("randomStuffingStat({}, {}, {})={}".format(lowLim, highLim, prob, result))
            return result


        # Random chance of at least min, but rising to max with max body weight.
        #
        # min   - minimum percentage 0..100
        # max   - maximum percentage 0..100
        #
        def randomWeight(self, min, max):
            wf = 100.0 * self.weightKg / self.massSoftLim
            if wf > 100:
                wf = 100
            chance = min + (max - min) * wf / 100.0
            result = pc.randomBool(chance)

            if debug:
                dbg("randomWeight({},{})={} chance={:0.1f}".format(min, max, result, chance))
            return result

        # Random chance of at least min, but rising to max with max body weight in a range.
        #
        # lowKg  - lowest weight: blow this will be min chance
        # highKg - highest weigh: above this will be max chance
        # minp   - minimum percentage 0..100
        # maxp   - maximum percentage 0..100
        #
        def randomWeightRangeKg(self, lowKg, highKg, minp=0, maxp=100):
            wf = (self.getMass() - lowKg) / (highKg - lowKg)
            wf = max(0.0, min(1.0, wf))
            chance = minp + (maxp - minp) * wf
            result = pc.randomBool(chance)

            if debug:
                dbg("randomWeightRangeKg({}, {}, {}, {})={} chance={:0.1f} wf={:0.3f}".format(lowKg, highKg, minp, maxp, result, chance, wf))
            return result

        # Random chance of at least min, but rising to max with max body weight in a range.
        #
        # lowLb - lowest weight: blow this will be min chance
        # highLb- highest weigh: above this will be max chance
        # min   - minimum percentage 0..100
        # max   - maximum percentage 0..100
        #
        def randomWeightRangeLb(self, lowLb, highLb, min=0, max=100):
            lowKg = lowLb / 2.2
            highKg = highLb / 2.2
            result = self.randomWeightRangeKg(lowKg, highKg, min, max)
            if debug:
                dbg("randomWeightRangeLb({}, {}, {}, {})={}".format(lowLb, highLb, min, max, result))
            return result



        # ---------------------------------------------------------------------
        # Text stuff
        # - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -

        # Adjective for butt size.
        #
        def adjArse(self):
            if self.gsize == 0:
                return "flat"
            if self.gsize == 1:
                return "small"
            if self.gsize == 2:
                return "soft"
            if self.gsize == 3:
                return "curvy"
            if self.gsize == 4:
                return "round"
            if self.gsize == 5:
                return "chubby"
            if self.gsize == 6:
                return "ample"
            if self.gsize == 7:
                return "phat"
            if self.gsize == 8:
                return "{alt}thick{/alt}{noalt}thicc{/noalt}"
            if self.gsize == 9:
                return "enormous"
            if self.gsize == 10:
                return "immense"
            if self.gsize == 11:
                return "shelf-like"
            if self.gsize <= 13:
                return "sofa-filling"
            else:
                return "bed-filling"

        # Adjective for belly size.
        #
        def adjBelly(self):
            if self.gsize == 0:
                return "hollow"
            if self.gsize == 1:
                return "flat"
            if self.gsize == 2:
                return "soft"
            if self.gsize == 3:
                return "round"
            if self.gsize == 4:
                return "plump"
            if self.gsize == 5:
                return "ample"
            if self.gsize == 6:
                return "large"
            if self.gsize == 7:
                return "weighty"
            if self.gsize == 8:
                return "huge"
            if self.gsize == 9:
                return "immense"
            if self.gsize == 10:
                return "pendulous"
            if self.gsize == 11:
                return "expansive"
            if self.gsize <= 13:
                return "vast"
            else:
                return "immobilising"
                

        # Adjective for boob size.
        #
        def adjBoob(self):
            if self.gsize == 0:
                return "small"
            if self.gsize == 1:
                return "perky"
            if self.gsize == 2:
                return "round"
            if self.gsize == 3:
                return "ample"
            if self.gsize == 4:
                return "large"
            if self.gsize == 5:
                return "generous"
            if self.gsize == 6:
                return "sizeable"
            if self.gsize == 7:
                return "hefty"
            if self.gsize == 8:
                return "heavy"
            if self.gsize == 9:
                return "voluminous"
            if self.gsize == 10:
                return "huge"
            if self.gsize == 11:
                return "head-sized"
            if self.gsize <= 13:
                return "monsterous"
            else:
                return "suffocating"

        # Adjective for stomach fullness.
        #
        def adjBloat(self):
            full = self.getFullness()

            if full <= 1:
                return "empty"
            if full <= 25:
                return "hungry"
            if full <= 75:
                return "happy"
            if full <= 100:
                return "sated"
            if full <= 150:
                return "gorged"
            if full <= 200:
                return "overladen"
            return "painfully overstuffed"

        # Adjust prompt based on time of day and meals eaten.
        #
        # prompt        - original prompt
        # hour          - hour of day, zero to not mention mealtimes
        # inMealsize    - mealsize, or None to avoid mentioning hunger unless dieting
        # acc           - plates of food already eaten since getMealSize called
        #
        # inMealsize is passed in to avoid multiple evaluations.
        #
        def appendPrompt(self, prompt, hour, inMealsize, acc=0):
            global mealsize
            if debugM:
                dbg("appendPrompt('{}', {}, {}, {}) mealsize={}".format(prompt, hour, inMealsize, acc, mealsize))
            # For accessibility, indicate this is a menu.
            prompt = "{alt}Menu. {/alt}" + prompt
            if hour >= 12 and hour < 16 and self.meals < 2:
                # Lunch time and not two meals.
                if inMealsize != None:
                    if inMealsize == 0:
                        prompt += " You aren't really hungry, but it is lunchtime."
                    else:
                        if inMealsize <= 4:
                            prompt += " Some lunch might be a good idea."
                            coJoin = " as"
                        else:
                            prompt += " It's lunchtime."
                            coJoin = " and"
                        if self.diet:
                            prompt = joinNotNone(prompt, coJoin, self.dietHungerPhrase(acc))
                            if self.cheatDay:
                                prompt = joinNotNone(prompt, ", and", self.dietPhrase())
                            else:
                                prompt = joinNotNone(prompt, ", but", self.dietPhrase())
                            prompt = appendNotNone(prompt, self.tumNoisePhrase())
                        else:
                            if inMealsize < 2:
                                coJoin = ", but"
                            prompt = joinNotNone(prompt, coJoin, self.hungerPhrase())
                            prompt = appendNotNone(prompt, self.tumNoisePhrase())
                else:
                    prompt += " Some lunch might be a good idea."
                    if self.diet:
                        mealsize = self.getMealSize()
                        prompt = joinNotNone(prompt, " as", self.dietHungerPhrase(acc))
                        prompt = appendNotNone(prompt, self.tumNoisePhrase())
            elif hour >= 18 and hour <= 23 and self.meals < 3:
                # Dinner time and not had three meals.
                if inMealsize != None:
                    if inMealsize == 0:
                        prompt += " You aren't really hungry, but it is dinnertime."
                    else:
                        if inMealsize <= 4:
                            prompt += " Some dinner might be a good idea."
                            coJoin = " as"
                        else:
                            prompt += " It's dinnertime."
                            coJoin = " and"
                        if self.diet:
                            prompt = joinNotNone(prompt, coJoin, self.dietHungerPhrase(acc))
                            if self.cheatDay:
                                prompt = joinNotNone(prompt, ", and", self.dietPhrase())
                            else:
                                prompt = joinNotNone(prompt, ", but", self.dietPhrase())
                            prompt = appendNotNone(prompt, self.tumNoisePhrase())
                        else:
                            if inMealsize < 2:
                                coJoin = ", but"
                            prompt = joinNotNone(prompt, coJoin, self.hungerPhrase())
                            prompt = appendNotNone(prompt, self.tumNoisePhrase())
                else:
                    prompt += " Some dinner might be a good idea."
                    if self.diet:
                        mealsize = self.getMealSize()
                        prompt = joinNotNone(prompt, " as", self.dietHungerPhrase(acc))
                        prompt = appendNotNone(prompt, self.tumNoisePhrase())
            elif inMealsize != None:
                if self.diet:
                    if self.cheatDay:
                        prompt = appendNotNone(prompt, self.mealsizePhrase(acc))
                        prompt = joinNotNone(prompt, " as", self.dietHungerPhrase(acc))
                        prompt = joinNotNone(prompt, ", and", self.dietPhrase())
                    else:
                        prompt = appendNotNone(prompt, self.dietHungerPhrase(acc))
                        prompt = joinNotNone(prompt, ", but", self.dietPhrase())
                else:
                    prompt = appendNotNone(prompt, self.mealsizePhrase(acc))
                    coJoin = " as"
                    if inMealsize < 2:
                        coJoin = ", but"
                    prompt = joinNotNone(prompt, coJoin, self.hungerPhrase(acc))
                prompt = appendNotNone(prompt, self.tumNoisePhrase(acc))
            elif self.diet:
                mealsize = self.getMealSize()
                prompt = appendNotNone(prompt, self.dietHungerPhrase(acc))
                prompt = appendNotNone(prompt, self.tumNoisePhrase(acc))

            if debugM:
                dbg("appendPrompt: result={}".format(prompt))
            return prompt

        # Describe player diet.
        # Produces non-null result when self.diet is true.
        #
        def dietPhrase(self):
            result = None
            if self.dietHard:
                result= "You need to stay mobile and are serious about dieting."
            elif self.diet:
                if self.cheatDay:
                    result = "It is, officially, a cheat day."
                elif self.dietWill >= 75:
                    result = "You are on a diet."
                elif self.dietWill >= 50:
                    result = "You are trying to diet."
                elif self.dietWill >= 25:
                    result = "You are sort of trying to diet."
                else:
                    result= "Your resolve to diet is fading."
            return result

        # Describe dieting player's hunger.
        # Valid after call to getMealSize.
        #
        def dietHungerPhrase(self, acc=0):
            result = None
            if acc == 0:
                adjSize = self.rawMealsize
            else:
                adjSize = self.rawMealsize - acc + 1
            if adjSize >= 14:
                result = "All you can think about is food."
            elif adjSize >= 10:
                result = "You are maddeningly hungry."
            elif adjSize >= 6:
                result = "Hunger is gnawing away at you."
            elif adjSize >= 4:
                result = "You are so very hungry."
            elif adjSize >= 2:
                result = "You are so hungry."
            elif adjSize == 1:
                result = "You are hungry."
            return result

        # Describe a greedy girl.
        #
        # altAdj    - optional additional adjective
        #
        def greedyGirlPhrase(self, altAdj=None):
            result = None
            if altAdj:
                if self.greed < 25:
                    result = "{} girl".format(altAdj)
                elif self.greed < 50:
                    result = "{} greedy girl".format(altAdj)
                elif self.greed < 75:
                    result = "very greedy {} girl".format(altAdj)
                else:
                    result = "really greedy {} girl".format(altAdj)
            else:
                if pc.greed < 25:
                    result = "girl"
                elif pc.greed < 50:
                    result = "greedy girl"
                elif pc.greed < 75:
                    result = "very greedy girl"
                else:
                    result = "really greedy girl"
            return result

        # Describe player hunger.
        # Valid after call to getMealSize.
        #
        def hungerPhrase(self, acc=0):
            result = None
            if acc == 0:
                adjSize = mealsize
            else:
                adjSize = mealsize - acc + 1
            if adjSize <= 0:
                result = "You aren't actually hungry."
            elif adjSize == 1:
                result = "You aren't really hungry."
            elif adjSize == 2:
                result = "You are a little hungry."
            elif adjSize == 3:
                result = "You are quite hungry."
            elif adjSize <= 5:
                result = "You are hungry."
            elif adjSize <= 8:
                result = "You are really hungry."
            elif adjSize <= 12:
                result = "You are so very hungry."
            elif adjSize <= 16:
                result = "You are so very, very hungry."
            else:
                result = "Your hunger knows no bounds."
            return result

        # How the player moves, for flavour.
        #
        def locoVerb(self, showDrunk=False):
            result = "walk"
            adverb = None
            if self.gsize >= 15:
                result = "waddle"
                adverb = " arduously"
            elif self.gsize >= 12:
                result = "waddle"
                adverb = " ponderously"
            elif self.gsize >= 9:
                result = "waddle"
            elif self.gsize >= 6:
                result = "wobble"
            elif self.gsize >= 3:
                result = "jiggle"
            if showDrunk: # uncertaint, unsteady, uncoordinated
                if self.drunk > 10:
                    adverb = " uncoordinatedly"
                elif self.drunk > 7:
                    adverb = " unsteadily"
                elif self.drunk > 3:
                    adverb = " uncertainly"
            if adverb:
                result += adverb
            return result

        # Describe player meal size estimate.
        # Valid after call to getMealSize.
        #
        def mealsizePhrase(self, acc=0):
            result = None
            if acc == 0:
                adjSize = self.rawMealsize
            else:
                adjSize = self.rawMealsize - acc + 1
            if adjSize <= 0:
                result = "You couldn't eat anything."
            elif adjSize == 1:
                result = "You could eat a little something."
            elif adjSize == 2:
                result = "You could manage a bite to eat."
            elif adjSize == 3:
                result = "You could eat."
            elif adjSize <= 5:
                result = "You could eat well."
            elif adjSize <= 8:
                result = "You could eat plenty."
            elif adjSize <= 12:
                result = "You could eat a lot."
            elif adjSize <= 16:
                result = "You could eat an awful lot."
            else:
                result = "You could eat a horse."
            if debugM:
                dbg("mealsizePhrase({})={} adjSize={}".format(acc, result, adjSize))
            return result

        # Describe player tummy noises.
        # Valid after call to getMealSize.
        #
        def tumNoisePhrase(self, acc=0):
            result = None
            if acc == 0:
                adjSize = self.rawMealsize
            else:
                adjSize = self.rawMealsize - acc + 1
            if adjSize > 15:
                result = "Your tummy growls loudly."
            elif adjSize > 10:
                result = "Your tummy growls."
            elif adjSize > 6:
                result = "Your tummy rumbles."
            elif adjSize > 3:
                result = "Your tummy gurgles."
            return result


        # ---------------------------------------------------------------------
        # Accessors
        # - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -

        # Add to greed.
        # Limit value to 0..100
        # Set gluttonRdy if mood is also 100
        #
        def addGreed(self, greed):
            self.greed = max(0, self.greed + greed)
            if self.greed >= 99.999:
                self.greed = 100
                self.gluttonRdy |= self.mood >= 99.99
                # dbg("greed={} mood={} ready={}".format(self.greed, self.mood, self.gluttonRdy))
            if debug:
                dbg("addGreed({})={:0.1f}".format(greed, self.greed))

        # Add to kinkyness.
        # Limit value to 0..100.
        #
        def addKink(self, delta):
            self.kink = max(0, min(100, self.kink + delta))

        # Add to fat limit.
        # Limit can't increase while hard dieting.
        # Limit can't fall below 10kg or go above massHardLim.
        #
        # delta     - the change to make
        # maxLim    - upper limit to apply to this increase (optional)
        #
        def addLim(self, delta, maxLim=None):
            if maxLim == None:
                maxLim = self.massHardLim
            if delta < 0:
                self.fatLim += delta
            elif not self.dietHard:
                newLim = min(maxLim, self.fatLim + delta)
                self.fatLim = max(self.fatLim, newLim)
            self.fatLim=max(10, self.fatLim)
            if debug:
                dbg("addLim({}, {}) = {:0.2f}".format(delta, maxLim, self.fatLim))

        # Add to mood value.
        # Limit value to 0..100
        # Set gluttonRdy if greed is also 100
        #
        def addMood(self, delta):
            oldMood = self.mood
            oldMoodStr = self.moodStr
            self.mood = max(0, self.mood + delta)
            if self.mood >= 99.999:
                self.mood = 100
                self.gluttonRdy |= self.greed >= 99.99
            self.quantMood()
            if debug:
                dbg("addMood({})={},{}".format(delta, self.mood, self.moodStr))
            # Announce any mood change.
            #
            if isSelfVoicing() and oldMoodStr != self.moodStr:
                if delta > 0:
                    if self.mood > 50:
                        alt("Your mood improves and you feel {}.".format(self.moodStr))
                    else:
                        alt("Your mood improves but you still feel {}.".format(self.moodStr))
                else:
                    if self.mood > 50:
                        alt("Your mood deteriorates but you still feel {}.".format(self.moodStr))
                    else:
                        alt("Your mood deteriorates and you feel {}.".format(self.moodStr))
                        

        # Add to stuffing trait when stuffing is enabled.
        # Limit value to 0..100.
        #
        # delta     - the vlue to add or subtract
        # maxLim    - upper limit, default 100
        #
        def addStuffing(self, delta, maxLim=100):
            if self.stuffing == None:
                return
            self.stuffing = max(0, min(100, min(maxLim, self.stuffing + delta)))

        # Clear down the glutton ready flag.
        #
        def clearGluttonRdy(self):
            self.gluttonRdy = False

        # Get fractional age.
        # Eg. half a year into the game this is 22.5
        #
        def getAgeFrac(self):
            return self.ageBase + gt.getYearFrac()

        # Get calories used per kg of body mass.
        # If the bmrMassFac is None or weight is below 350kg/770lb use the normal calculation.
        # Else if bmrMassFac is set and weight is above 450kg/990lb use the bmrMassFac value.
        # Else interpolate between the regular value and the set value.
        #
        def getBmrMassFac(self):
            result = bmrMassFac     # From const.rpy
            if self.bmrMassFac:
                if self.weightKg >= 450:
                    result = self.bmrMassFac
                elif self.weightKg >= 350:
                    prop = (self.weightKg-350.0)/100.0
                    result = bmrMassFac + (self.bmrMassFac - bmrMassFac) * prop
            if debugM:
                dbg("getBmrMassFac() {:0.2f}".format(result))
            return result

        # Get body fat percentage
        #
        def getBodyFatPer(self):
            result = (100 * self.fat)/(self.base + self.fat)
            if debugM:
                dbg("getBodyFatPer() {:0.1f}".format(result))
            return result

        # Get body fat percentage for a given mass
        #
        def getBodyFatPerAt(self, mass):
            fat = mass - self.base
            result = (100 * fat)/mass
            if debug:
                dbg("getBodyFatPerAt({:0.1f}kg/{:0.1f}lb)={:0.1f} ".format(mass, mass * 2.2, result))
            return result

        # Get remaining capacity, up to full.
        # Returns minCap if result is less than this.
        #
        # minCap        - minimum result
        #
        def getCapacityRemain(self, minCap=0.5):
            result = self.getFullLimit() - self.full
            if result < minCap:
                result = minCap
            if debug:
                dbg("getCapacityRemain({})={:0.2f}".format(minCap, result))
            return result

        # Get remaining capacity in litres, up to full for the time of day.
        # Returns minCap if result is less than this.
        #
        # minCap    - minimum capacity to return, default 0
        # tgt       - target stuffing, default 1.0 (100%)
        #
        def getCapacityRemainNow(self, minCap=0.0, tgt=1.0):
            fl = self.getFullLimit()
            ff = self.getFullFactor()
            fm = fl * ff * tgt
            fr = fm - self.full
            result = max(minCap, fr)
            #result = self.getFullLimit() * self.getFullFactor() - self.full
            #result = max(0.0, min(minCap, result))
            if debug:
                dbg("getCapacityRemainNow({:0.1f},{:0.1f})={:0.3f} fl={:0.2f} ff={:0.2f} fm={:0.2f} fr={:0.2f}".format(minCap, tgt, result, fl, ff, fm, fr))
            return result

        # How many times has the PC started a diet?
        #
        def getDietCount(self):
            return self.dietCount

        # Get the weight loss target in kg.
        # This is the difference between the start weight and goal weight.
        #
        def getDietDelta(self):
            return self.dietStartKg - self.dietGoal

        # Get diet progress as a percentage of weight loss goal.
        # Only valid while diet is active.
        #
        # Normal range is 0..100, but may go negative is weight is gained,
        # and over 100% until pc recognises diet end.
        #
        def getDietProgress(self):
            result = 0
            if self.diet and self.dietGoal > 0:
                lost = self.dietStartKg - self.getMass()
                result = lost * 100.0 / self.getDietDelta()
            if debug:
                dbg("getDietProgress()={}".format(result))
            return result

        # Get the number of days of dieting to reach a goal weight.
        #
        def getDaysToDietTo(self, goalKg):
            kgIn = 0.14         # 1100kCal
            kgOut = (0.18 + self.getFatBmr()) / 2.0
            kgDelta = self.getMass() - goalKg
            kgDay = kgOut - kgIn
            if kgDay > 0.1:
                days = kgDelta / kgDay
            else:
                days = kgDelta * 6.6
            days = int(math.ceil(days))
            if debug:
                dbg("getDaysToDiet({:0.2f}kg)={} kgOut={:0.2f} kdDay={:0.2f} kgDelta={:0.1f}".format(goalKg, days, kgOut, kgDay, kgDelta))
            return days

        # Calculate how much body fat is burned per day by the player character.
        #
        def getFatBmr(self):
            kcal = self.getBmrMassFac() * self.weightKg
            kcal += bmrHeightFac * self.heightCm
            kcal += bmrAgeFac * self.getAgeFrac()
            kcal += bmrBaseConst
            mass = kcal / kCalKg
            return mass

        # Get the player's fat limit.
        #
        def getFatLim(self):
            return self.fatLim

        # Get the difference between the current fat limit and fat stored in kg.
        # Positive values indicate PC happyto gain.
        #
        def getFatLimDelta(self):
            return self.fatLim - self.fat

        # Get the current fat carried in kg.
        #
        def getFatMass(self):
            return self.fat

        # Get progress towards (or beyond) fat limit.
        # A result of 100 is at their fat limit.
        #
        def getFatProgress(self):
            result = 100.0 * self.fat / self.fatLim
            return result

        # Determine what proportion of their capacity the pc will be willing
        # to consume by hour.
        #
        # This is a generally increasing function, starting at 6am, with steps
        # at 12:00 an 18:00. From 18:00 to 05:00 the result is 1.0.
        #
        def getFullFactor(self):
            hour = gt.hour
            result = 1.0
            if hour >=6 and hour < 12:
                result = 0.2 + 0.05 * (hour - 6)
            elif hour >=12 and hour < 18:
                result = 0.6 + 0.05 * (hour - 12)
            #if debugM: (needs to not generate debug as getFullnessNow is in debug screen)
                #dbg("getFullFactor()={:0.2f} at {}".format(result, hour))
            return result

        # Get fullness limit in litres.
        #
        # 0:    3
        # 1:    3
        # 2:    3       Typical over consumption
        # 3:    3.5
        # 4:    4
        # 5:    4.5
        # 6:    5
        # 7:    5.5
        # 8:    6
        # 9:    6.5
        # 10:   7
        # 11:   7.5     WR eating contest contender
        # 12:   8
        #
        def getFullLimit(self):
            if self.gsize < 3:
                result = 3
            elif self.weightLb >= 800:
                result = 8
            elif self.gsize >= 3:
                result = 3 + (self.gsize - 2) / 2
#            if debugM:
#                dbg("getFullLimit()={}".format(result))
            return result

        # Get fullness as a percentage of the whole day 0..100++
        #
        def getFullness(self):
            result = self.full * 100 / self.getFullLimit()
#            if debugM:
#                dbg("getFullness: {}".format(result))
            return result

        # Get fullness as a percentage of the capacity the PC has now 0..100++
        #
        def getFullnessNow(self):
            capNow = self.getFullLimit() * self.getFullFactor()
            result = self.full * 100 / capNow
            return result

        # Get current kink level 0..100.
        #
        def getKink(self):
            return self.kink

        # Get height in cm
        #
        def getHeightCm(self):
            return self.heightCm

        # Get current body mass in kg.
        #
        def getMass(self):
            return self.base + self.fat

        # Get body mass in kg if PC reached their current fat limit.
        #
        def getMassFatLim(self):
            return self.base + self.fatLim

        # Get the body mass for a given body fat percentage.
        #
        def getMassForBfp(self, bfp):
            mass = self.base / (1 - bfp/100.0)
            if debug:
                dbg("getMassForBfp({:0.1f})={:0.1f}".format(bfp, mass))
            return mass

        # Get mass hard limit.
        #
        def getMassHardLim(self):
            return self.massHardLim

        # Get the highest mass in kg.
        #
        def getMassMaxKg(self):
            return self.weightMaxKg

        # Get the current mass as a proportion of the maximum mass.
        # This is a number between 0.0 and 1.0.
        #
        def getMassMaxProp(self):
            maxKg = max(self.weightMaxKg, self.weightKg)
            return self.weightKg / maxKg

        # Get mass soft limit.
        #
        def getMassSoftLim(self):
            return self.massSoftLim

        # Get the current graphical size.
        #
        def getSize(self):
            return self.gsize

        # Get the fractional size of the player.
        # This is a value between gsize and gsize+1 (inclusive).
        # Uses external function kgLookup.
        #
        def getSizeFrac(self):
            result = self.gsize + self.getSizeProgress(self.gsize)
            if debug:
                dbg("getSizeFrac()={:0.2f}".format(result))
            return result

        # Get the current graphical size.
        # This result is not limited by gsizeMax.
        # Uses external functions sizeLookup.
        #
        def getSizeUnlim(self):
            return sizeLookup(pc.getMass())

        # Get the fractional size of the player.
        # This result is not limited by gsizeMax.
        # Uses external functions sizeLookup and kgLookup.
        #
        def getSizeFracUnlim(self):
            gsize = self.getSizeUnlim()
            result = gsize + self.getSizeProgress(gsize)
            if debug:
                dbg("getSizeFracUnlim()={:0.2f}".format(result))
            return result

        # Get the progress within the current graphical size.
        # This is a number between 0.0 and 1.0.
        # Uses external function kgLookup.
        #
        # gsize - base graphical size
        #
        def getSizeProgress(self, gsize):
            lowKg = float(kgLookup(gsize - 1))
            maxKg = float(kgLookup(gsize))
            massKg = float(self.getMass())
            frac = max(0.0, min(1.0, (massKg-lowKg)/(maxKg-lowKg)))
            if debug:
                dbg("getSizeProgress()={:0.2f}".format(frac))
            return frac

        # Get current stuffing level 0..100.
        #
        def getStuffing(self):
            if self.stuffing == None:
                return 0
            return self.stuffing

        # Is the PC having a cheat day?
        #
        def isCheatDay(self):
            return self.diet and self.cheatDay

        # Is PC dieting? Also true on cheat days.
        #
        def isDiet(self):
            return self.diet

        # Does the pc think they need to diet?
        #
        # deltaKg - adjustment to PCs mass
        #
        def isDietConsider(self, deltaKg=0):
            result = self.fat + deltaKg > self.fatLim
            if debugM:
                dbg("isDietConsider({:0.2f})={}".format(deltaKg, result))
            return result

        # Has the PC dieted at least this many times before?
        # If count is None, have they dieted before?
        #
        def isDietCount(self, count=None):
            if count == None:
                count = 1
            return self.dietCount >= count

        # Is the PC hard dieting?
        #
        def isDietHard(self):
            return self.dietHard

        # Is the PC dieting to regain mobility?
        # Set by diet hard, cleared by diet end.
        #
        def isDietImmob(self):
            return self.dietImmob

        # Is the pc restricting their diet?
        # Dieting and not a cheat day.
        #
        def isDietRestricted(self):
            return self.diet and not self.cheatDay

        # Is the pc not dieting or on a cheat day?
        #
        def isDietUnrestricted(self):
            return self.cheatDay or not self.diet

        # Is the pc as drunk or more than a threshold?
        #
        # thresh    - threshold (4 tipsy, 8-drunk)
        #
        def isDrunk(self, thresh=4):
            return self.drunk >= thresh

        # Is the pc fuller than a threshold?
        #
        # thresh    - threshold 0..100++
        #
        def isFull(self, thresh):
            return self.getFullness() >= thresh

        # Is the pc fuller than a threshold now?
        #
        # thresh    - threshold 0..100++
        #
        def isFullNow(self, thresh):
            return self.getFullnessNow() >= thresh

        # Has the PC become a glutton?
        #
        def isGlutton(self):
            return self.glutton

        # Is the PC ready to become a glutton?
        #
        def isGluttonRdy(self):
            return self.gluttonRdy and not self.glutton and self.greed >= 95 and self.mood >= 95

        # Test greediness.
        #
        def isGreedy(self, thresh=None):
            if thresh == None:
                return self.greed > 0
            return self.greed >= thresh

        # Has the PC exceeded the hard limit?
        #
        def isHardLim(self):
            return self.weightKg > self.massHardLim

        # Test kinkyness.
        #
        def isKink(self, thresh=None):
            if thresh == None:
                return self.kink > 0
            return self.kink >= thresh

        # Test mood.
        # True if mood is greater than or equal to value
        #
        def isMood(self, thresh):
            return self.mood >= thresh

        # Has the PC exceeded the soft limit?
        #
        def isSoftLim(self):
            return self.weightKg > self.massSoftLim

        # Has the PC developed the stuffing trait?
        # True if stuffing is greater than or equal to value
        #
        def isStuffing(self, thresh = None):
            if thresh == None:
                return self.stuffing != None
            return self.stuffing >= thresh

        # Is the PC at her highest weight today?
        #
        def isWeightPeak(self):
            return self.weightPeak

        # Does the PC have world record breaking hips?
        #
        def isWrHips(self):
            return self.weightKg > hipWrKg

        # Set or clear the hard diet flag.
        # Sets dietImmob if value is True.
        # Does not set the diet flag, that needs a call to dietStart().
        #
        def setDietHard(self, boolValue):
            self.dietHard = boolValue
            self.dietImmob |= boolValue

        # Lower fat lim to match given body mass.
        #
        def setFatLimLowerKg(self, massKg):
            oldLim = self.fatLim
            self.fatLim = min(self.fatLim, massKg - self.base)
            if debug:
                dbg("setFatLimLowerKg({:0.1f})={:0.1f} was {:0.1f}".format(massKg, self.fatLim, oldLim))

        # Raise fat lim to match given body mass.
        #
        def setFatLimRaiseKg(self, massKg):
            oldLim = self.fatLim
            self.fatLim = max(self.fatLim, massKg - self.base)
            if debug:
                dbg("setFatLimRaiseKg({:0.1f}kg/{:0.1f}lb)={:0.1f}kg/{:0.1f}lb was {:0.1f}kg/{:0.1f}lb".format(massKg, massKg*2.2, self.fatLim, self.fatLim*2.2, oldLim, oldLim*2.2))

        # Set whether or not the PC is a glutton.
        # If clearing the flag also clear gluttonRdy so PC has to max mood and
        # greed again.
        #
        def setGlutton(self, boolValue):
            self.glutton = boolValue
            self.gluttonRdy &= boolValue

        # Adjust the bmr calculation so that PC will plateau at a given weight
        # more or less.
        #
        # massKg        - plateau mass (None to reset)
        # stuffed       - player typical stuffed state
        # foodFatVol    - volume of food that is fat
        #
        def setMassPlateau(self, massKg=None, stuffed=2.1, foodFatVol=0.25):
            if massKg == None:
                self.bmrMassFac = None
                if debug:
                    dbg("setMassPlateau(None, {:0.2f}, {:0.2f})=None".format(stuffed, foodFatVol))
            else:
                kCalIn = 8.0 * stuffed * foodFatVol * kCalKg
                kCalUsed = bmrAgeFac * self.getAgeFrac() + bmrHeightFac * self.heightCm + bmrBaseConst
                kCalDelta = kCalIn - kCalUsed
                self.bmrMassFac = kCalDelta / massKg
                if debug:
                    dbg("setMassPlateau({:0.1f}, {:0.2f}, {:0.2f})={:0.2f} kcalIn={:0.1f} kCalUsed={:0.1f} kCalDelta={:0.1f}".format(massKg, stuffed, foodFatVol, self.bmrMassFac, kCalIn, kCalUsed, kCalDelta))
            return self.bmrMassFac

        # Set mood to a specific value 0..100
        #
        def setMood(self, value):
            self.mood = max(0, min(100, value))
            self.quantMood()

        # Set mood to the midpoint of a named mood.
        #
        def setMoodName(self, name):
            old = 0
            for mood in moods:
                # dbg("setMoodName: {} {}".format(mood[0], mood[1]))
                if name == mood[0]:
                    self.moodStr = name
                    self.mood = (old + mood[1])/2

                    if debug:
                        dbg("setMoodName({})={}".format(name, self.mood))

                    return
                else:
                    old = mood[1]

            dbg("setMoodName({}) mood not known".format(name))
            return

        # Set the pc name.
        # Default to Erin.
        #
        def setName(self, name):
            if name == None:
                self.name = "Erin"
            else:
                name = name.strip()
                if not name:
                    self.name = "Erin"
                else:
                    self.name = name.capitalize()

        # Enable or disable stuffing.
        # If stuffing is already enabled then leave it unchanged.
        #
        def setStuffingEnabled(self, boolValue):
            if not boolValue:
                self.stuffing = None
            elif self.stuffing == None:
                self.stuffing = 0



        # ---------------------------------------------------------------------
        # Internals
        # - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -

        # Quantify the mood value to the mood string.
        #
        def quantMood(self):
            for mood in moods:
                # dbg("quantMood: {} {}".format(mood[0], mood[1]))
                if self.mood < mood[1]:
                    self.moodStr = mood[0]
                    break
            if debugM:
                dbg("quantMood()={}".format(self.moodStr))


# The PC's mood descriptions and how they relate to her mood value.
#
define moods = [
        ('miserable', 15),
        ('unhappy', 30),
        ('meh', 45),
        ('okay', 60),
        ('fair', 75),
        ('happy', 90),
        ('blissful', 101)
    ]
# EoF
