#
# Game world.
#
init python:

    # The world class drives the main game loop.
    #
    # Provides methods for working out which renpy labels to call.
    # See placeIf.rpy
    #
    class World:
        homeLabel = "nwGas"     # Where the pc usually sleeps label
        atHome = False          # Treat where I am as home
        locLabel = "nwGas"      # The pc current location label
        workLabel = "nwGas"     # Where the pc currently works label
        workDays = [0,1,2,3,4]  # The days the pc works
        workStartHr = 9         # The hour work starts
        workEndHr = 17          # The hour work ends
        wakeHr = 8              # The hour to wake up

        # Part time work.
        #
        partWorkLabel = None    # Where the pc is temping label
        partWorkStartDay = 0    # Part time work start day number
        partWorkLastDay = 0     # Part time work last day number
        partWorkDays = None     # The days the pc is working part time, or None
        partWorkStartHr = 0     # The part time work start hour
        partWorkEndHr = 0       # The part time work end hour

        # Tempoary home.
        #
        tmpHomeLabel = None     # Temporary home label
        tmpHomeLastDay = 0      # Temporary home last day number
        tmpWakeHr = None        # The temporary wake time, or None

        # Temporary work.
        #
        tmpWorkLabel = None     # Where the pc is temping label
        tmpWorkStartDay = 0     # Temporary work start day number
        tmpWorkLastDay = 0      # Temporary work last day number
        tmpWorkDays = None      # The days the pc is temping, or None
        tmpWorkStartHr = 0      # The temp work start hour
        tmpWorkEndHr = 0        # The temp work end hour

        # Day control.
        #
        working = False         # Is pc working
        workLastLabel = None    # Last work job
        workOver = False        # Force work over for the day
        dayOver = False         # True when end of day process begins
        noReturn = False        # True to prevent return of player to home
        noSleep = False         # True to prevent normal sleeping

        # Endings.
        #
        epiLabel = None         # Epilogue label to call
        gameOver = False        # True on game over, exit main loop

        # Clear flags at start of day.
        #
        def newDay(self):
            self.working = False
            self.workLastLabel = None
            self.workOver = False
            self.dayOver = False
            self.noReturn = False
            self.noSleep = False

        # Clear last work label.
        #
        def clearLastWorkLabel(self):
            self.workLastLabel = None

        # Clear part time work.
        #
        def clearPartWork(self):
            self.partWorkLabel = None
            self.partWorkStartDay = 0
            self.partWorkLastDay = 0
            self.partWorkDays = None
            self.partWorkStartHr = 0
            self.partWorkEndHr = 0

        # Clear temporary home.
        #
        def clearTmpHome(self):
            self.tmpHomeLabel = None
            self.tmpHomeLastDay = 0
            self.tmpWakeHr = None

        # Clear temporary work.
        #
        def clearTmpWork(self):
            self.tmpWorkLabel = None
            self.tmpWorkStartDay = 0
            self.tmpWorkLastDay = 0
            self.tmpWorkDays = None
            self.tmpWorkStartHr = 0
            self.tmpWorkEndHr = 0

        # Get the current home location.
        #
        # Return the temporary home label if it is not None.
        # Otherwise return the home label.
        #
        def getCurHomeLoc(self):
            if self.tmpHomeLabel != None:
                return self.tmpHomeLabel
            return self.homeLabel

        # Get epilogue label.
        #
        def getEpilogueLabel(self):
            return self.epiLabel

        # Get the label used for player exhausted.
        # - try locLabel.pcExhausted
        # - use homeLabel.pcExhausted
        #
        def getExhaustedLabel(self):
            result = self.locLabel + ".pcExhausted"
            if not self.hasLabel(result):
                result = self.homeLabel + ".pcExhausted"
                if not self.hasLabel(result):
                    result = None
            return result

        # Get the label used for player free choice.
        # - try locLabel.freeChoice
        # - use homeLabel.freeChoice
        #
        def getFreeChoiceLabel(self):
            result = self.locLabel + ".freeChoice"
            if not self.hasLabel(result):
                result = self.homeLabel + ".freeChoice"
            if debugM:
                dbg("World.getFreeChoiceLabel()={}".format(result))
            return result

        # Get the home location label.
        #
        def getHomeLoc(self):
            return self.homeLabel

        # Get the current location label.
        #
        def getLoc(self):
            return self.locLabel

        # Get the label to call for part time work.
        #
        def getPartWorkLabel(self):
            result = None
            if self.partWorkLabel != None:
                result = self.partWorkLabel + ".workPart"
                self.workLastLabel = result
            if debugM:
                dbg("World.getPartWorkLabel()={}".format(result))
            return result

        # Get the label when returned home.
        #
        def getRetHomeLabel(self):
            result = self.homeLabel + ".retHome"
            if not self.hasLabel(result):
                result = None
            if debugM:
                dbg("World.getRetHomeLabel()={}".format(result))
            return result

        # Get the label to return home from current location.
        #
        # Returns .retHomeFromHere at current location if available, or None
        #
        def getRetHomeFromLabel(self):
            result = self.locLabel + ".retHomeFromHere"
            if not self.hasLabel(result):
                result = None
            if debug:
                dbg("World.getRetHomeFromLabel()={}".format(result))
            return result

        # Get the label to return home to home location.
        #
        # Returns .retHomeToHere for tmpHomeLabel if it is defined and label is available.
        # Returns .retHomeToHere at home location if available.
        # Returns None otherwise.
        #
        def getRetHomeToLabel(self):
            result = None
            if self.tmpHomeLabel != None:
                result = self.tmpHomeLabel + ".retHomeToHere"
                if not self.hasLabel(result):
                    result = None
            if result == None:
                result = self.homeLabel + ".retHomeToHere"
                if not self.hasLabel(result):
                    result = None
            if debugM:
                dbg("World.getRetHomeToLabel()={}".format(result))
            return result

        # Get the label to return home.
        # - try .retHomeFromHere at current location
        # - try .retHomeToHere at home location
        # - None if neither exists
        #
        def getRetToHomeLabel(self):
            result = self.locLabel + ".retHomeFromHere"
            if not self.hasLabel(result):
                result = self.homeLabel + ".retHomeToHere"
                if not self.hasLabel(result):
                    result = None
            if debugM:
                dbg("World.getRetToHomeLabel()={}".format(result))
            return result

        # Get the label to call on sleep.
        #
        def getSleepLabel(self):
            result = self.locLabel + ".sleep"
            if not self.hasLabel(result):
                result = self.homeLabel + ".sleep"
            if debugM:
                dbg("World.getSleepLabel()={}".format(result))
            return result

        # Get the temporary home label.
        # May be None.
        #
        def getTmpHomeLoc(self):
            return self.tmpHomeLabel

        # Get the label to call for temporary work.
        #
        def getTmpWorkLabel(self):
            result = None
            if self.tmpWorkLabel != None:
                result = self.tmpWorkLabel + ".workTmp"
                self.workLastLabel = result
            if debugM:
                dbg("World.getTmpWorkLabel()={}".format(result))
            return result

        # Get the label used for player up too late.
        # - try locLabel.pcTooLate
        # - use homeLabel.pcTooLate
        #
        def getTooLateLabel(self):
            result = self.locLabel + ".pcTooLate"
            if not self.hasLabel(result):
                result = self.homeLabel + ".pcTooLate"
                if not self.hasLabel(result):
                    result = None
            return result

        # Get the travel from here to location label if there is one.
        #
        def getTravelToLabel(self):
            result = self.locLabel + ".travelFromHereTo"
            if not self.hasLabel(result):
                result = None
            if debug:
                dbg("World.getTravelToLabel()={} loc={}".format(result, self.locLabel))
            return result

        # Get the time to wake up.
        #
        def getWakeHour(self):
            if self.tmpWakeHr != None:
                return self.tmpWakeHr
            return self.wakeHr

        # Get the label to call on wake.
        #
        def getWakeLabel(self):
            result = None
            if self.tmpHomeLabel != None:
                result = self.tmpHomeLabel + ".wakeTmp"
                if not self.hasLabel(result):
                    result = None
            if result == None:
                result = self.locLabel + ".wake"
                if not self.hasLabel(result):
                    result = self.homeLabel + ".wake"
            if debugM:
                dbg("World.getWakeLabel()={}".format(result))
            return result

        # Get the label to call for normal work.
        #
        def getWorkLabel(self):
            result = self.workLabel + ".work"
            self.workLastLabel = result
            if debugM:
                dbg("World.getWorkLabel()={}".format(result))
            return result

        # Get the label to call for end of work.
        #
        def getWorkEndLabel(self):
            result = None
            if self.workLastLabel != None:
                result = self.workLastLabel + "End"
                # dbg("World.getWorkEndLabel: trying {}".format(result))
                if not self.hasLabel(result):
                    result = None
            if debugM:
                dbg("World.getWorkEndLabel()={}".format(result))
            return result

        

        # Get the current work location.
        #
        # During temporary work time, this is the tmpWork label.
        # During part time work time, this is the partWork label.
        # Otherwise it is the regular work label.
        #
        def getWorkLoc(self):
            if self.tmpWorkLabel != None and self.isTmpWorkTime():
                return self.tmpWorkLabel
            if self.partWorkLabel != None and self.isPartWorkTime():
                return self.partWorkLabel
            return self.workLabel

        # Does a lable exist?
        #
        def hasLabel(self, labelName):
            return renpy.has_label(labelName)

        # Is today over?
        #
        def isDayOver(self):
            return self.dayOver

        # Has an epilogue been set?
        #
        def isEpilogue(self):
            return self.epiLabel != None

        # Is the game over?
        #
        def isGameOver(self):
            return self.gameOver

        # Is the pc at their home location?
        # If labelName is given, tests if that is their home location (not where they are)
        #
        def isHome(self, labelName=None):
            if self.atHome:
                return True
            if labelName != None:
                if self.tmpHomeLabel != None:
                    return self.tmpHomeLabel == labelName
                return self.locLabel == labelName
            if self.tmpHomeLabel != None:
                return self.locLabel == self.tmpHomeLabel
            return self.locLabel == self.homeLabel

        # Is the current location the given one?
        #
        def isLoc(self, labelName):
            return self.locLabel == labelName

        # Does the current location begin with the given partial label name?
        #
        def isLocLike(self, labelStart):
            return self.locLabel.startswith(labelStart)

        # Is the no return flag set?
        #
        def isNoReturn(self):
            return self.noReturn

        # Is the no sleep flag set?
        #
        def isNoSleep(self):
            return self.noSleep

        # Does the PC have a part time job?
        #
        def isPartWork(self):
            return self.partWorkLabel is not None

        # Is it a part time work day?
        #
        def isPartWorkDay(self):
            if self.partWorkLabel == None:
                return False
            dayNum = gt.dayNum
            if dayNum < self.partWorkStartDay:
                return False
            if self.partWorkLastDay != 0 and dayNum > self.partWorkLastDay:
                self.clearPartWork()
                return False
            return self.partWorkDays != None and gt.dayOw in self.partWorkDays

        # Is a given day of the week a part time work day?
        #
        # dayOw     - day of week 0..6 to check
        #
        def isPartWorkDayOfWeek(self, dayOw):
            return self.partWorkDays is not None and dayOw in self.partWorkDays

        # Is it time for part time work?
        #
        def isPartWorkTime(self):
            return self.isPartWorkDay() and gt.hour >= self.partWorkStartHr and gt.hour < self.partWorkEndHr and not self.workOver

        # Is it a temporary work day?
        #
        def isTmpWorkDay(self):
            if self.tmpWorkLabel == None:
                return False
            dayNum = gt.dayNum
            if dayNum < self.tmpWorkStartDay:
                return False
            if self.tmpWorkLastDay != 0 and dayNum > self.tmpWorkLastDay:
                self.clearTmpWork()
                return False
            return self.tmpWorkDays != None and gt.dayOw in self.tmpWorkDays

        # Is it time for temporary work?
        #
        def isTmpWorkTime(self):
            return self.isTmpWorkDay() and gt.hour >= self.tmpWorkStartHr and gt.hour < self.tmpWorkEndHr and not self.workOver

        # Is it a normal work day?
        #
        def isWorkDay(self):
            return gt.dayOw in self.workDays

        # Is workLastLabel defined?
        #
        def isWorkLastLabel(self):
            return self.workLastLabel != None

        # Is the pc at their work location?
        #
        def isWorkLoc(self):
            if self.isTmpWorkDay():
                return self.locLabel == self.tmpWorkLabel
            return self.locLabel == self.workLabel

        # Is it time for normal work?
        #
        def isWorkTime(self):
            return self.isWorkDay() and gt.hour >= self.workStartHr and gt.hour < self.workEndHr and not self.workOver

        # Is/was the PC working and workEnd description needs to occur?
        #
        def isWorking(self):
            return self.working

        # Set day over flag.
        # Causes main loop to end the day.
        #
        def setDayOver(self, over=True):
            self.dayOver = over

        # Define isEpilogue.
        #
        def setEpilogue(self, labelName):
            if labelName == None or self.hasLabel(labelName):
                self.epiLabel = labelName
            else:
                dbg("World.setEpilogue('{}'') - unknown label".format(labelName))

        # Set game over flag.
        # Causes main loop to return.
        #
        def setGameOver(self, over=True):
            self.gameOver = over

        # Set home location.
        #
        def setHome(self, homeLabel):
            self.homeLabel = homeLabel

        # Set the player location to a new one.
        #
        # loc       - the new location label
        # atHome    - treat this location as home
        #
        def setLoc(self, loc, atHome=False):
            if self.hasLabel(loc):
                self.locLabel = loc
                self.atHome = atHome
            else:
                dbg("World.setLoc({}) - unknown label".format(loc))

        # Set the player location to home.
        #
        def setLocHome(self):
            self.locLabel = self.homeLabel

        # Set the player location to their normal work location.
        #
        def setLocWork(self):
            self.locLabel = self.workLabel

        # Set no return flag.
        # Causes main loop to not make calls to describe the return to home.
        #
        def setNoReturn(self, flag=True):
            self.noReturn = flag

        # Set no sleep flag.
        # Causes main loop to not make calls to describe falling asleep.
        #
        def setNoSleep(self, flag=True):
            self.noSleep = flag

        # Set up part time work.
        #
        def setPartWork(self, workLabel, startHr, endHr, days):
            self.partWorkLabel = workLabel
            self.partWorkStartHr = startHr
            self.partWorkEndHr = endHr
            self.partWorkDays = days
            daysStr = ""
            for dayOw in days:
                daysStr += str(dayOw) + " "
            if debug:
                dbg("World.setPartWork({}, {}, {}, {})".format(workLabel, startHr, endHr, daysStr))

        # Set a given day of the week to be part time work or not.
        #
        # dayOw     - the day of the week
        # boolValue - True to add, False to remove
        #
        def setParkWorkDayOfWeek(self, dayOw, boolValue=True):
            if boolValue and not dayOw in self.partWorkDays:
                self.partWorkDays.append(dayOw)
            elif dayOw in self.partWorkDays:
                self.partWorkDays.remove(dayOw)

        # Set temporary home.
        #
        def setTmpHome(self, homeLabel):
            self.tmpHomeLabel = homeLabel

        # Set up temporary work.
        #
        def setTmpWork(self, workLabel, startHr, endHr, days):
            self.tmpWorkLabel = workLabel
            self.tmpWorkStartHr = startHr
            self.tmpWorkEndHr = endHr
            self.tmpWorkDays = days
            daysStr = ""
            for dayOw in days:
                daysStr += str(dayOw) + " "
            if debug:
                dbg("World.setTmpWork({}, {}, {}, {})".format(workLabel, startHr, endHr, daysStr))

        # Set the hour PC will wake, unless in tempoary home.
        #
        def setWakeHour(self, hour):
            self.wakeHr = hour

        # Set work location.
        #
        def setWork(self, workLabel):
            self.workLabel = workLabel

        # set wwork hours.
        #
        # starHr    - hour to start
        # endHr     - hour to end (exclusive)
        #
        def setWorkHours(self, startHr, endHr):
            self.workStartHr = startHr
            self.workEndHr = endHr

        # Set the working day to be over for the day, overriding other settings.
        #
        def setWorkOver(self, boolValue=True):
            self.workOver = boolValue

        # Set working flag.
        # Causes main loop to invoke .workEnd after work.
        #
        def setWorking(self, flag=True):
            self.working = flag
