#
# Fountain in the town park.
#
label nwFount:
    if gt.hour > 19 or gt.hour < 5:
        "
        You pick your way in the dark over to the foutain at the centre of the
        park. It's so quiet here.
        Huh. The fountain isn't even running,
        must be on some kind of timer to save energy or something.
        "
    elif gt.hour >= 9:
        "
        You make your way over to the fountain where water is gently
        cascading into the pool below.
        Like every other fountain you've ever seen there are coins below the water
        from people making wishes.
        "
    else:
        "
        As you approach the fountain the water is perfectly still,
        reflecting the golden colours of the sunrise.
        "
    $ world.setLoc('nwFount')
    return

    # Present the player with choices.
    # Do one of them.
    #
label .freeChoice:
    $ dbgLog("nwFount.freeChoice")
    $ mealsize = pc.getMealSize()
    $ prompt = pc.appendPrompt("What to do?", gt.hour, mealsize)
    menu:
        "[prompt]"
        "Make a wish" if not dayFlags.isSet('pcWish') and pc.canAfford(1):
            $ dayFlags.set('pcWish')
            call .wish from nw_fount_free_wish
            if dayFlags.isSet('pcWish'):
                "Having made your wish you return to the park."
                $ world.setLoc('nwTownPark')
        "Leave the fountain":
            $ world.setLoc('nwTownPark')
        "Head home" if gt.hour >= 20 or pc.energy <= 2:
            $ world.setDayOver()
        "Debug" if debugMenus:
            call debugMenu from nw_fount_free_dbg
    return



    # Return home from here.
    #
    # toLoc     - the eventual destination
    #
label .retHomeFromHere(toLoc=None):
    $ dbgLog("nwTownPark.retHomeFromHere({})".format(toLoc))
    $ tmp = pc.locoVerb()
    "You [tmp] out of the park in the town square."
    call nwTown from nw_fount_ret_home
    return



    # -------------------------------------------------------------------------
    # Local calls
    # - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -

    # Toss a coin in the fountain and make a wish.
    #
label .wish:
    $ dbgLog("nwFount.wish")
    $ prompt = addMenuAlt("What should I wish for?")
    menu:
        "[prompt]"
        "An end to my diet" if pc.isDiet() and not pc.isDietHard():
            call .wishDietEnd from nw_fount_wish_diet_end
        "Being happier with my weight" if pc.gsize >= 3 and not pc.isDietHard():
            call .wishHappyWeight from nw_fount_wish_happy_weight
        "Cold hard cash":
            call .wishCash from nw_fount_wish_cash
        "Lose some weight" if pc.isDietUnrestricted() and pc.gsize >= 2:
            call .wishLoseWeight from nw_fount_wish_lose_weight
        "I've changed my mind, this is stupid":
            $ dayFlags.clear('pcWish')
    return



    # -------------------------------------------------------------------------
    # Wishes
    # -------------------------------------------------------------------------

    # Wish for some cash.
    # This is deliberately set up to provide a small net return.
    #
label .wishCash:
    $ dbgLog("nwFount.wishCash")
    "
    You think about having more money and toss a coin into the fountain.
    As it plops into the water and shimmies to the bottom you realise this
    might not have been the smartest plan.
    "
    $ pc.spend(0.25)
    $ rnd = pc.randomBool(10)
    # $ rnd = True
    if rnd:
        "
        Looking away from the pool you see a banknote on the floor,
        and stoop to pick it up. It's a five spot.
        Wow, you weren't expecting that to work. Cool.
        "
        $ pc.earn(5)
    return



    # Wish for diet to end.
    # This could go one of two ways...
    #
label .wishDietEnd:
    $ dbgLog("nwFount.wishDietEnd")
    $ renpy.dynamic('wasCheat')
    $ wasCheat = pc.cheatDay
    "
    You think about the gnawing hole in your tummy, wishing that your diet
    was over, and toss a coin in the fountain.
    Ripples spread across the pool as it sinks to the bottom.
    "
    $ pc.spend(0.25)
    $ rnd = pc.randomBool(10)
    # $ rnd = True
    if rnd:
        if pc.testDietWill():
            # Player has succeded in resist test.
            # Automatic in hard diet mode.
            #
            $ rnd = pc.random(10, 40)
            $ pc.fat -= pc.getDietDelta()/rnd    # 2.5% to 10%
            $ pc.updateWeight()
            "
            You get this odd inexplicable feeling that you might be a little
            closer to your goal.
            "
        else:
            # Player failed their resist test.
            #
            "
            There at the side of the fountain is a large bar of chocolate you
            somehow hadn't noticed before.
            Driven by hunger you rip it open and take a huge bite.
            Then another. Then cram the rest in your mouth. Bliss.
            "
            $ food.add("chocl")
            $ pc.eatFood(food)
            $ gt.addHours(1)
            if flags.isConsume("dietStopWill"):
                pc "I give in. I'm done with dieting!
                    Chocolate is too good.
                    Wait, does that mean I got my wish?"
            elif wasCheat:
                pc "Oh well, I was already having a cheat day,
                    and I wouldn't be dieting anymore today anyway.
                    Fickle fountain, that's not what I meant at all!"
            else:
                pc "Oh well, I guess this is going to be a cheat day now,
                    and I won't be dieting anymore until tomorrow anyway.
                    Fickle fountain, that's not what I meant at all!"
    else:
        pc "Well, it was worth a try."
    return



    # Wishing you were happier about your weight.
    #
label .wishHappyWeight:
    $ dbgLog("nwFount.wishHappyWeight")
    if pc.isDietConsider():
        if gsize >= 7:
            "
            You think about how very fat you've become,
            how unhappy you are to have let yourself get this big,
            and toss a coin into the pool.
            It plops into the water, ripples spreading outwards.
            "
        elif gsize >= 4:
            "
            You think about how fat you've become,
            how unhappy you are about it,
            and toss a coin into the pool.
            It plops into the water, ripples spreading outwards.
            "
        else:
            "
            You think about how plump you are getting,
            how unhappy you are about putting on the pounds,
            and toss a coin into the pool.
            It plops into the water, ripples spreading outwards.
            "
        $ rnd = pc.randomBool(10)
        # $ rnd = True
        if rnd:
            $ pc.fatLim = int(round(pc.fat * 11 + 6)/10)
            "
            Against all odds, you no longer feel unhappy about your size
            at all, coming to terms with the fact that you are just a bigger girl now.
            "
        else:
            "
            Well, your pocket is lighter, but you don't feel any different.
            As you turn away you notice someone checking you out, clearly
            appreciating your form, and that brings a smile to your face.
            "
            $ pc.addLim(1.0)
    else:
        if gsize == 11:
            "
            You think about how massively fat you've become,
            how you just want to keep going, eating as much as you can,
            getting even fatter,
            and toss a coin into the pool.
            It plops into the water, ripples spreading outwards.
            "
        elif gsize >= 7:
            "
            You think about how very fat you've become,
            how you've enjoyed it, how you'd like to be even bigger,
            and toss a coin into the pool.
            It plops into the water, ripples spreading outwards.
            "
        elif gsize >= 4:
            "
            You think about how fat you've become,
            how much bigger you might get,
            and toss a coin into the pool.
            It plops into the water, ripples spreading outwards.
            "
        else:
            "
            You think about how plump you are getting,
            how you might like to get a little heavier,
            and toss a coin into the pool.
            It plops into the water, ripples spreading outwards.
            "
        $ rnd = pc.randomBool(10)
        # $ rnd = True
        if rnd:
            $ gsize = pc.gsize
            if gsize == 11:
                $ pc.fatLim = max(pc.fatLim+3, hardLimKg)
                "
                You get this odd and inexplicable feeling that you could
                be as fat as you could ever want and still be happy.
                You also vaguely recall fairy tales of wishes gone wrong.
                [pc.name] have you done something unwise?
                "
                if pc.getMealSize() >= 3:
                    "
                    Your tummy rumbles with hunger. It needs feeding. Again.
                    What have you done [pc.name]?
                    "
            else:
                # Either 10% over the next size limit or +3.
                $ tmp = int(round(sizeLims[gsize] * 11 + 6)/10)
                $ pc.fatLim = max(pc.fatLim+3, tmp)
                # dbg "tmp=[tmp] pc.fatLim=[pc.fatLim]"
                "
                You get this odd and inexplicable feeling that you could
                go up a size and still be happy.
                "
            # Continue dieting?
            #
            if pc.isDiet() and not pc.isDietConsider():
                "Taking a moment to reflect on your ongoing diet, you feel
                so comfortable now with your weight that you don't feel a
                real need to continue."
                $ prompt = addMenuAlt("You are dieting, but no longer feel you need to.")
                menu:
                    "[prompt]"
                    "Might as well carry on":
                        pc "Well, I've started so I might as well finish it.
                            Though I certainly feel less motivated now."
                        $ pc.dietWill = max(1, pc.dietWill * 2 / 3)
                    "Stop, I'm happy with how I am now":
                        pc "I feel so happy with the way I am, what's even the
                            point of punishing myself with a diet?"
                        $ pc.dietStop()
        else:
            "
            Well, your pocket is lighter, but you don't feel any different.
            As you turn away you notice someone checking you out, clearly
            appreciating your form, and that brings a smile to your face.
            "
            $ pc.addLim(1.0)
    return



    # Wish for losing weight when not on a diet.
    # Either removes some weight or allows PC to choose to diet.
    #
label .wishLoseWeight:
    $ dbgLog("nwFount.wishLoseWeight")
    $ gsize = pc.gsize
    if pc.isDietConsider():
        if gsize >= 7:
            "
            You think about how very fat you've become,
            how unhappy you are to have let yourself get this big,
            and toss a coin into the pool.
            It slips into the water with a plink, barely causing a ripple.
            "
        elif gsize >= 4:
            "
            You think about how fat you've become,
            how unhappy you are about it,
            and toss a coin into the pool.
            It slips into the water with a plink, barely causing a ripple.
            "
        else:
            "
            You think about how plump you are getting,
            how unhappy you are about putting on the pounds,
            and toss a coin into the pool.
            It slips into the water with a plink, barely causing a ripple.
            "
    else:
        if gsize >= 10:
            "
            You think about how massively fat you've become,
            and toss a coin into the pool.
            It plops into the water, ripples spreading outwards.
            "
        elif gsize >= 7:
            "
            You think about how very fat you've become,
            and toss a coin into the pool.
            It slips into the water with a plink, barely causing a ripple.
            "
        elif gsize >= 4:
            "
            You think about how fat you've become,
            and toss a coin into the pool.
            It slips into the water with a plink, barely causing a ripple.
            "
        else:
            "
            You think about how plump you are getting,
            and toss a coin into the pool.
            It slips into the water with a plink, barely causing a ripple.
            "
    $ pc.spend(0.25)
    $ rnd = pc.randomBool(10)
    # $ rnd = True
    if rnd:
        $ rnd = pc.random(50, 100)
        $ pc.fat -= pc.fat / rnd        # Between 1% and 2%
        $ pc.updateWeight()
        "
        You get this odd inexplicable feeling that you might be a little
        lighter than you were before.
        "
    else:
        "
        Well, your pocket is lighter, but that's all.
        Oh well, there are more conventional means of losing weight, like
        diet and exercise. That and not eating everything you see.
        "
        call pcDiet.think from nw_fount_wish_lose_weight_diet
    return
