#
# Warehouse work.
#
define  vachMinWageHr = 7.25             # Minimum wage
default vachPcWageHr = vachMinWageHr    # Actual wage

init python:
    # Earn a wage for each hour.
    # hours     - number of hours worked
    def vachWareEarn(hours):
        counters.add('vachWage', hours * vachPcWageHr)

label nwVachonWarehouse(silent=True):
    $ dbgLog("nwVachonWarehouse")
    if not silent:
        "
        The Vachon warehouse is a large box-like building with the company's
        blue trim and stylised cow logo.
        It has loading bays for vans and trucks along one side and a separate
        entrance for staff.
        "
    $ world.setLoc('nwVachonWarehouse')
    return

    # -------------------------------------------------------------------------
    # Interface calls
    # - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -

    # Part time work.
    # Invoked when the pc is part-time working here.
    # Morning 9-13
    # Lunch 13-14
    # Afternoon 14-18
    #
label .workPart:
    $ dbgLog("nwVachonWarehouse.workPart")
    $ renpy.dynamic('workDays')
    $ world.setWorking()
    $ workDays = evtCounters.get('vachWareWork')
    if not world.isLoc('nwVachonWarehouse'):
        if world.isLocLike('nwVachon'):
            "You walk back to the warehouse."
            $ world.setLoc('nwVachonWarehouse')
        else:
            call .travelToWork from nw_vach_ware_work_part_travel
            $ world.setLoc('nwVachonWarehouse')
    if workDays > 0 and not dayFlags.isSet('pcVachCover'):
        # Get changed.
        #
        call .coverCheck from nw_vach_ware_work_part_cover_check
        call .coverOn from nw_vach_ware_work_part_cover_on
    if gt.isBefore(13):
        call .workAm from nw_vach_ware_work_part_work_am
    elif gt.isBetween(13, 14):
        if workDays == 0:
            "
            Stanley takes you across to the cafeteria building.
            "
        else:
            "
            You walk across to the cafetria building with the other warehouse
            workers.
            "
        call nwVachonCafeteria.lunch from nw_vach_ware_work_part_lunch    
    elif gt.isBefore(18):
        call .workPm from nw_vach_ware_work_part_work_pm
    
    return



    # Part time work end.
    # Invoked when the pc stops part time job here for the day.
    #
label .workPartEnd:
    $ dbgLog("nwVachonWarehouse.workPartEnd")
    $ renpy.dynamic('days', 'dismiss', 'endProb', 'friday', 'officeCount', 'stanSize', 'wage')

    # Record days work.
    $ days = evtCounters.inc('vachWareWork')

    # Does PC need to visit the office for any reason?
    #
    $ dismiss = not world.isPartWork()                              # Dismissed
    $ endProb = not dismiss and days == 4                           # End of probation
    $ friday = not dismiss and days >= 12 and cooldowns.isZero('wareFriday') and not world.isPartWorkDayOfWeek(4) # Fridays too
    $ stanSize = not dismiss and dayFlags.isSet('stanPcWeight')     # Need to talk about gains
    $ wage = dismiss or gt.dayOw == world.partWorkDays[-1]          # Get paid on the last work day of the week.
    $ officeCount = dismiss + endProb + friday + stanSize + wage
    if officeCount > 0:
        call .workOffice from nw_vach_ware_work_end_office

    # Take off work clothes.
    #
    call .coverOff from nw_vach_ware_work_end_cover_off

    # Leave, trip home or elsewhere.
    #
    call .travelFromWork from nw_vach_ware_work_end_travel_from
    return



    # -------------------------------------------------------------------------
    # Local calls
    # - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -

    # Get a bigger coverall if needed.
    #
label .coverCheck:
    $ dbgLog("nwVachonWarehouse.coverCheck")
    $ renpy.dynamic('gsize')
    $ gsize = pc.getSize()
    if not pcClothes.has(gsize, ckVachWare):
        "
        You head over to the foreman's office as you are definitely going to need a
        bigger coverall today.
        "
        pc "Stanley, can I get a bigger coverall?"
        stanley "Sure, sure, [pc.name]. Just wait here a moment."
        "
        He steps out and soon returns with a new coverall and hands it to you.
        "
        stanley "Here you go.
            You know I'll have to deduct the cost from your wages, right?"
        pc "Yes, I know."
        $ pcClothes.buy(ckVachWare, gsize)
        $ counters.add('vachWage', min(-32, -4*gsize))
    return



    # Get ready for work.
    #
label .coverOn:
    $ dbgLog("nwVachonWarehouse.coverOn")
    $ renpy.dynamic('cvfit', 'gsize', 'unsure')
    $ gsize = pc.getSize()
    $ cvfit = pc.getSizeProgress(pc.gsize)
    $ unsure = pc.isDietConsider()
    if gsize >= 11:
        "
        You squeeze yourself through the doorway into the locker room
        to get changed into your coverall and safety gear.
        "
    elif gsize >= 10:
        "
        Your hips brush both sides of the door frame as you go into the locker room
        to get changed into your coverall and safety gear.
        "
    else:
        "
        You head into the locker room to get changed into your coverall and
        safety gear.
        "
    if gsize >= 8: # 400lbs+
        "
        You sit down to take off your own boots and put them in your locker,
        then pull on your coverall over your clothes and fasten all the poppers.
        Lacing up your work boots leaves you panting.
        Then it's just your gloves and hard hat.
        "
        if cvfit > 0.9:
            if unsure:
                "
                A quick check in the mirror, and even a functional garment like
                this isn't going to just how much fat girl is stuffed inside
                or the size of your ass and thighs.
                "
                pci "Let's hope those poppers hold, [pc.name]."
                pc "Great, something else to worry about."
            else:
                "
                A quick check in the mirror, and even a functional garment like
                this can barely contain all this sexyness.
                The fabric is taut across your chest, belly, butt, and thighs.
                "
                pci "Those poppers are working overtime, aren't they, [pc.name]?"
                if pc.randomKinkStat(10, 40, 50):
                    pc "Aren't they just. It would be a shame if they came undone."
                    pci "You are a naughty girl, aren't you, [pc.name]."
        elif unsure:
            "
            A quick check in the mirror, and there's no disguising just how
            huge you are these days. It's supposed to be loose fitting, but
            with your hips that's not really the case.
            "
        else:
            "
            A quick check in the mirror, and there's no disguising your
            abundant sexyness even in a functional garment like this.
            "
    elif gsize >= 6: #237lbs +
        "
        You sit down to take off your own boots and put them in your locker,
        then pull on your coverall over your clothes and fasten all the poppers.
        Then your work boots, gloves and hard hat go on.
        "
        if cvfit > 0.9:
            if unsure:
                "
                A quick check in the mirror, and even a functional garment like
                this isn't going to just how much fat girl is stuffed inside
                or the size of your ass and thighs.
                "
            else:
                "
                A quick check in the mirror, and even a functional garment like
                this can barely contain all this sexyness.
                The fabric is taut across your chest, belly, butt, and thighs.
                "
        elif unsure:
            "
            A quick check in the mirror, and even a functional garment like
            this isn't going to hide how porky you are these days.
            "
        else:
            "
            A quick check in the mirror, and there's no disguising your sexy
            shape even in a functional garment like this.
            "
    else: # Less than 237lbs.
        "
        You take off your own boots and put them in your locker, then pull
        on your coverall over your clothes and fasten all the poppers. Then
        your work boots, gloves and hard hat go on.
        "
        if gsize >= 5 and cvfit >= 0.9:
            "
            A quick check in the mirror, and ... this is actually looking
            kind of fitted and snug. Hmmm.
            "
        else:
            "
            A quick check in the mirror, and, yes, it's completely shapeless,
            as expected.
            "
    $ dayFlags.set('pcVachCover')
    return



    # Get out of work clothes.
    #
label .coverOff:
    $ dbgLog("nwVachonWarehouse.coverOff")
    $ renpy.dynamic('cvfit', 'gsize')
    $ gsize = pc.getSize()
    $ cvfit = pc.getSizeProgress(pc.gsize)
    "
    You head into the locker room to take off your coverall and
    safety gear.
    "
    if gsize >= 8: # 400lbs+
        if cvfit > 0.8:
            "
            You sit down to take off your work boots and put them and your
            safety gear in your locker. A deep breath is all that is needed to
            open most of the poppers on your coverall. You put it away, and sit
            down again, somewhat breathless, to put your own boots back on.
            "
        else:
            "
            You sit down to take off your work boots and put them and your
            safety gear in your locker. Then you pop all the fasteners on your
            coverall, shed it, and put it away. Sitting down again, somewhat
            breathless, you put your own boots back on.
            "
    elif gsize >= 6: #237lbs +
        if cvfit > 0.8:
            "
            You sit down to take off your work boots and put them and your
            safety gear in your locker. The fabric of your coverall is already
            taut, and the poppers open easily. You put it away, and sit down
            again to put your own boots back on.
            "
        else:
            "
            You sit down to take off your work boots and put them and your
            safety gear in your locker. Then you pop all the fasteners on your
            coverall, shed it, and put it away. Sitting down again, you put your
            own boots back on.
            "
    else: # Less than 237lbs.
        "
        You take off your work boots and put them and your safety gear in your
        locker. Then you pop all the fasteners on your coverall, shed it, and
        put it away. After putting your own boots back on you are ready to leave.
        "
    $ dayFlags.clear('pcVachCover')
    return


    # First day orientation.
    #
label .orientation:
    $ dbgLog("nwVachonWarehouse.orientation")
    $ renpy.dynamic('gsize')
    $ gsize = pc.getSize()
    "
    As you walk into the little entranceway there's a greying middleaged man
    who seems to be waiting for you.
    "
    stanley "Hi, I'm Stanley, warehouse foreman.
        You must be [pc.name], the new hire starting today?"
    pc "Yes, sir, that's me."
    stanley "Great, great. If you'd like to follow me."
    "
    He leads you into a small and cluttered office. Despite quite a few filing
    cabinets, every flat surface seems to be covered in piles of catalogues,
    documents, and binders.
    "
    pc "So, is this, like, an interview, sir?"
    stanley "Not really, no. Let me be honest with you, this isn't a taxing
        job. You seem personable enough, Steve seems to think you're up for it,
        and, well, we need staff."
    stanley "Also you don't need to call me sir. Stanley, or even Stan is fine."
    pc "Okay si ... Stanley."
    stanley "Now, you'll be starting out at minimum wage during your probation
        period. I hope that's okay."
    pc "Yes, I expected that."
    stanley "Good, good. Hopefully that won't last too long.
        So, do you want the job?"
    pc "It's mine? Yes, please."
    stanley "Right, right. On with the formalities."
    "
    He digs around on his desk and finds a form, and eventually a ballpoint that
    works.
    "
    stanley "Okay, what's your full name?"
    "
    Ugh, you'd been dreading this.
    "
    pc "Do I have to? Please? I hate it."
    stanley "I'm afraid we need it for the contract.
        Don't worry, you'll still be [pc.name] here."
    pc "It's ... {size=-4}[pcFullName]{/size}.
        I think my mother might still have been a bit trippy from the
        pain meds when they named me."
    stanley "Oh, oh my. Yes, quite a mouthful. I can see why you go by [pc.name]."
    pc "That doesn't leave this office, right?"
    stanley "It's just for the form, I promise.
        The only people who will know are myself and HR.
        Social security number?"
    pc "I don't know it, sorry. It's complicated."
    stanley "I'm sure HR can track it down for you. Where were you born?"
    "
    You tell him, and work through other sundry questions to the best of your
    knowledge. Your employment history takes a while. Finally he hands you the
    contract to sign, and he countersigns it.
    "
    stanley "Welcome to the Vachon family, [pc.name]!"
    stanley "Next up is the safety training, just a minute."
    "
    He potters around the office and unearths an ancient looking television
    and slots a videocassette into it.
    "
    stanley "There's a certain amount of risk associated with working in the
        warehouse, forklifts, chemicals, and so on. I need you to sit and watch this.
        I hope you don't mind if I get on with other stuff."
    "
    You sit and watch. It is so dreadfully upbeat and cheaply produced which almost
    distracts from the important messages about hazards, safe lifiting, and
    caution around the agricultural equipment and chemicals.
    "
    $ gt.addHours(1)
    stanley "Still with us? Dreadful isn't it, and it doesn't get any better after
        the umpteenth time. Any questions?"
    pc "Just about, and no, no questions."
    stanley "Right, lets get you kitted out."
    "
    He leads you to outside a locker room, and hands you a blue coverall, a
    hard hat, a pair of boots, work gloves, and a locker key.
    "
    # The smallest coverall fits up to size 5.
    #
    $ pcClothes.buy(ckVachWare, max(5, gsize))
    $ counters.add('vachWage', min(-32, -4*gsize))
    stanley "If you'd like to pop in and get changed, I'll wait out here.
        The coverall goes over your street clothes.
        I'm afraid there is a nominal charge for the work gear that comes
        out of your first day's wages."
    call .coverOn from nw_vach_ware_orient_cover_on
    $ gt.addHours(1)
    "
    As you come out of the locker room Stanley hands you an employee badge.
    "
    stanley "You'll need to keep that pinned on your ... chest pocket
        while you're working."
    if gsize >= 4:
        pc "Okay, done. No one's going to miss that are they?"
        stanley "Erm, no, no they're not."
    else:
        pc "Okay, done."
    stanley "Right, let's go through into the warehouse."
    "
    He pushes through a set of double doors into a huge cavernous space
    filled with racking as far as you can see. Most of it is palletised,
    with the ones on the ground level opened for picking from. It's low-key
    impressive.
    "
    "
    He walks you up and down the isles, explaining the layout and how each
    rack has a letter, and each bay is numbered. Some of the smaller items
    are in pick bins.
    "
    $ gt.addHours(1)
    pc "It's a bit like Ikea, isn't it?"
    stanley "Same kind of idea really, but without the confusing names."
    pc "I don't know what half of this stuff is, let alone what it's for."
    stanley "You'll pick it up, but you don't really need to know.
        Let me show you to your station."
    "
    He walks you around to a table with a PC, printer, and scanner.
    "
    stanley "So the orders for you to pick are printed out here, and each
        item has the location of where to find it and whether you'll be
        packing it into boxes, or onto a pallet. Let me walk you through
        a simple one."
    pc "Okay."
    "
    He presses a key and the printer spits out a page with the pick list.
    "
    stanley "So grab that, the cart, and the scanner. First item is in
        C.10.3..."
    "
    You both traipse off to the appropriate bay, and he shows you how to scan
    each item, and the scanner acknowledges each one. Item follows item.
    "
    pc "What are these things? They're like tiny scrunchies."
    stanley "Those are for a horse's mane, so you're not far wrong.
        His daughter Amelia is one of those horse-obsessed girls."
    "
    With the last item done you both head back to your station and pack
    the things into a cardboard box. The printer spits out a packing note
    which goes on top, and a label for the outside.
    "
    stanley "All done, we just need to wheel this over to bay five ready to
        go on one of the vans. Got it?"
    pc "Yes, it seems simple enough. What if I can't reach something though?
        I'm not exactly tall."
    stanley "There are wheelable steps around, so use those."
    pc "And when stuff needs to go onto a pallet?"
    stanley "Well, you can either take the things to the pallet with the cart,
        or sometimes it's easier to take the empty pallet to the bay and
        load it there."
    "
    He shows you how to operate one of the manual pallet jacks which use
    hydraulics to lift and wheels to maneuver.
    "
    pc "These aren't easy to steer, are they?"
    stanley "It's just because you're steering from the back, you'll soon
        get used to it, I promise."
    pc "I hope so!"

    if gt.isBefore(13):
        $ vachWareEarn(4)
        $ gt.advanceTo(13, False)
        stanley "Right, it must be about time for some lunch.
            There's a works canteen on site.
            Let's go across and I can introduce you to the rest of the gang."
        pc "Great! Some lunch sounds good to me."
    else:
        $ vachWareEarn(4)
        $ gt.advanceTo(18, False)
    $ pc.useEnergy(2)
    $ dayFlags.set('vachOrient')
    return



    # Tell Steve about getting the sack.
    #
label .steveWareFaint:
    $ dbgLog("nwVachonWarehouse.steveWareFaint")
    "
    Steve can tell you're upset about something.
    "
    if flirt:
        $ pet = stevePetName()
        steve "What's the matter [pet]?"
        pc "I fainted. It was really scary.
            They had to put me on a fucking pallet to move
            me out of the warehouse. With the forklift!"
        steve "Are you okay?"
        pc "Yes, I had a rest and feel a fair bit better now. But..."
        steve "But what?"
        pc "But they're worried I might die from the physical work at my size.
            I hadn't really thought about it before."
        $ pet = stevePetName()
        steve "I've been worried about that for a while [pet].
            I didn't like to say anything."
        pc "I know. It kind of brought it all home."
        pc "And then I promised to get my weight down to seven fifty
            and keep it that way."
        steve "Oh, that's a big promise."
        pc "I know. But it was that or lose my job."
        $ pet = stevePetName()
        steve "Do you want me to help you keep your weight down [pet]?"
        $ prompt = "Steve's offering to help me keep my weight down. Do I want his help?"
        $ prompt = addMenuAlt(prompt)
        menu:
            "[prompt]"
            "Yes, I really think I need your help":
                pc "Babe, I know I'm going to struggle, and I could really use your support."
                steve "I'll be here for you my love."
                # Set up Steve to be controlling PC's weight (again) and targetting
                # 750lbs max.
                #
                $ steve.setMaxBfpByWeightLb(750, 100, True)
                $ flags.set('steveControl')
                $ pc.setFatLimLowerKg(750/2.2)
            "No, I can do this myself":
                pc "Thank you babe, but I think I can do this by myself."
                steve "Okay. As you wish."
                pci "Can you [pc.name]? Can you really?
                    You don't exactly have a good track record."
    else:
        steve "What's the matter [pc.name]?"
        pc "I fainted. It was really scary.
            They had to put me on a fucking pallet to move
            me out of the warehouse. With the forklift!"
        steve "Are you okay?"
        pc "Yes, I had a rest and feel a fair bit better now. But..."
        steve "But what?"
        pc "But they're worried I might die from the physical work at my size.
            I hadn't really thought about it before."
        steve "I've been worried about that for a while.
            I didn't like to say anything."
        pc "I know. It kind of brought it all home."
        pc "And then I promised to get my weight down to seven fifty
            and keep it that way to keep my job."
        steve "Oh, that's a big promise."
        pc "I know. Fuck. What am I going to do?"
        steve "Get used to smaller portions?"
        pc "Yes, but for ever and ever?"
    return



    # Tell Steve about getting the sack.
    #
label .steveWareSack:
    $ dbgLog("nwVachonWarehouse.steveWareSack")
    "
    You're trying not to, but the tears come now.
    "
    if flirt:
        $ pet = stevePetName()
        steve "What's up [pet]?"
        pc "I got the sack babe."
        steve "Why?"
        if evtCounters.isEq('wareFaint', 1):
            pc "I fainted. It was really scary.
                They had to put me on a fucking pallet to move
                me out of the warehouse. With the forklift!"
            steve "That hardly seems like a sackable offence."
            pc "No. But they're worried I might die from the physical work.
                Stanley, the foreman, asked me to promise to get my weight
                down, and I refused."
            steve "Oh. I can understand that. I don't want you to die either!"
            pc "I know babe. I don't want to die. But I can't keep working there either."
            $ pet = stevePetName()
            steve "At least you're alive [pet]. There is that."
        else:
            pc "I fainted again. I'm too fucking fat for physical work."
            steve "I have been so worried about that."
            pc "And after the first time, when I promised I'd keep my weight under control..."
            steve "That hasn't gone well, has it."
            pc "No. I guess I didn't give them a lot of choice."
        "
        Steve leans over and hugs you until you feel a little better.
        "
    else:
        steve "What's up [pc.name]?"
        pc "I got the sack."
        steve "Why?"
        if evtCounters.isEq('wareFaint', 1):
            pc "I fainted. It was scary.
                They had put put me on a fucking pallet to move
                me out of the warehouse."
            steve "That hardly seems like a sackable offence."
            pc "No. But they're worried I might die from the physical work.
                Stanley, the foreman, asked me to promise to get my weight
                down, and I refused."
            steve "Oh. I can understand that. I don't want you to die either!"
            pc "I know. I don't want to die. But I can't keep working there either."
            steve "At least you're still alive [pc.name]. There is that."
        else:
            pc "I fainted again. I'm too fucking fat for physical work."
            steve "I have been so worried about that."
            pc "And after the first time, when I promised I'd keep my weight under control..."
            steve "That hasn't gone well, has it."
            pc "No. I guess I didn't give them a lot of choice."
        steve "Do you need a hug?"
        pc "Yes. I think I do."
        "
        Steve leans over and hugs you until you feel a little better.
        "
    return


    # Travel away from work.
    #
label .travelFromWork:
    $ dbgLog("nwVachonWarehouse.travelFromWork")
    $ renpy.dynamic('destLoc', 'destName', 'event', 'film', 'flirt', 'pet', 'tease')
    $ flirt = steve.isRomance()
    $ tease = flags.isSet('steveTease') and pc.randomBool(60)
    $ pet = stevePetName(tease)
    if steve.isBreak() or gt.isBefore(18):
        # Broken up with Steve or heading home early.
        #
        $ tmp = pc.locoVerb()
        if flags.isConsume('steveWareFired'):
            "
            What a miserable fucking day!
            You won't be getting a lift home either so you [tmp] straight to the bar
            for a drink.
            "
            call nwBar.enter from nw_vach_ware_travel_from_alone_fired_enter
            call nwBarDrink from nw_vach_ware_travel_from_alone_fired
        elif flags.isConsume('steveWareFaint'):
            "
            What a scary day!
            You hadn't really given much serious thought to how much your weight
            could affect your health, let alone though about your own mortality.
            Fuck!
            "
            "
            It would have been nice to have a lift home, and a chat, but that's
            not going to be happening.
            Instead you [tmp] straight to the bar for a drink.
            "
            call nwBar.enter from nw_vach_ware_travel_from_alone_faint_enter
            call nwBarDrink from nw_vach_ware_travel_from_alone_faint
        else:
            "
            You can't really expect a lift from Steve in the circumstances, so
            you [tmp] back into town.
            "
            call nwTown from nw_vach_ware_travel_from_alone
        return
    
    "
    You walk over to the car park, where Steve is waiting for you.
    "
    call nwSteveCar.pcIn(tease) from nw_vach_ware_steve_travel_from_pc_in
    call nwSteveCar.steveIn(tease) from nw_vach_ware_steve_travel_from_steve_in
    if evtCounters.isEq('vachWareWork', 1):
        if flirt:
            steve "So, how did your first day go [pet]?"
            pc "Okay, I think. It wasn't as bad as I was expecting.
                My workmates seem like good people."
            steve "That's good. So, you're going to stick at it?"
            pc "Yes, I will. It's nice to have some extra cash."
        else:
            steve "So, how did your first day go [pc.name]?"
            pc "Okay, I think. It wasn't as bad as I was expecting.
                My workmates seem like good people."
            steve "That's good. So, you're going to stick at it?"
            pc "Yes, I will. It's nice to have some extra cash."
    if flags.isConsume('steveWareFired'):
        call .steveWareSack from nw_vach_ware_travel_steve_sack
    if flags.isConsume('steveWareFaint'):
        call .steveWareFaint from nw_vach_ware_travel_steve_faint
        
    $ event = gt.getEventNow()
    $ film = event is not None and event.evLabel == 'steveHomeFilm'
    $ prompt = "Steve is giving you a lift, where do you want to go?"
    if film:
        $ prompt = "Steve has invited you over for dinner and a film. Do you still want to go?"
        $ gt.doneEvent(event)
    $ prompt = addMenuAlt(prompt)
    $ pet = stevePetName(tease)
    menu:
        "[prompt]"
        "Yes, go with him to his house" if film:
            steve "Shall we head straight over to my place [pet]?"
            pc "That would be great! Thanks babe."
            "
            Steve drives through town and pulls up outside his house.
            "
            $ destLoc = 'nwSteveHome'

        "Can you drop me off at home?":
            if film:
                steve "Shall we head straight over to my place [pet]?"
                pc "I'm sorry babe, but I really don't feel up to it tonight.
                    Can you take me home please?"
                "
                Of course, he's disappointed.
                "
                steve "Oh. Okay. I can do that."
                $ film = False
            elif flirt:
                pc "Can you drop me off at home please babe?"
                steve "Sure, no problem [pet]."
            else:
                pc "Can you drop me off at home please?"
                steve "Sure, I can do that."
            $ destLoc = world.getCurHomeLoc()
            if destLoc == 'nwGas':
                "
                Steve drives through town and pulls up in the gas station forecourt.
                "
            else:
                "
                Steve drives through town and pulls up outside you home.
                "

        "Can you drop me off in town?" if not film:
            if flirt:
                pc "Can you drop me off in town please babe?"
                steve "Sure, no problem [pet]."
            else:
                pc "Can you drop me off in town please?"
                steve "Sure, I'm going that way."
            $ destLoc = 'nwTown'

    $ world.setLoc(destLoc)
    call nwSteveCar.pcOut(tease) from nw_vach_ware_steve_travel_from_pc_out
    if film:
        "
        He opens his front door and lets you inside.
        "
        call nwSteveHomeFilm from nw_vach_ware_steve_travel_from_film
    return



    # Travel to work.
    #
label .travelToWork:
    $ dbgLog("nwVachonWarehouse.travelToWork")
    $ renpy.dynamic('flirt', 'pet', 'tease')
    $ flirt = steve.isRomance()
    $ tease = flags.isSet('steveTease') and pc.randomBool(60)
    $ pet = stevePetName(tease)
    if steve.isBreak():
        # Broken up with Steve.
        #
        "
        You can't really expect a lift to work from Steve in the circumstances,
        so you'll have to walk to work.
        "
        if flags.isConsume('gasEvidence'):
            pci "Aren't you forgetting something [pc.name]? Your empties?"
            pc "Oh crap."
            "
            You lumber back upstairs, grab your trash, and quickly head
            round the back to the dumpster to dispose of the evidence.
            "
        call main.travelTo('nwVachonWarehouse') from nw_vach_ware_travel_to_alone
        return

    if world.isLoc('nwGas'):
        if flags.isConsume('gasEvidence'):
            "
            You hear Steve's car pull up outside the gas station.
            "
            pci "Aren't you forgetting something [pc.name]? Your empties?"
            pc "Oh crap."
            "
            You lumber back upstairs, grab your trash, and quickly head outside.
            Steve seems puzzled as you go round the back to the dumpster
            then return.
            "
        else:
            "
            Steve's car pulls up outside the gas station and you go outside.
            "
    elif world.isLocLike('nwDiner'):
        "
        You hear Steve's car pull up outside the diner, ready to take you
        to work, so you rush outside.
        "
    else:
        "
        Steve's car pulls up ready to take you to work.
        "
    call nwSteveCar.pcIn(tease) from nw_vach_ware_steve_travel_gas_pc_in
    call nwSteveCar.steveIn(tease) from nw_vach_ware_steve_travel_gas_steve_in
    if workDays == 0:
        if flirt:
            steve "Are you ready for your first day, [pet]?"
            pc "I think so, I'm not really sure what to expect, babe."
            steve "I'm sure there'll be some introduction and training on the first day."
            pc "I hope so. I'm a bit nervous."
        else:
            steve "Are you ready for your first day, [pc.name]?"
            pc "I think so, I'm not really sure what to expect."
            steve "I'm sure there'll be some introduction and training on the first day."
            pc "I hope so."
    "
    Steve drives through town to the Vachon complex and parks up.
    "
    call nwSteveCar.pcOut(tease) from nw_vach_ware_steve_travel_gas_pc_out
    if workDays == 0:
        if flirt:
            steve "That large building over there is the warehouse, [pet].
                Have a good day!"
            "
            He points over to a very box like building with a number of loading
            bays and then gives you a kiss and a hug.
            "
            pc "Thanks, babe. Here goes..."
        else:
            steve "That large building over there is the warehouse, [pc.name].
                Have a good day!"
            "
            He points over to a very box like building with a number of loading
            bays.
            "
            pc "Okay, here goes..."
    return



    # Morning's work.
    #
label .workAm:
    $ dbgLog("nwVachonWarehouse.workAm")
    $ renpy.dynamic('hours')
    if workDays == 0:
        call .orientation from nw_vach_were_work_am_orientation
    else:
        call nwVachonWarehouseWork from nw_vach_were_work_am
        $ hours = gt.advanceTo(13, False)
        $ vachWareEarn(hours)
        $ pc.exercise(500, hours)
        $ pc.useEnergy(3)
    return



    # Afternoon work.
    #
label .workPm:
    $ dbgLog("nwVachonWarehouse.workPm")
    $ renpy.dynamic('hours')
    if workDays == 0 and not dayFlags.isSet('vachOrient'):
        call .orientation from nw_vach_were_work_pm_orientation
    else:
        call nwVachonWarehouseWork from nw_vach_were_work_pm
        $ hours = gt.advanceTo(18, False)
        $ vachWareEarn(hours)
        $ pc.exercise(500, hours)
        $ pc.useEnergy(3)
    return


    # After work interactions with Stanley in his office.
    #
    # days          - number of days worked
    # dismiss       - dismissed from work
    # friday        - can work friday too
    # officeCount   - number of interactions
    # endProb       - pc given a raise at end of probation
    # stanSize      - talk about PCs weight
    # wage          - give pay packet
    # 
label .workOffice:
    $ dbgLog("nwVachonWarehouse.workOffice")
    $ renpy.dynamic('dieting', 'first', 'goalKg', 'gsize', 'unsure')
    $ dieting = pc.isDiet()
    $ first = True
    $ gsize = pc.getSize()
    $ unsure = pc.isDietConsider()
    stanley "Before you go can you pop into my office [pc.name]?"
    if gsize >= 11:
        "
        You squeeze your way into the foreman's cluttered office.
        There's not really anything suitably sized or robust enough for you to sit on.
        "
    elif gsize >= 10:
        "
        You waddle into the foreman's cluttered office, hips brushing the
        door frame.
        There's not really anything suitably sized for you to sit on.
        "
    elif gsize >= 9:
        "
        You waddle through the narrow doorway into the foreman's cluttered office.
        There's not really anything suitably sized for you to sit on.
        "
    else:
        "
        You go into the foreman's cluttered office and take a seat.
        "
    # PC's wages.
    #
    if wage:
        if officeCount == 1:
            stanley "I just wanted to give you your pay packet."
        else:
            stanley "Firstly, here's your pay packet."
        call .workWage from nw_vach_ware_office_pay
        pc "Thank you!"
        $ officeCount -= 1
        $ first = False

    # Being sacked.
    #
    if dismiss:
        stanley "I really am sorry to have to let you go [pc.name].
            I wish things had turned out differently."
        pc "So do I."
        pci "Your greed really messed that up didn't it [pc.name].
            Couldn't help yourself, could you?
            You actually ate yourself out of a job!"
        $ pc.addMood(-20)
        "
        Stanley hands you some paperwork as you are struggling to keep
        your composure. You want to cry, but know it wouldn't be appropriate.
        He stands and shakes your plump little hand.
        "
        stanley "[pc.name], best of luck."
        stanley "And, seriously, please take care of yourself."
        pci "Keep it together [pc.name]."
        pc "Thank you Stanley, I'll try."
        $ flags.set('steveWareFired')

    # Probabtion over.
    #
    if endProb:
        if officeCount == 1:
            stanley "And I wanted to let you know with your probation period
                over we'll be increasing your pay. Starting next shift you'll
                be earning an extra dollar an hour before deductions."
        else:
            stanley "Next I wanted to let you know with your probation period
                over we'll be increasing your pay. Starting next shift you'll
                be earning an extra dollar an hour before deductions."
        stanley "I know it's not much, but it all helps doesn't it."
        pc "Thanks Stanley. That will help."
        $ vachPcWageHr += 1
        $ officeCount -= 1
        $ first = False

    # Work Fridays?
    #
    if friday:
        if officeCount == 1:
            stanley "Lastly, I wanted to ask you if you'd like to work here on Fridays too.
                We've been getting busier, and you are doing well here.
                The extra day is yours if you want it."
        else:
            stanley "Next, I wanted to ask you if you'd like to work here on Fridays too.
                We've been getting busier, and you are doing well here.
                The extra day is yours if you want it."
        $ prompt = addMenuAlt("Would I like to work in the warehouse on Fridays?")
        menu:
            "[prompt]"
            "Yes, I'd like to work Fridays":
                pc "Yes please, that would be appreciated Stanley. Thank you."
                stanley "Great, great. See you on Friday [pc.name]!"
                $ world.setParkWorkDayOfWeek(4)
            "No, I'd like to keep my days the same":
                pc "I don't want to leave you in a bind,
                    but I'm happy with my current schedule."
                stanley "Okay, okay, that's fine [pc.name]"
                $ cooldowns.set('wareFriday', 28)
        $ officeCount -= 1
        $ first = False

    # Player weight issue.
    #
    if stanSize:
        $ goalKg = kgAtSizeProgress(gsize-1, 75)
        if first:
            stanley "[pc.name], can we talk about
                your increasing weight? It wouldn't normally be any of my business,
                but I do have a responsibility for the health of you all."
        else:
            stanley "Lastly, and this is a little more delicate, can we talk about
                your increasing weight? It wouldn't normally be any of my business,
                but I do have a responsibility for the health of you all."
        if unsure:
            pc "Oh. That."
        else:
            pc "Okay."
        if gsize >= 10:
            if gsize >= 11:
                stanley "I have noticed how you are really struggling now with
                    the active nature of the work here, not to mention the doors,
                    and that's a concern."
                $ goalKg = kgAtSizeProgress(9, 75)
            else:
                stanley "I have noticed how you are really struggling now with
                    the active nature of the work here, and that's a concern."
            if dieting:
                pc "Believe me, I've noticed it too, and I've started dieting
                    already."
                stanley "Okay, okay. That's good to know [pc.name].
                    I'm sure you'll find things get a bit easier without the extra weight."
            elif unsure:
                pc "Believe me, I've noticed too.
                    I'm not exactly happy with how fat I am."
                $ prompt = "Stanley thinks I'm too heavy for warehouse work and he's not wrong."
                call pcDiet.dietPromptMenu(prompt, goalKg) from nw_vach_ware_office_size10_diet_unsure
                if _return:
                    pc "Right, starting right now I am determined to shift some of this flab!"
                    stanley "Okay, okay. That's good to know [pc.name].
                        I'm sure you'll find things get a bit easier without the extra weight."
                else:
                    pc "I'll see what I can do Stanley."
            else:
                pc "I can't say I'd really noticed.
                    I mean I know I'm pretty fat, but is it really a problem?"
                stanley "It's getting that way. I'm just trying to look out for you [pc.name]."
                $ prompt = "I'm happy with my weight, but Stanley thinks I'm too heavy for warehouse work."
                call pcDiet.dietPromptMenu(prompt, goalKg) from nw_vach_ware_office_size10_diet_happy
                if _return:
                    pc "I suppose you're right. I'll start dieting and try to shift some of this flab!"
                    stanley "Okay, okay. That's good to know [pc.name].
                        I'm sure you'll find things get a bit easier without the extra weight."
                else:
                    pc "I'll see what I can do Stanley."
            if not pc.isDiet():
                stanley "Another option might be to look for office work where
                    you won't be on your feet all day. Perhaps in purchasing?
                    If you're still friends with Steve he might know of an opening."
                if steve.isOnBreak():
                    pc "That's going to be difficult, we aren't really speaking right now."
                    stanley "I'm sorry to hear that."
                else:
                    pc "I'll have a word with him."
                $ flags.set('pcSteveAskOffice')

        else:
            stanley "I have noticed how you are beginning to struggle with
                the active nature of the work here, and that's a concern."
            if dieting:
                pc "Believe me, I've noticed it too, and I've started dieting
                    already."
                stanley "Okay, okay. That's good to know [pc.name].
                    I'm sure you'll find things get a bit easier without the extra weight."
            elif unsure:
                pc "Believe me, I've noticed too.
                    I'm not exactly happy with how fat I am."
                $ prompt = "Stanley thinks I'm too heavy for warehouse work and he's not wrong."
                call pcDiet.dietPromptMenu(prompt, goalKg) from nw_vach_ware_office_size8_diet_unsure
                if _return:
                    pc "Right, starting right now I am determined to shift some of this flab!"
                    stanley "Okay, okay. That's good to know [pc.name].
                        I'm sure you'll find things get a bit easier without the extra weight."
                else:
                    pc "I'll see what I can do Stanley."
            else:
                pc "I can't say I'd really noticed.
                    I mean I know I'm pretty fat, but is it really a problem?"
                stanley "It could be. I'm just trying to look out for you [pc.name]."
                $ prompt = "I'm happy with my weight, but Stanley thinks I'm too heavy for warehouse work."
                call pcDiet.dietPromptMenu(prompt, goalKg) from nw_vach_ware_office_size8_diet_happy
                if _return:
                    pc "I suppose you're right. I'll start dieting and try to shift some of this flab!"
                    stanley "Okay, okay. That's good to know [pc.name].
                        I'm sure you'll find things get a bit easier without the extra weight."
                else:
                    pc "I'll see what I can do Stanley."
    # End of stanSize
    return

    # Take home pay.
    #
    # Note: the US pay system isn't like this, but I don't want to be
    # fussing with tax returns/demands and their ilk at the end of the
    # tax year. Let's assume some minimal modicum of progress has happened
    # and PAYE is a thing.
    #
label .workWage:
    $ dbgLog("nwVachonWarehouse.workWage")
    $ renpy.dynamic('cents', 'dollars', 'deductions', 'takeHome', 'wage')
    $ wage = roundPrec(counters.get('vachWage') * 100, 1)
    $ deductions = roundPrec(wage * 0.20, 1)
    $ takeHome = wage - deductions
    $ cents = takeHome % 100
    $ dollars = takeHome // 100
    # "wage=[wage] ded=[deductions] res=[takeHome] dollars=[dollars] cents=[cents]"
    if cents > 0:
        $ tmp = cardinal(dollars) + " dollars and " + cardinal(cents) + " cents in change"
        "
        Stanley hands you a  small heavy envelope.
        You open it up and tip out [tmp].
        "
    else:
        $ tmp = cardinal(dollars)
        "
        Stanley hands you a small envelope.
        You open it up and take out [tmp] dollars.
        "
    $ pc.earn(takeHome / 100.0)
    $ counters.delete('vachWage')
    return