#
# The high path through the woods to the lake.
#
label nwWoodsHigh:
    $ dbgLogDet("nwWoodsHigh")
    $ gsize = pc.gsize
    $ dayCounters.set('climbHour', gt.hour)    # Record ascent time

    #
    # Only once per day.
    #
    $ sofiaHere = sofiaHere and not dayFlags.isSet('sofiaWoods')
    $ steveHere = steveHere and not dayFlags.isSet('steveWoods')

    # De-dup.
    #
    if steveHere and sofiaHere:
        if steve.isBreak():
            $ steveHere = False
        elif sofia.isBreak():
            $ sofiaHere = False
        elif steve.romance > sofia.romance and pc.randomBool(80):
            $ sofiaHere = False
        else:
            $ steveHere = False

    # Testing.
    #
    # $ sofiaHere = True
    # $ sofiaHere = False
    # $ steveHere = False
    # $ steveHere = True

    if sofiaHere:
        call nwWoodsHighSofia.climb from nw_woods_high_sofia_climb
    elif steveHere:
        call nwWoodsHighSteve.climb from nw_woods_high_steve_climb
    else:
        call .climbAlone from nw_woods_high_alone_climb
    return



    # Descend back to the car park.
    #
label .descend:
    $ dbgLogDet("nwWoodsHigh.descend")
    # If no time has elapsed, add an hour.
    #
    if dayCounters.get('climbHour') == gt.hour:
        $ gt.addHours(1)
    $ dayCounters.delete('climbHour')

    if sofiaHere:
        call nwWoodsHighSofia.descend from nw_woods_high_descend_sofia
        return
    elif steveHere:
        call nwWoodsHighSteve.descend from nw_woods_high_descend_steve
        return
    elif gsize >= 10:
        call .descendImpos from nw_woods_high_descend_impos
    elif gsize >= 8:
        call .descendDiff from nw_woods_high_descend_diff
    elif gsize >= 6:
        call .descendHard from nw_woods_high_descend_hard
    elif gsize >= 3:
        call .descendMed from nw_woods_high_descend_med
    else:
        call .descendEasy from nw_woods_high_descend_easy
    $ world.setLoc('nwWoods')
    return



    # Ascend alone.
    #
label .climbAlone:
    $ dbgLogDet("nwWoodsHigh.climbAlone")
    if gsize >= 10:
        call .climbImpos from nw_woods_high_climb_alone_impos
    elif gsize >= 8:
        call .climbDiff from nw_woods_high_climb_alone_diff
    elif gsize >= 6:
        call .climbHard from nw_woods_high_climb_alone_hard
    elif gsize >= 3:
        call .climbMed from nw_woods_high_climb_alone_med
    else:
        call .climbEasy from nw_woods_high_climb_alone_easy
    return



    # Ascend to the lake alone.
    # No weight issues.
    #
label .climbEasy:
    $ dbgLogDet("nwWoodsHigh.climbEasy")
    $ tmp = pc.locoVerb()
    """
    You [tmp] into the woods and find the one of path that
    heads up the hillside. It's a gentle gradient to begin with and as
    you get into the trees the noise from the town fades
    and there's just the snapping of twigs underfoot.

    The path is well worn, although you seem to be the only person here.
    After a while it starts to zig-zag it's way up a steeper part. Then
    there are a set of steps with a handrail and a short bridge over
    a bubbling stream.

    You pause looking down between the trees, slightly surprised about
    how high you are now, before pressing on with the next set of zig zags.

    The path starts to level out, and you must be near the top. Not
    far now.
    """
    $ pc.useEnergy(1)
    $ pc.exercise(500, 0.5)
    call nwWoodsLake.arrive from nw_woods_high_climb_easy
    return



    # Ascend to the lake alone.
    # Some weight issues.
    #
label .climbMed:
    $ dbgLogDet("nwWoodsHigh.climbMed")
    $ tmp = pc.locoVerb()
    """
    You head into the woods and [tmp] to the one path that
    heads up the hillside. It's a gentle gradient to begin with and as
    you get into the trees the noise from the town fades
    and there's just the snapping of twigs underfoot.

    The path is well worn, although you seem to be the only person here.
    After a while it starts to zig-zag it's way up a steeper part. You can
    definitely feel the extra weight you are carrying now, and the way
    your tummy is jiggling with each footfall.

    Next there's a set of steps with a handrail and a short bridge over
    a bubbling stream. You pause to catch your breath and let your eyes
    follow the stream down between the trees. You've climbed quite a way.

    More zig zags next as you continue climbing, but before long the
    hillside is leveling out. Must be nearly there now.
    """
    $ pc.useEnergy(1)
    $ pc.exercise(600, 0.5)
    call nwWoodsLake.arrive  from nw_woods_high_climb_med
    "Those log seats are a welcome sight and you flop down onto one."
    return



    # Ascend to the lake alone.
    # Significant weight issues.
    #
label .climbHard:
    $ dbgLogDet("nwWoodsHigh.climbHard")
    $ tmp = pc.locoVerb()
    """
    You head into the woods and [tmp] to the one path that
    heads up the hillside. It's a gentle gradient to begin with and as
    you get into the trees the noise from the town fades
    and there's just the snapping of twigs underfoot.

    The path is well worn, although you seem to be the only person here.
    After a while it starts to zig-zag it's way up a steeper part. You can
    really feel the extra weight you are carrying now, and your body
    shakes with each footfall.

    You pause at the top of the zig zag and contemplate the flight of
    steps ahead as you catch your breath. Holding firmly onto the handrail
    you climb them and cross the bridge. You pause again to catch your
    breath.

    You let your eyes follow the stream down between the trees. You've carried
    your weight up quite a way. More zig zags next as you continue climbing,
    pausing to rest as needed. Before long the hillside is leveling out
    thank goodness. Must be nearly there now.
    """
    $ pc.useEnergy(1)
    $ pc.exercise(700, 0.5)
    call nwWoodsLake.arrive  from nw_woods_high_climb_hard
    "Those log seats are a welcome sight and you flop down onto one."
    return



    # Ascend to the lake alone.
    # Serious weight issues.
    #
label .climbDiff:
    $ dbgLogDet("nwWoodsHigh.climbDiff")
    $ tmp = pc.locoVerb()
    """
    You head into the woods and [tmp] to the one path that
    heads up the hillside. It's a gentle gradient to begin with and as
    you get into the trees the noise from the town fades
    and there's just the snapping of twigs underfoot.

    The path is well worn, although you seem to be the only person here.
    After a while it starts to zig-zag it's way up a steeper part. You are
    panting before you are half way up and lean against a tree before
    continuing up.

    You pause again at the top of the zig zag and contemplate the flight of
    steps ahead. Holding firmly onto the handrail you climb them and cross the
    bridge. Another pause to catch your breath and wipe your brow.

    You  let your eyes follow the stream down between the trees. You've carried
    your weight up quite a way. More zig zags next as you continue climbing,
    pausing, climbing, pausing. Thankfully the hillside is leveling out,
    nearly there [pc.name]!
    """
    $ pc.useEnergy(1)
    $ pc.exercise(800, 0.5)
    call nwWoodsLake.arrive  from nw_woods_high_climb_diff
    "Those log seats are a welcome sight and you flop down onto one, baring your
    belly to cool off.
    "
    return



    # Ascend to the lake alone.
    # Major weight issues.
    #
label .climbImpos:
    $ dbgLogDet("nwWoodsHigh.climbImpos")
    $ tmp = pc.locoVerb()
    "
    You head into the woods and [tmp] to the one path that
    heads up the hillside. It's a gentle gradient to begin with and as
    you get into the trees the noise from the town fades
    and there's just the snapping of twigs underfoot.
    "
    $ tmp = cardDecades(pc.fat*2.2)
    "
    The path is well worn, although you seem to be alone.
    After a while it zig-zags it's way up a steeper part. You find you
    have to rest at each turn, breath ragged with the effort of carrying
    [tmp] pounds of fat up with you.
    "
    $ prompt = "This is crazy {}, you're not even half way up yet.".format(pc.name)
    $ prompt = addMenuAlt(prompt)
    menu:
        "[prompt]"
        "Turn back you silly moo":
            $ tmp = False
        "No, I can do this can't I?":
            $ tmp = not pc.randomWeightRangeLb(820, 860, 0, 100)
    if not tmp:
        """
        I'm not even half way up, and my heart's going ten to the dozen.
        There's no way you and your fat are going to make it to the lake
        [pc.name]. Don't be silly.

        One mis-step, one stumble and you'll be rolling back down this
        hill all the way to the bottom!

        You start to retrace your steps, fearful now you are looking downhill.
        When the ground starts to level out you feel like rejoicing.
        """
        $ pc.useEnergy(1)
        $ pc.exercise(900, 0.5)
        $ gt.addHours(1)
        return

    """
    Next the steps. Fuck steps. Holding firmly onto the handrail you climb
    them and cross the bridge. Another pause to catch your breath and wipe
    your brow. Not looking down, you instead focus on the next switchback.

    You continue climbing, panting, resting, climbing, panting, resting, climbing.
    So much climbing. At last though the hillside is leveling out. Just a few more
    steps [pc.name]!
    """
    $ pc.useEnergy(2)
    $ pc.exercise(900, 0.5)
    $ gt.addHours(1)
    call nwWoodsLake.arrive  from nw_woods_high_climb_impos
    "
    Those log seats are a welcome sight and you flop down onto one, feeling it
    move under your weight. Stripping your top off is the only way to cool down.
    You don't care if anyone sees all your rolls, it's coming off.
    "
    $ dayFlags.set('rest')
    return



    # Descend from to the lake alone.
    # No weight issues.
    #
label .descendEasy:
    $ dbgLogDet("nwWoodsHigh.descendEasy")
    $ tmp = pc.locoVerb()
    """
    You [tmp] over to the trees and find the start of the path going back
    down. It gradually gets steeper until you begin the first set of zig zags.
    You cross the stream, skip down the steps and descend the second set
    of zig zags.

    The path begins to level out, and you begin to hear the road noise.
    Not far to the car park now.
    """
    $ pc.exercise(400, 0.3)
    return



    # Descend from to the lake alone.
    # Slight weight issues.
    #
label .descendMed:
    $ dbgLogDet("nwWoodsHigh.descendMed")
    $ tmp = pc.locoVerb()
    """
    You [tmp] over to the trees and find the start of the path going back
    down. It gradually gets steeper until you begin the first set of zig zags.
    You cross the stream, go down the steps and descend the second set
    of zig zags.

    The path begins to level out, and you begin to hear the road noise.
    Your legs ache a little, but it's not far to the car park now.
    """
    $ pc.exercise(450, 0.3)
    return



    # Descend from to the lake alone.
    # Significant weight issues.
    #
label .descendHard:
    $ dbgLogDet("nwWoodsHigh.descendHard")
    $ tmp = pc.locoVerb()
    """
    You [tmp] over to the trees and find the start of the path going back
    down. It gradually gets steeper until you begin the first set of zig zags.
    Everything is jiggling with each heavy footfall.

    You cross the stream, hold tight to the handrail to go down the steps and
    carefully descend the second set of zig zags. The path begins to level out,
    and you begin to hear the road noise.
    Your legs ache, but it's not far to the car park now.
    """
    $ pc.exercise(500, 0.3)
    return



    # Descend from to the lake alone.
    # Significant weight issues.
    #
label .descendDiff:
    $ dbgLogDet("nwWoodsHigh.descendDiff")
    $ tmp = pc.locoVerb()
    """
    You [tmp] over to the trees and find the start of the path going back
    down. It gradually gets steeper until you begin the first set of zig zags.
    Everything is jiggling with each heavy footfall.

    You have to take a breather after the first switchback. Even going downhill,
    something you thought you'd be well suited for, is hard work. It would be
    all too easy to stumble and fall with heavy you are now.

    You cross the stream, hold tight to the handrail to go down the steps and
    carefully descend the second set of zig zags after another break. The path
    begins to level out, and you begin to hear the road noise.
    Your legs ache, but it's not far to the car park now.
    """
    $ pc.exercise(550, 0.3)
    return



    # Descend from to the lake alone.
    # Major weight issues.
    #
label .descendImpos:
    $ dbgLogDet("nwWoodsHigh.descendImpos")
    $ tmp = pc.locoVerb()
    "
    You [tmp] over to the trees and find the start of the path going back
    down. It gradually gets steeper until you begin the first set of zig zags.
    Everything is jiggling with each heavy footfall. and you have to take
    regular breaks.
    "
    $ rnd = pc.randomWeightRangeLb(780, 880, 0, 90)
    if rnd:
        "
        Going down one of the switchbacks you stumble and fall. It's so steep
        you can't stop yourself and roll down the rest of the stretch, fetching
        up, thankfully, in some bushes.
        "
        pc "Fuck, that could have ended so badly [pc.name]!"
        call achievements.award('raiders') from nw_woods_high_decsend_raiders_award
        "
        You wait for the panic to subside before trying to heave yourself back
        upright. Not easy at the best of times, but so much harder on uneven
        ground.
        "
        pc "Damn, that hurts. I need to be much more careful."
        $ pc.useEnergy(1)
    else:
        "
        It's hard work going down, something you thought you'd be well suited
        for. But stopping your considerable weight getting ahead of yourself
        isn't easy.
        "
    "
    You cross the stream, go carefully down the steps and cautiously descend
    the second set of zig zags. The path begins to level out, and you begin to
    hear the road noise. Your legs ache a lot, but you're down, and it's not
    far to the car park.
    "
    $ pc.exercise(650, 0.3)
    return
