#
# Main game loop.
#
label main:

label .newDay:
    if debug:
        dbg "main.newDay"

    # Save game version for compatibility work.
    #
    $ version = config.version

    # Start day.
    # Set up variables for the day ahead.
    #
    $ world.newDay()
    $ pc.newDay()

    # Earnings.
    #
    $ socFans.growth()
    $ socFans.getNewIncome()
    $ pornFans.growth()
    $ pornFans.getNewIncome()
    if regJoint != None:
        $ regJoint.newDay()

    # Wake time.
    #
    $ gt.nextDay(world.getWakeHour())

    # Title any potential save game.
    # Eg. Erin day237 412lb v0.13
    #
    $ updateSaveName()

    # If the pc is hard dieting, check to see if they are at the soft limit.
    #
    if pc.isDietHard():
        if not pc.isSoftLim():
            # Soft limit reached, allow normal willpower to take over.
            #
            $ pc.setDietHard(False);
            if debug:
                dbg "main.newDay: Soft limit reached, hard diet disabled"
        else:
            # Some assistance in losing weight while above soft limit.
            #
            $ pc.fat -= 0.4
            $ pc.addGreed(-0.5)
            $ pc.addLim(-0.1)
            $ pc.updateWeight()
    elif pc.isDiet():
        # Dieting: give an assist, gradually reduce greed and accepted fat limit.
        #
        $ pc.fat -= 0.2
        $ pc.addGreed(-0.1)
        $ pc.addLim(-0.05)
        $ pc.updateWeight()

    # Look for significant weight changes after diet applied.
    #
    $ pc.checkPeak()
    call pcSize.check from main_size

    # Opportunity to check state and trigger events at start of day.
    #
    call triggers from main_triggers

    # Start of day debug menu.
    #
    if debugMenus:
        call debugMenu from main_start_day
        $ dayFlags.set('dayDebug')

    # If epilogue has been set, call it before waking.
    # If game over is set as a result, end the game; otherwise continue.
    # Epilogue should call world.setDayOver() to start a new day.
    #
    if world.isEpilogue():
        call expression world.getEpilogueLabel() from main_epilogue
        if world.isGameOver():
            call achievements.award('gameover') from main_game_over_achievement
            return
        else:
            $ world.setEpilogue(None)
            if world.isDayOver():
                jump .newDay

    # Achievements.
    #
    call achievements.scan from main_achievements

    # Wake up.
    #
    if isSelfVoicing():
        # Since the status menu is not voiced, announce the new day.
        #
        $ tmp = "Today is day {}, a {}, and it is {}.".format(gt.dayNum, gt.dayName, clockTime(gt.hour))
        alt "[tmp]"
    call expression world.getWakeLabel() from main_wake

label .loop:
    $ dbgLogDet("main.loop")

    # Return from main on game over.
    #
    if world.isGameOver():
        return

    # Test for end of day conditions.
    #
    if not world.isDayOver():
        if pc.energy == 0:
            # No energy left - exhausted.
            #
            $ renLabel = world.getExhaustedLabel()
            if renLabel == None:
                call pcThoughts.exhausted from main_exhausted
            else:
                call expression renLabel from main_exhausted_if
            $ world.setDayOver()
        elif gt.hour < 8:
            # No time left - too late.
            #
            $ renLabel = world.getTooLateLabel()
            if renLabel == None:
                call pcThoughts.tooLate from main_too_late
            else:
                call expression renLabel from main_too_late_if
            $ world.setDayOver()

    # Work, free choice, or events.
    #
    if not world.isDayOver():
        if world.isTmpWorkTime():
            $ renLabel = world.getTmpWorkLabel()
            $ dbgLogDet("Main: Temporary work [renLabel]")
        elif world.isPartWorkTime():
            $ renLabel = world.getPartWorkLabel()
            $ dbgLogDet("Main: Part time work [renLabel]")
        elif world.isWorkTime():
            $ renLabel = world.getWorkLabel()
            $ dbgLogDet("Main: Work [renLabel]")
        elif world.isWorking():
            # End of work day hook.
            #
            $ dbgLogDet("Main: Ending work")
            call .workEnd from main_work_end
            jump .loop
        else:
            # Not working.
            #
            $ renLabel = None
            $ dbgLogDet("Main: Free time")

        if renLabel != None:
            call util.checkEventOr(renLabel) from main_work
        else:
            call .freeOrEvent from main_free
        jump .loop

    #
    # Falling into...

    # Return home if required.
    #
label .endDayRet:
    # dbg "main.endDayRet"
    if not world.isHome() and not world.isNoReturn():
        while not world.isHome():
            # Try to find .retHomeFromHere label first.
            #
            $ renLabel = world.getRetHomeFromLabel()
            if renLabel != None:
                # If found, call with destination location.
                # May only move towards destination, not reach it in one step.
                #
                $ toLoc = world.getCurHomeLoc()
                call expression renLabel pass (toLoc) from main_ret_to_home
            else:
                # Try to find .retHomeToHere label.
                #
                $ renLabel = world.getRetHomeToLabel()
                if renLabel != None:
                    # If found call this label.
                    #
                    call expression renLabel from main_ret_from_home
                else:
                    # No travel options available: put pc at home.
                    #
                    $ world.locLabel = world.getCurHomeLoc()
                
        # Call returning home label.
        #
        $ renLabel = world.getRetHomeLabel()
        if renLabel != None:
            call expression renLabel from main_ret_home
    #
    # Falling into...

    # Go to sleep
    #
label .endDay:
    # dbg "main.endDay"
    # Sleep..
    #
    if not world.isNoSleep():
        call expression world.getSleepLabel() from main_sleep
    jump .newDay



    # -------------------------------------------------------------------------
    # Callable labels
    # - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -

    # Either have an event of offer free choice.
    #
label .freeOrEvent:
    # dbg "main.freeOrEvent"
    $ renLabel = world.getFreeChoiceLabel()
    call util.checkEventOr(renLabel) from main_free_event
    return



    # Travel to a location from wherever the pc is.
    # This may involve multiple calls to different locations'
    # travelFromHereTo label.
    #
label .travelTo(destLoc):
    $ dbgLog("main.travelTo({})".format(destLoc))
    if world.locLabel == destLoc:
        return
    $ renpy.dynamic('renLabel')
    while world.getLoc() != destLoc:
        $ renLabel = world.getTravelToLabel()
        if renLabel != None:
            call expression renLabel pass (destLoc) from main_travel_to_dyn
        else:
            call expression destLoc from main_travel_to_fallback
            $ world.setLoc(destLoc)
    return



    # Call the end of work hook.
    # Resets world.working.
    #
label .workEnd:
    if debugM:
        dbg "main.workEnd workLastLabel=[world.workLastLabel]"
    $ renpy.dynamic('workEndlabel')
    if world.isWorkLastLabel():
        $ workEndlabel = world.getWorkEndLabel()
        # dbg "workEndLabel=[workEndlabel]"
        if workEndlabel != None:
            call expression workEndlabel from main_world_work_end
        $ world.clearLastWorkLabel()
    $ world.setWorking(False)
    return
