#
# Achievements.
#

default persistent.achievements = dict()

# Achievements are stored in a dictionary that maps a key to a list.
# 0 - visible True/False
# 1 - awarded True/False
# 2 - detail pc name, day, date
#
# Note: only native Python types can be used as attributes of persistent.
#
init python:
    from datetime import date

    # Need a dictionary for achievements.
    #
    if persistent.achievements is None:
        persistent.achievements = dict()

    # How to merge achievements.
    #
    # old       - the newer set
    # new       - the older set
    # current   - the destination
    #
    def mergeAchievements(old, new, current):
        current.update(old)
        current.update(new)
        return current

    # Register achievement merging.
    #
    renpy.register_persistent('achievements', mergeAchievements)

    # Get how many achievements have been awarded.
    #
    def getAchievementCount():
        result = 0
        for award in persistent.achievements.values():
            if award[1]:
                result += 1
        return result

    # Get how many achievements are possible.
    #
    def getAchievementCountTotal():
        return len(achievementsList)

    # Get the player's score.
    #
    def getAchievementScore():
        result = 0
        for achievement in achievementsList:
            achKey = achievement[0]
            if isAchievementAwarded(achKey):
                result += achievement[3]
        return result

    # Get the highest score possible.
    #
    def getAchievementScoreTotal():
        result = 0
        for achievement in achievementsList:
            result += achievement[3]
        return result

    # Get the player's achievements text.
    #
    def getAchievementsText():
        result = ""
        for achievement in achievementsList:
            achKey = achievement[0]
            if isAchievementAwarded(achKey):
                detail = persistent.achievements[achKey][2]
                result += "{{b}}{}:{{/b}} {} {{size=-2}}- {}{{/size}}\n".format(achievement[4], achievement[5], detail)
        if result == "":
            result = "None"
        return result

    # Get the player's visible achievements text.
    #
    def getAchievementsVisText():
        result = ""
        for achievement in achievementsList:
            achKey = achievement[0]
            if not isAchievementAwarded(achKey) and isAchievementVisible(achKey):
                result += "{{b}}{}:{{/b}} {}\n".format(achievement[4], achievement[5])
        if result == "":
            result = "None"
        return result


    # Get the achievement list entry for a key.
    #
    # achKey - the achievement key
    #
    def getAchievementEntry(achKey):
        for achievement in achievementsList:
            if achievement[0] == achKey:
                return achievement
        return None

    # Is an achievement awarded?
    #
    # achKey - the achievement key
    #
    def isAchievementAwarded(achKey):
        if not achKey in persistent.achievements:
            return False
        return persistent.achievements[achKey][1]

    # Is an achievement visible?
    #
    # achKey - the achievement key
    #
    def isAchievementVisible(achKey):
        if not achKey in persistent.achievements:
            return False
        return persistent.achievements[achKey][0]

    # Make an achievement visible if it is not already.
    #
    # achKey - the achievement key
    #
    def setAchievementVisible(achKey):
        if getAchievementEntry(achKey) == None:
            dbg("setAchievementVisible('{}') - unknown achievement".format(achKey))
            return
        if not achKey in persistent.achievements:
            persistent.achievements[achKey] = [True, False, None]
        else:
            persistent.achievements[achKey][0] = True
        renpy.save_persistent()

    # Award an achievment.
    #
    # achKey - the achievement key
    #
    def awardAchievement(achKey):
        if getAchievementEntry(achKey) == None:
            dbg("awardAchievement('{}') - unknown achievement".format(achKey))
            return
        detail = "{} day {} on {}".format(pc.name, gt.dayNum, date.today().isoformat())
        if not achKey in persistent.achievements:
            # Entirely new award.
            persistent.achievements[achKey] = [True, True, detail]
            renpy.save_persistent()
        else:
            award = persistent.achievements[achKey]
            if not award[1]:
                # Award if not already awarded.
                award[0] = True
                award[1] = True
                award[2] = detail
                renpy.save_persistent()

    # Clear all achievements.
    # Re-work achievement visibility.
    #
    def clearAchievements():
        persistent.achievements.clear()
        scanAchievementVisibility()
        renpy.save_persistent()

    # Check achievement list and mark as visible all that should be.
    #
    def scanAchievementVisibility():
        found = False
        for achievement in achievementsList:
            achKey = achievement[0]
            if not isAchievementVisible(achKey):
                # Not visible - should it be?
                achVisCond = achievement[1]
                if achVisCond != None and eval(achVisCond):
                    setAchievementVisible(achKey)
                    found = True
        if found:
            renpy.save_persistent()

    # Check achievement list to see if conditions will evaluate to booleans.
    #
    def scanAchievementsForErrors():
        global debug
        oldDebug = debug
        debug = False
        found = False
        for achievement in achievementsList:
            achKey = achievement[0]
            achVisCond = achievement[1]
            achCond = achievement[2]
            if achVisCond != None:
                try:
                    result = eval(achVisCond)
                except Exception as rte:
                    msg = "Error in achievement '{}' visibility condition \"{}\": {} {}".format(achKey, achVisCond, str(type(rte)).split("'")[1], rte)
                    dbg(msg)
                    found = True
                else:
                    if not isinstance(result, bool):
                        msg = "Error in achievement '{}' visibility condition \"{}\": returned {} not a bool".format(achKey, achVisCond, str(type(result)).split("'")[1])
                        dbg(msg)
                        found = True
            if achCond != None:
                try:
                    result = eval(achCond)
                except Exception as rte:
                    msg = "Error in achivevement '{}' award condition \"{}\": {} {}".format(achKey, achCond, str(type(rte)).split("'")[1], rte)
                    dbg(msg)
                    found = True
                else:
                    if not isinstance(result, bool):
                        msg = "Error in achievement '{}' award condition \"{}\": returned {} not a bool".format(achKey, achCond, str(type(result)).split("'")[1])
                        dbg(msg)
                        found = True
        debug = oldDebug
        return found


    # Find an achievement that is not awarded, but where the award condition is met.
    #
    # Return an achievement key, or None.
    #
    def nextAchievementAward():
        for achievement in achievementsList:
            achKey = achievement[0]
            if not isAchievementAwarded(achKey):
                achAwardCond = achievement[2]
                if achAwardCond != None and eval(achAwardCond):
                    return achKey
        return None



#
# Achievements RenPy labels.
#
label achievements:

    # Award an achievement if it is not already awarded.
    #
    # achKey - the achievement key
    #
label .award(achKey):
    $ dbgLog("achievements.award({})".format(achKey))
    if isAchievementAwarded(achKey):
        return

    $ renpy.dynamic('condTxt', 'name', 'tuple')
    $ listTuple = getAchievementEntry(achKey)
    if listTuple == None:
        dbg "In achievements.award: selected key \"[achKey]\" was not found."
        return
    $ name = listTuple[4]
    $ condTxt = listTuple[5]
    centered "Achievement Awarded: {b}[name]{/b}\n\n[condTxt]"
    $ awardAchievement(achKey)
    return

    # Scan achievements list.
    # Make visible hidden achievements where visible condition is met.
    # Award new achievements where award condition is now met.
    #
label .scan:
    $ dbgLog("achievements.scan")
    $ renpy.dynamic('achKey', 'done', 'debug', 'found')
    $ debug = False
    $ scanAchievementVisibility()
    $ done = False
    $ found = False
    while not done:
        $ achKey = nextAchievementAward()
        #dbg "achKey=[achKey]"
        if achKey == None:
            $ done = True
        else:
            call .award(achKey) from achievements_scan
            $ found = True
    if found:
        $ scanAchievementVisibility()
    #dbg "Scan complete."
    return

    # Reset achievements.
    #
label .reset:
    $ dbgLog("achievements.reset")
    $ clearAchievements()
    $ scanAchievementVisibility()
    return

    

    


            

