#
# Compatibility with old saves.
#
# Label after_load is called automatically when a save game is loaded.
#
#
label after_load:
    $ renpy.dynamic('debug', 'fromv', 'upgraded')
    $ debug = config.developer
    $ upgraded = False
    if not hasattr(renpy.store,'version'):
        # This should never happen as version is defaulted to None in vars.rpy.
        $ dbgLog("after_load: version not defined by defaults !!!")
        $ version = None
    if version == None:
        $ dbgLog("after_load: loaded version < 0.13")
        $ fromv = "v0.10, v0.11, or v0.12"
    else:
        # $ dbgLog("after_load: loaded version [version]")
        $ fromv = "v" + version

    # There were some unfortunate variable name choices in earlier versions.
    #
    call .varHideErrors from after_load_hide_errors

    # Fixes needed as current head version evolves.
    #
    if config.developer:
        call .devModeFixes from after_load_dev_mode_fixes

    # Upgrade steps.
    #
    if version == None:
        call .upgrade013 from after_load_upgrade013
    if version == "0.13":
        call .upgrade014 from after_load_upgrade014
    if version == "0.14":
        call .upgrade015 from after_load_upgrade015

    # Set NPC varnames.
    # Always set to the same values so it doesn't hurt to do again.
    #
    call .npcVarnames from after_load_npc_var_names

    # Notification.
    if upgraded:
        dbg "Save game upgraded from [fromv] to v[version]."
    
    # Achievement checks.
    #
    $ scanAchievementsForErrors()
    $ scanAchievementVisibility()
    
    # $ dbgLog("after_load: done")
    return



    # Upgrade from 0.10..0.12 to 0.13.
    #
label .upgrade013:
    $ dbgLog("after_load.upgrade013")

    # Re-init object storage.
    # It's unknown which are set up correctly as these happened at start time.
    #
    $ gt.setEventList(eventList)
    $ flags.setSet(flagSet)
    $ dayFlags.setSet(daySet)
    $ counters.setMap(countMap)
    $ dayCounters.setMap(dayMap)
    $ foodMenu.setMap(foodMenuMap)
    $ pcClothes.setMap(pcClothesMap)    # Resets clothing.
    $ picker.setList(pickerOpts)

    # Player clothing.
    # Ensure thry have their current sizes.
    #
    $ gsize = pc.gsize
    if not pcClothes.has(gsize, ckBra):
        $ pcClothes.buy(ckBra, gsize)
    if not pcClothes.has(gsize, ckJeans):
        $ pcClothes.buy(ckJeans, gsize)
    if not pcClothes.has(gsize, ckPanties):
        $ pcClothes.buy(ckPanties, gsize)
    if not pcClothes.has(gsize, ckTshirt):
        $ pcClothes.buy(ckTshirt, gsize)
    if not pcClothes.has(gsize, ckWork):
        $ pcClothes.buy(ckWork, gsize)

    # Trip alicia's matchmaking if steve broken up and pc is now too big to
    # trigger first event.
    #
    if steve.breakup and gsize >= 5:
        $ flags.set('aliciaMatch')

    $ version = "0.13"
    $ upgraded = True
    return



    # Upgrade from 0.13 to 0.14.
    #
label .upgrade014:
    $ dbgLog("after_load.upgrade014")

    # Debug menus - default to debug setting.
    #
    $ debugMenus = debug

    # Add lastScaleLb to pc.
    #
    $ condAddAttr(pc, 'lastScaleLb', 90)
    if not "?" in pc.lastWeightLb:
        $ pc.lastScaleLb = int(pc.lastWeightLb.rstrip("lb"))

    # Initialise new class storage.
    #
    $ evtCounters.setMap(evtMap)
    $ cooldowns.setMap(coolMap)
    $ gasDeck.setList(gasDeckList)
    $ gasDeck.setName("gasDeck")

    # Intialise gas parts/project
    #
    $ restockGasParts()
    $ flags.clear('joeNewProject')

    # Move event counters
    #
    python:
        for name in ['barFall', 'barSeconds', 'barSecondsJoe', 'barShames', 'barVisits']:
            evtCounters.set(name, counters.get(name))
            counters.delete(name)

    # Move cooldowns
    #
    if not counters.isZero('dinerWorkDay'):
        $ tmp = 7 + counters.get('dinerWorkDay') - gt.dayNum
        if tmp > 0:
            $ cooldowns.set('dinerWork', tmp)
    $ counters.delete('dinerWorkDay')

    # New characters
    #
    if not hasattr(renpy.store,'antonio'):
        $ antonio = Npc()
    $ antonio.pcMinBfp = pc.getBodyFatPerAt(250/2.2)
    $ antonio.pcMaxBfp = pc.getBodyFatPerAt(660/2.2)
    $ antonio.weight = 148

    if not hasattr(renpy.store,'nicola'):
        $ nicola = Npc()
    $ nicola.pcMaxBfp = pc.getBodyFatPerAt(270/2.2)
    $ nicola.setWeightLb(170)

    if not hasattr(renpy.store,'reg'):
        $ reg = Npc()
    $ reg.pcMinBfp = pc.getBodyFatPerAt(330/2.2)
    $ reg.pcMaxBfp = pc.getBodyFatPerAt(hardLimKg + 20)

    # Fix unused gasIcecream.
    #
    $ tmp = counters.get('gasIcecream')
    if pc.isGreedy(60) and tmp > 12:
        $ tmp = 12
    elif pc.isGreedy(30) and tmp > 6:
        $ tmp = 6
    elif tmp >= 6:
        $ tmp = 5
    if tmp > 0:
        $ counters.set('gasIcecream', tmp)

    # Establish sex scene settings.
    #
    call start.veilSetting from after_load_upgrade014_veils

    $ version = "0.14"
    $ upgraded = True
    return



    # Upgrade from 0.14 to 0.15.
    #
label .upgrade015:
    # PC gsizeMax and limits.
    #
    $ condAddAttr(pc, 'gsizeMax', 11)
    $ condAddAttr(pc, 'massSoftLim', softLimKg)
    $ condAddAttr(pc, 'massHardLim', hardLimKg)

    # New storage.
    #
    $ vachLunchDeck.setList(vachLunchDeckList)
    $ vachLunchDeck.setName("vachLunchDeck")

    # Counters.
    #
    $ condAddAttr(counters, 'cbMap', None)
    $ condAddAttr(dayCounters, 'cbMap', None)
    $ condAddAttr(evtCounters, 'cbMap', None)
    $ condAddAttr(cooldowns, 'cbMap', coolCbMap)

    # Fans
    #
    $ socFans = FanBase()
    $ socFans.setFansMax(1327623)           # More than Bailey Peyton
    $ socFans.setIncomeMax(213/7.0)         # 30/day max
    $ socFans.setVideosMax(100)
    $ pornFans = FanBase()
    $ pornFans.setFansMax(28954)
    $ pornFans.setIncomeMax(226320/365)     # 620/day max
    $ pornFans.setVideosMax(100)

    # Cooking skill
    #
    # $ cookMap = dict()
    # $ cookSkill = CookSkill()
    # $ cookSkill.setMap(cookMap)
    $ cookSkill.addRecipeDict('frybfast', 50)

    # Player.
    #
    $ condAddAttr(pc, 'stuffing', None)
    $ condAddAttr(pc, 'dietImmob', pc.dietHard)
    if hasattr(pc, 'oldKg'):
        $ condAddAttr(pc, 'dietStartKg', pc.oldKg)
    elif pc.isDiet():
        $ condAddAttr(pc, 'dietStartKg', pc.getMass()+1)
    else:
        $ condAddAttr(pc, 'dietStartKg', 0)
    $ condAddAttr(pc, 'glutton', False)
    $ condAddAttr(pc, 'gluttonRdy', False)

    # World.
    #
    $ condAddAttr(world, 'partWorkLabel', None)
    $ condAddAttr(world, 'partWorkStartDay', 0)
    $ condAddAttr(world, 'partWorkLastDay', 0)
    $ condAddAttr(world, 'partWorkDays', None)
    $ condAddAttr(world, 'partWorkStartHr', 0)
    $ condAddAttr(world, 'partWorkEndHr', 0)
    $ condAddAttr(world, 'workOver', False)

    # New LRUs
    #
    $ regBfastLru = []
    $ regLunchLru = []
    $ regDinnerLru = []
    $ regSnackLru = []

    $ version = "0.15"
    $ upgraded = True
    return



    # Init NPC varnames.
    # These were added during v0.13.
    # Also sex characteristic added in v0.15
    #
label .npcVarnames:
    $ alicia.varname = 'alicia'
    $ alicia.genFemale = True
    $ antonio.varname = 'antonio'
    $ antonio.genMale = True
    $ lacy.varname = 'lacy'
    $ lacy.genFemale = True
    $ nicola.varname = 'nicola'
    $ nicola.genFemale = True
    $ reg.varname = 'reg'
    $ reg.genMale = True
    $ sofia.varname = 'sofia'
    $ sofia.genFemale = True
    $ steve.varname = 'steve'
    $ steve.genMale = True
    return



    # Fix variable names hiding python stuff.
    #
label .varHideErrors:
    if hasattr(renpy.store,'len'):
        $ del len
    if hasattr(renpy.store,'list'):
        $ del list
    return



    # Fixes for things as development occurs in current version.
    #
label .devModeFixes:
    dbg "devModeFixes"
    return



init python:

    # If an object doesn't have an attribute add it set to value.
    #
    # obj       - the object
    # attrName  - the attribute name to check
    # value     - the value if the attribute is not known
    #
    def condAddAttr(obj, attrName, value):
        if not hasattr(obj, attrName):
            setattr(obj, attrName, value)
