#
# Food menu tests.
#
label testFoodMenu:
    call .initState from test_food_menu_init
    call .oneDonut from test_food_menu_one_donut
    call .threeDonuts from test_food_menu_three_donuts
    call .minSize from test_food_menu_min_size
    call .nothingHidden from test_food_menu_nothing_hidden
    call .maxSize from test_food_menu_max_size
    call .heavy from test_food_menu_heavy
    return


# Individual test selection.
#
label .testMenu:
    $ renpy.dynamic('done')
    # $ debug = True
    $ done = False
    while not done:
        $ prompt = addMenuAlt("Select test")
        menu:
            "[prompt]"
            "Init":
                $ rpyUnit.reset()
                $ rpyUnit.setShowError(True)
                call .initState from test_food_menu_free_init
                call rpyUnit.report  from test_food_menu_free_init_rpt
            "One donut":
                $ rpyUnit.reset()
                $ rpyUnit.setShowError(True)
                call .oneDonut from test_food_menu_free_one_donut
                call rpyUnit.report from test_food_menu_free_one_donut_rpt
            "Three donuts (addScale)":
                $ rpyUnit.reset()
                $ rpyUnit.setShowError(True)
                call .threeDonuts from test_food_menu_free_three_donuts
                call rpyUnit.report from test_food_menu_free_three_donuts_rpt
            "Min size":
                $ rpyUnit.reset()
                $ rpyUnit.setShowError(True)
                call .minSize from test_food_menu_free_min_size
                call rpyUnit.report from test_food_menu_free_min_size_rpt
            "Nothing hidden":
                $ rpyUnit.reset()
                $ rpyUnit.setShowError(True)
                call .nothingHidden from test_food_menu_free_nothing_hidden_check
                call rpyUnit.report from test_food_menu_free_min_nothing_hidden_rpt
            "Max size":
                $ rpyUnit.reset()
                $ rpyUnit.setShowError(True)
                call .maxSize from test_food_menu_free_max_size
                call rpyUnit.report from test_food_menu_free_max_size_rpt
            "Heavy":
                $ rpyUnit.reset()
                $ rpyUnit.setShowError(True)
                call .heavy from test_food_menu_free_heavy
                call rpyUnit.report from test_food_menu_free_heavy_rpt
            "Back":
                $ done = True
    return



    # -------------------------------------------------------------------------
    # Tests.
    # - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -

    # Test initial state
    #
label .initState:
    $ dbgLog("testFoodMenu.initState")
    $ renpy.dynamic('uut')
    $ rpyUnit.startTest("testFoodMenu", "initState")
    $ uut = FoodMenu(dict())
    $ uut.startClear('Title', 'Subtitle')
    $ assertEquals('Title', uut.getTitle(), "Title should be 'title'")
    $ assertEquals('Subtitle', uut.getSubtitle(), "Subtitle should be 'subtitle'")
    $ assertEquals(0, uut.getLen(), "Map should be empty")
    $ assertEquals(0, uut.getAccPlates(), "Plates should be zero")
    $ assertFalse(uut.has('donut'), "Map should not have 'donut'")
    $ assertEquals(0, uut.getCost('donut'), "Cost of missing item should be zero")
    return



    # Test one donut
    #
label .oneDonut:
    $ dbgLog("testFoodMenu.oneDonut")
    $ renpy.dynamic('cost', 'foodMenuSeln', 'item', 'uut')
    $ rpyUnit.startTest("testFoodMenu", "oneDonut")
    # Test
    $ cost = 0
    $ mealsize = 10
    $ uut = FoodMenu(dict())
    $ uut.startClear('Title', 'Subtitle')
    $ uut.addFood('donut', "A donut")
    $ assertEquals(1, uut.getLen(), "Map should have one entry")
    $ assertEquals(0, uut.getAccPlates(), "Initial plates should be zero")
    $ assertTrue(uut.has('donut'), "Map should contain 'donut'")
    $ item = uut.get('donut')
    # FoodItem
    $ assertTrue(item.available, "Item should be available")
    $ assertEquals(1.5, item.cost, "Cost should be 1.50")
    $ assertFalse(item.header, "Item should not be a header")
    $ assertEquals(0, item.mealSize, "Item meal size should be 0")
    $ assertEquals("", item._postM, "Item should not have a postfix")
    $ assertEquals("A donut", item._promptM, "Item should be 'A donut'")
    $ assertFalse(item.treat, "Item should not be a treat")
    $ assertEquals("A donut $1.50", uut.text('donut'), "Menu text should be 'A donut $1.50'")
    $ assertEquals(1.5, uut.getCost('donut'), "Cost should be 1.50")
    # Purchase
    $ foodMenuSeln='donut'
    $ uut.addMealAcc(foodMenuSeln)
    $ cost += uut.getCost(foodMenuSeln)
    $ assertEquals(0, uut.getAccPlates(), "Plates should still be zero")
    $ assertEquals(1.5, cost, "Cost should now be 1.50")
    return



    # Test three donuts
    #
label .threeDonuts:
    $ dbgLog("testFoodMenu.threeDonuts")
    $ renpy.dynamic('cost', 'foodMenuSeln', 'item', 'uut')
    $ rpyUnit.startTest("testFoodMenu", "threeDonuts")
    # Test
    $ cost = 0
    $ mealsize = 10
    $ uut = FoodMenu(dict())
    $ uut.startClear('Title', 'Subtitle')
    $ uut.addFoodScale(3, 'donut', "Three donuts", cost, 1)
    $ assertEquals(1, uut.getLen())
    $ assertEquals(0, uut.getAccPlates())
    $ assertFalse(uut.has('donut'), "Map should not have 'donut'")
    $ assertTrue(uut.has('donut3'), "Map should have 'donut3'")
    $ item = uut.get('donut3')
    # FoodItem
    $ assertTrue(item.available, "Item should be available")
    $ assertEquals(4.5, item.cost, "Cost should be 4.50")
    $ assertFalse(item.header, "Item should not be a header")
    $ assertEquals(1, item.mealSize, "Item meal size should be 1")
    $ assertEquals("", item._postM, "Item should not have a postfix")
    $ assertEquals("Three donuts", item._promptM, "Item should be 'Three donuts'")
    $ assertFalse(item.treat, "Item should not be a treat")
    $ assertEquals("Three donuts $4.50", uut.text('donut3'), "Menu text should be 'Three donuts $4.50'")
    $ assertEquals(4.5, uut.getCost('donut3'), "Cost should be 4.50")
    # Purchase
    $ foodMenuSeln='donut3'
    $ uut.addMealAcc(foodMenuSeln)
    $ cost += uut.getCost(foodMenuSeln)
    $ assertEquals(1, uut.getAccPlates(), "Plates should now be 1")
    $ assertEquals(4.5, cost, "Cost should now be 4.50")
    return



    # Test min size
    #
label .minSize:
    $ dbgLog("testFoodMenu.minSize")
    $ renpy.dynamic('cost', 'foodMenuSeln', 'item', 'uut')
    $ rpyUnit.startTest("testFoodMenu", "minSize")
    # $ debug=True
    # Test
    $ cost = 0
    $ mealsize = 10
    $ uut = FoodMenu(dict())
    $ uut.startClear('Title', 'Subtitle')
    $ uut.setMinSize(1)
    $ uut.addFood('donut', "A donut")
    $ assertEquals(1, uut.getLen(), "Map should have one entry")
    $ assertEquals(0, uut.getAccPlates(), "Initial plates should be zero")
    $ assertTrue(uut.has('donut'), "Map should contain 'donut'")
    $ item = uut.get('donut')
    # FoodItem
    $ assertFalse(item.available, "Item should not be available")
    $ assertEquals(1.5, item.cost, "Cost should be 1.50")
    $ assertFalse(item.header, "Item should not be a header")
    $ assertEquals(0, item.mealSize, "Item meal size should be 0")
    $ assertEquals("(too small)", item._postM, "Item should have a postfix")
    $ assertEquals("A donut", item._promptM, "Item should be 'A donut'")
    $ assertFalse(item.treat, "Item should not be a treat")
    $ assertEquals("A donut $1.50 (too small)", uut.text('donut'), "Menu text should be 'A donut $1.50 (too small)'")
    $ assertEquals(1.5, uut.getCost('donut'), "Cost should be 1.50")
    # Menu
    $ assertFalse(uut.isAny(), "No items should be selectable")
    $ uut.check()
    $ assertTrue(uut.isAny(), "Item should be selectable")
    # Revised FoodItem
    $ assertTrue(item.available, "Item should be available")
    $ assertEquals("", item._postM, "Item should not have a postfix")
    $ assertEquals("A donut $1.50", uut.text('donut'), "Menu text should be 'A donut $1.50'")
    return



    # Test nothing hidden.
    #
label .nothingHidden:
    $ dbgLog("testFoodMenu.nothingHidden")
    $ renpy.dynamic('cost', 'foodMenuSeln', 'item', 'uut')
    $ rpyUnit.startTest("testFoodMenu", "nothingHidden")
    # $ debug=True
    # Test
    $ cost = 0
    $ mealsize = 10
    $ uut = FoodMenu(dict())
    $ uut.startClear('Title', 'Subtitle')
    $ uut.setMinSize(2)
    $ uut.addNothing(False)
    $ assertEquals(1, uut.getLen(), "Map should have one entry")
    $ assertEquals(0, uut.getAccPlates(), "Initial plates should be zero")
    $ assertTrue(uut.has('nothing'), "Map should contain 'nothing'")
    $ item = uut.get('nothing')
    # Nothing FoodItem
    $ assertFalse(item.available, "Item should not be available")
    $ assertEquals(0, item.cost, "Cost should be 0.00")
    $ assertFalse(item.header, "Item should not be a header")
    $ assertEquals(0, item.mealSize, "Item meal size should be 0")
    $ assertTrue(item.hidden, "Item should be hidden")
    $ assertEquals("", item._postM, "Item should not have a postfix")
    $ assertEquals("Nothing", item._promptM, "Item should be 'Nothing'")
    $ assertFalse(item.treat, "Item should not be a treat")
    $ assertEquals("Nothing", uut.text('nothing'), "Menu text should be 'Nothing'")
    $ assertEquals(0.00, uut.getCost('nothing'), "Cost should be 0.00")
    # Menu
    $ assertFalse(uut.isAny(), "No items should be selectable")
    $ uut.check()
    $ assertTrue(uut.isAny(), "Item should be selectable")
    # Revised Nothing
    $ assertFalse(item.hidden, "Item should not be hidden")
    $ assertTrue(item.available, "Item should be available")
    $ assertEquals("", item._postM, "Item should not have a postfix")
    $ assertEquals("Nothing", uut.text('nothing'), "Menu text should be 'Nothing'")
    return



    # Test max size
    #
label .maxSize:
    $ dbgLog("testFoodMenu.maxSize")
    $ renpy.dynamic('cost', 'foodMenuSeln', 'item', 'uut')
    $ rpyUnit.startTest("testFoodMenu", "maxSize")
    #$ debug=True
    #$ debugM=True
    # Test
    $ cost = 0
    $ mealsize = 10
    $ uut = FoodMenu(dict())
    $ uut.startClear('Title', 'Subtitle')
    $ uut.setMaxSize(0, "too big")
    $ uut.addFood('donut', "A donut")
    $ assertEquals(1, uut.getLen(), "Map should have one entry")
    $ assertEquals(0, uut.getAccPlates(), "Initial plates should be zero")
    $ assertTrue(uut.has('donut'), "Map should contain 'donut'")
    $ item = uut.get('donut')
    # FoodItem
    $ assertFalse(item.available, "Item should not be available")
    $ assertEquals(1.5, item.cost, "Cost should be 1.50")
    $ assertFalse(item.header, "Item should not be a header")
    $ assertEquals(0, item.mealSize, "Item meal size should be 0")
    $ assertEquals("(too big)", item._postM, "Item should have a postfix")
    $ assertEquals("A donut", item._promptM, "Item should be 'A donut'")
    $ assertFalse(item.treat, "Item should not be a treat")
    $ assertEquals("A donut $1.50 (too big)", uut.text('donut'), "Menu text should be 'A donut $1.50 (too big)'")
    $ assertEquals(1.5, uut.getCost('donut'), "Cost should be 1.50")
    return



    # Test too heavy
    #
label .heavy:
    $ dbgLog("testFoodMenu.heavy")
    $ renpy.dynamic('cost', 'foodMenuSeln', 'item', 'uut')
    $ rpyUnit.startTest("testFoodMenu", "heavy")
    #$ debug=True
    #$ debugM=True
    # Test
    $ cost = 0
    $ mealsize = -1
    $ uut = FoodMenu(dict())
    $ uut.startClear('Title', 'Subtitle')
    $ uut.addFood('donut', "A donut")
    $ assertEquals(1, uut.getLen(), "Map should have one entry")
    $ assertEquals(0, uut.getAccPlates(), "Initial plates should be zero")
    $ assertTrue(uut.has('donut'), "Map should contain 'donut'")
    $ item = uut.get('donut')
    # FoodItem
    $ assertFalse(item.available, "Item should not be available")
    $ assertEquals(1.5, item.cost, "Cost should be 1.50")
    $ assertFalse(item.header, "Item should not be a header")
    $ assertEquals(0, item.mealSize, "Item meal size should be 0")
    $ assertEquals("(too heavy)", item._postM, "Item should have a postfix")
    $ assertEquals("A donut", item._promptM, "Item should be 'A donut'")
    $ assertFalse(item.treat, "Item should not be a treat")
    $ assertEquals("A donut $1.50 (too heavy)", uut.text('donut'), "Menu text should be 'A donut $1.50 (too heavy)'")
    $ assertEquals(1.5, uut.getCost('donut'), "Cost should be 1.50")
    return
