#
# Player unit tests
#
label testPlayer:
    call .initState from test_player_init
    call .bmr from test_player_bmr
    call .plateau from test_player_plateau
    return

# Individual test selection.
#
label .testMenu:
    $ renpy.dynamic('done')
    # $ debug = True
    $ done = False
    while not done:
        $ prompt = addMenuAlt("Select test")
        menu:
            "[prompt]"
            "Init":
                $ rpyUnit.reset()
                $ rpyUnit.setShowError(True)
                call .initState from test_player_free_init
                call rpyUnit.report from test_player_free_init_rpt
            "BMR base":
                $ rpyUnit.reset()
                $ rpyUnit.setShowError(True)
                call .bmr from test_player_free_bmr
                call rpyUnit.report from test_player_free_bmr_rpt
            "BMR plateau":
                $ rpyUnit.reset()
                $ rpyUnit.setShowError(True)
                call .plateau from test_player_free_plateau
                call rpyUnit.report from test_player_free_plateau_rpt
            "Back":
                $ done = True
    return



    # -------------------------------------------------------------------------
    # Tests.
    # - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -

    # Test initial state
    #
label .initState:
    $ dbgLog("testPlayer.initState")
    $ renpy.dynamic('oldDayNum', 'uut')
    $ rpyUnit.startTest("testPlayer", "initState")
    $ oldDayNum = gt.dayNum
    $ gt.dayNum = 1
    $ uut = Player()
    # Getters
    $ assertEquals(22, uut.ageBase, "Age should start at 22")
    $ assertWithin(22, 0.005, uut.getAgeFrac(), "Age frac should be near 22")
    $ assertWithin(0, 0.005, uut.getBodyFatPer(), "Body fat should be near 0")
    $ assertEquals(3, uut.getCapacityRemain(), "Capacity remaining should be 3")
    $ assertEquals(0, uut.getDietCount(), "Diet count should be 0")
    $ assertWithin(0, 0.005, uut.getFullness(), "Fullness should be near 0")
    $ assertEquals(3, uut.getFullLimit(), "Full limit remaining should be 3")
    $ assertEquals(158, uut.getHeightCm(), "Height should be 158cm")
    $ assertWithin(0, 0.005, uut.getKink(), "Kink should be near 0")
    $ assertWithin(40, 0.005, uut.getMass(), "Mass should be near 40kg")
    $ assertWithin(52, 0.005, uut.getMassFatLim(), "Mass at fat limit should be near 52kg")
    $ assertWithin(400, 0.005, uut.getMassHardLim(), "Mass at hard limit should be near 400kg")
    $ assertWithin(382, 0.005, uut.getMassSoftLim(), "Mass at soft limit should be near 382kg")
    $ assertEquals(0, uut.getSize(), "Size should be 0")
    $ assertWithin(0, 0.005, uut.getSizeFrac(), "Fractional size should be near 0")
    $ assertWithin(50, 0.005, uut.getMassMaxKg(), "Max mass should be near 50kg")
    $ assertWithin(0, 0.005, uut.getMassMaxProp(), "Max mass should be near 0")
    # Bools
    $ assertFalse(uut.isCheatDay(), "Not a cheat day")
    $ assertFalse(uut.isDiet(), "Not on a diet")
    $ assertFalse(uut.isDietRestricted(), "Not on a restricted diet")
    $ assertTrue(uut.isDietUnrestricted(), "On an unrestricted diet")
    $ assertFalse(uut.isGlutton(), "Not a glutton")
    $ assertFalse(uut.isGluttonRdy(), "Not ready to become a glutton")
    $ assertFalse(uut.isHardLim(), "Not at hard limit")
    $ assertFalse(uut.isSoftLim(), "Not at soft limit")
    $ assertFalse(uut.isWrHips(), "Does not have world record hips")
    $ gt.dayNum = oldDayNum
    return



    # Test bmr calculation.
    #
label .bmr:
    $ dbgLog("testPlayer.bmr")
    $ renpy.dynamic('oldDayNum', 'uut')
    $ rpyUnit.startTest("testPlayer", "bmr")
    $ oldDayNum = gt.dayNum
    $ gt.dayNum = 1
    $ uut = Player()
    $ assertEquals(bmrMassFac, uut.getBmrMassFac(), "Fat bmr mass factor should be same as constant")
    $ assertWithin(0.16, 0.005, uut.getFatBmr(), "BMR fat should be near 0.16kg")
    $ gt.dayNum = oldDayNum
    return



    # Test bmr plateau calculation.
    #
label .plateau:
    $ dbgLog("testPlayer.plateau")
    $ renpy.dynamic('oldDayNum', 'uut')
    $ rpyUnit.startTest("testPlayer", "plateau")
    $ oldDayNum = gt.dayNum
    $ gt.dayNum = 1
    # $ debug = True
    $ uut = Player()
    $ massFac = uut.setMassPlateau(1600/2.2)
    $ assertWithin(43.71, 0.005, massFac, "Fat bmr mass factor should be around 43.71")
    $ assertWithin(bmrMassFac, 0.005, uut.getBmrMassFac(), "BMR mass factor should be near 9.56 initially")
    $ assertWithin(0.16, 0.005, uut.getFatBmr(), "BMR fat should be near 0.16kg initially")
    $ uut.weightKg = 350
    $ assertWithin(bmrMassFac, 0.005, uut.getBmrMassFac(), "BMR mass factor should be near 9.56 at 350kg")
    $ assertWithin(0.54, 0.005, uut.getFatBmr(), "BMR fat should be near 0.16kg at 350kg")
    $ uut.weightKg = 400
    $ assertWithin(26.635, 0.005, uut.getBmrMassFac(), "BMR mass factor should be near 26.635 at 400kg")
    $ assertWithin(1.48, 0.005, uut.getFatBmr(), "BMR fat should be near 1.48kg at 400kg")
    $ uut.weightKg = 450
    $ assertWithin(massFac, 0.005, uut.getBmrMassFac(), "BMR mass factor should be near 43.71 at 450kg")
    $ assertWithin(2.64, 0.005, uut.getFatBmr(), "BMR fat should be near 2.64kg at 450kg")
    $ uut.weightKg = 1600/2.2
    $ assertWithin(massFac, 0.005, uut.getBmrMassFac(), "BMR mass factor should be near 43.71 at 1600lb")
    $ assertWithin(4.20, 0.005, uut.getFatBmr(), "BMR fat should be near 4.20kg at 1600lb")
    # $ debug = False
    $ gt.dayNum = oldDayNum
    return
