#
# Useful functions for backgrounds.
#
init python:

    # Does an image file exist?
    # Looks for .png or .jpg.
    #
    def existsImg(filename):
        result = renpy.loadable(filename + '.png')
        if not result:
            result = renpy.loadable(filename + '.jpg')
        if debugM:
            dbg("existsImg({})={}".format(filename, result))
        return result

    # Get a background image file.
    # If shot is not None/empty search:
    # - loc-shot-gsize
    # - loc-shot
    # - shot
    # Else search:
    # - loc-gsize
    # - loc
    #
    def getBgFile(shot, location=None):
        # dbg("getBgFile({},{})".format(shot, location))
        if location == None:
            location = world.locLabel
        gsize = pc.gsize
        if isEmpty(shot):
            candidate = "{}-{}".format(location, gsize)
            # dbg("trying {}".format(candidate))
            if not existsImg(candidate):
                candidate = "{}".format(location)
        else:
            candidate = "{}-{}-{}".format(location, shot, gsize)
            # dbg("trying {}".format(candidate))
            if not existsImg(candidate):
                candidate = "{}-{}".format(location, shot)
                # dbg("trying {}".format(candidate))
                if not existsImg(candidate):
                    candidate = "{}".format(shot)
        # dbg("trying {}".format(candidate))
        if not existsImg(candidate):
            candidate = None
            if debug:
                dbg("getBgFile({},{}) gsize={} could not find any file".format(shot, location, gsize))
        elif debug:
            dbg("getBgFile({},{})={} gsize={}".format(shot, location, candidate, gsize))
        return candidate

    # Show a new scene with a background image.
    # See getBgFile.
    # Sets global lastBg.
    #
    def showBg(shot, location=None):
        global lastBg
        bgFile = getBgFile(shot, location)
        if bgFile == None:
            bgFile = "blank"
        # dbg("Using '{}'".format(bgFile))
        renpy.scene()
        renpy.show(bgFile)
        lastBg = bgFile

    # Restore a new scene with a background image.
    #
    def restoreBg(bgFile):
        global lastBg
        renpy.scene()
        renpy.show(bgFile)
        lastBg = bgFile
