#
# Useful functions for npcHere flags.
#
init python:

    # Clear all the npcHere flags.
    #
    def clearHere():
        global aliciaHere, nicolaHere, regHere, sofiaHere, steveHere
        aliciaHere = False
        nicolaHere = False
        regHere = False
        sofiaHere = False
        steveHere = False
        if debugM:
            dbg("clearHere: alicia={} nicola={} reg={} sofia={} steve={}".format(aliciaHere, nicolaHere, regHere, sofiaHere, steveHere))



    # Set one of the npcHere flags.
    # npc   - the npc object
    #
    def setHere(npc):
        global aliciaHere, nicolaHere, regHere, sofiaHere, steveHere
        if npc == alicia:
            aliciaHere = True
        if npc == nicola:
            nicolaHere = True
        if npc == reg:
            regHere = True
        elif npc == sofia:
            sofiaHere = True
        elif npc == steve:
            steveHere = True
        if debugM:
            dbg("setHere: alicia={} nicola={} reg={} sofia={} steve={}".format(aliciaHere, nicolaHere, regHere, sofiaHere, steveHere))



    # Pick ONE random npc to be here, each equally likely.
    # Sets/clears npcHere flags.
    #
    # noone - chance of no one here 0..100
    # npcs  - list of npc who could be here
    #
    def whoIsHere(noone, *npcs):
        clearHere()
        if noone < 0:
            noone = 0
        if noone < 100:
            rnd = renpy.random.randint(0, 100)
            count = len(npcs)
            rndDec = (100.0 - noone) / count
            # dbg("whoIsHere: rnd={} count={} rndDec={}".format(rnd, count, rndDec))
            for npc in npcs:
                if rnd <= rndDec:
                    setHere(npc)
                    break
                rnd -= rndDec
        if debug:
            dbg("whoIsHere({}, {} npcs): alicia={} sofia={} steve={}".format(noone, count, aliciaHere, sofiaHere, steveHere))



    # Pick zero or more random npcs to be here.
    # Sets/clears npcHere flags.
    #
    # noone - required chance of no one here 0..100
    # min   - chance of npc being here at minimum romance
    # max   - chance of npc being here at max romance
    # only  - True to only include romantic partners
    # npcs  - list of npc who could be here
    #
    def whoIsHereAny(noone, min, max, only, *npcs):
        clearHere()
        if noone < 0:
            noone = 0
        if noone < 100:
            pTgt = 1 - noone / 100.0
            count = len(npcs)
            pNpc = []
            pAdj = []

            # Calculate probability of each npc being present taking into
            # account the only flag.
            #
            for idx in range(0, count):
                npc = npcs[idx]
                if only and (npc.breakup or npc.romance == 0):
                    chance = 0
                elif npc.breakup:
                    chance = min / 100.0
                else:
                    chance = (min + (max - min) * npc.romance / 100.0) / 100.0
                pNpc.append(chance)
                pAdj.append(chance)
                #dbg("npc={} chance={}".format(idx, chance))
            # dbg("whoIsHereAny: count={} pTgt={}".format(count, pTgt))

            # Compute chance of being alone.
            #
            pNone = 1.0
            for idx in range(0, count):
                p = pNpc[idx]
                pNone *= 1 - p
                # dbg("idx={} p={} pNone={}".format(idx, p, pNone))
            pAny = 1 - pNone
            if debugM:
                dbg("whoIsHereAny: pNone={} pAny={} pTgt={}".format(pNone, pAny, pTgt))

            # Scale back if needed.
            #
            if pNone < noone / 100.0 and pAny > 0:
                # Guess scale and perform up to five steps of successive
                # approximation until within 0.5% of being alone.
                #
                scale = pTgt / pAny
                delta = pAny - pTgt
                step = 0
                while (step < 5 and abs(delta) > 0.005):
                    if debugM:
                        dbg("whoIsHereAny: step={} delta={} scale={}".format(step, delta, scale))
                    pNone = 1.0
                    for idx in range(0, count):
                        pAdj[idx] = pNpc[idx] * scale
                        pNone *= 1 - pAdj[idx]
                    pAny = 1 - pNone
                    delta = pAny - pTgt
                    scale *= pTgt / pAny
                    step += 1
                if debugM:
                    dbg("whoIsHereAny: done step={} pNone={} delta={} scale={}".format(step, pNone, delta, scale))

            # Who is here?
            #
            for idx in range(0, count):
                p = pAdj[idx]
                rnd = renpy.random.random()
                here = (p > 0) and (p >= rnd)
                if debugM:
                    dbg("{} p={:0.3f} rnd={:0.3f} here={}".format(idx, p, rnd, here))
                if here:
                    setHere(npcs[idx])

        if debug:
            dbg("whoIsHereAny({}, {}, {}, {}, {} npcs): alicia={} sofia={} steve={}".format(noone, min, max, only, count, aliciaHere, sofiaHere, steveHere))



    # Who in a list of NPCs is the PC's highest romance score with.
    # Sets/clears npcHere flags.
    # Score must be above minimum.
    # Ignores NPCs on break.
    #
    # min   - minimum romance score
    # npcs  - list of NPCs
    # Returns selected Npc
    #
    def whoIsHereMax(min, *npcs):
        result = None
        curRom = min
        for npc in npcs:
            if not npc.breakup and npc.romance > curRom:
                result = npc
                curRom = npc.romance
        clearHere()
        if result != None:
            setHere(result)
        if debug:
            dbg("whoIsHereMax({}, npcs)={}".format(min, result.varname))
        return result



    # List vehicles here.
    # Returns a string.
    #
    def listVehicleHere():
        who = []
        if aliciaHere:
            who.append("Alicia's car")
        if nicolaHere:
            who.append("Nicola's car")
        if regHere:
            who.append("Reg's van")
        if sofiaHere:
            who.append("Sofia's vespa")
        if steveHere:
            who.append("Steve's car")

        count = len(who)
        if count == 0:
            return ""
        prompt = " You think you see {} here.".format(commaList(who))
        return prompt
